/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.bpel.deploy.metadb.processor.impl;

import org.eclipse.core.resources.IFile;
import org.eclipse.stp.im.BasicProperty;
import org.eclipse.stp.im.ImFactory;
import org.eclipse.stp.im.Process;
import org.eclipse.stp.im.Property;
import org.eclipse.stp.im.StpIntermediateModel;
import org.spagic3.utils.ImPreprocessor;
import org.spagic3.utils.SpagicJBPMPreprocessor;


/**
 * JBI ServiceMix preprocessor
 *
 */
public class JBPMProcessor extends BaseProcessor {
	
	private ImPreprocessor imPreprocessor = null;
	protected ImFactory factory = null;
	private static String MARK_AS_PREPROCESSED = "MARK_AS_PREPROCESSED";
	
	public JBPMProcessor(String name, String binding, String technology) {
		super(name, binding, technology);
		this.imPreprocessor = new SpagicJBPMPreprocessor(binding);
		this.factory = ImFactory.eINSTANCE;
	}
	

	/**
	 * @see org.spagic3.bpel.deploy.metadb.processor.IProcessor#preProcess(org.eclipse.stp.im.Process, StpIntermediateModel)
	 */
	public org.eclipse.stp.im.Process preProcess(IFile imFile, org.eclipse.stp.im.Process process, StpIntermediateModel stpIntermediateModel) {
		String processToEnhnaceName = process.getName();
		
		if (!isAlreadyPreprocessed(stpIntermediateModel)){
			imPreprocessor.process(imFile, stpIntermediateModel);
			markAsPreprocessed(stpIntermediateModel);
		}

		for (Process aProcess : stpIntermediateModel.getProcessCollection().getProcesses()) {
			if (aProcess.getName().equalsIgnoreCase(processToEnhnaceName))
				return aProcess;
		}
		
		return null;

		
	}
	private boolean isAlreadyPreprocessed(StpIntermediateModel stpIntermediateModel){
		Property p = stpIntermediateModel.getProperties().get(MARK_AS_PREPROCESSED);
		return p != null;
	}
	
	private void markAsPreprocessed(StpIntermediateModel stpIntermediateModel){
		Property p = factory.createBasicProperty();
		p.setKey("MARK_AS_PREPROCESSED");
		((BasicProperty)p).setValue("TRUE");
		stpIntermediateModel.getProperties().put(MARK_AS_PREPROCESSED, p);
	}

	public boolean handleAutomaticVersioning() {
		
		return true;
	}
}
