/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.bpel.deploy.ui.actions;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.xml.namespace.QName;

import org.eclipse.core.resources.IFile;
import org.eclipse.emf.common.util.EMap;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.stp.im.ExtractDataRule;
import org.eclipse.stp.im.ImPackage;
import org.eclipse.stp.im.IterationControl;
import org.eclipse.stp.im.JoinControl;
import org.eclipse.stp.im.MapProperty;
import org.eclipse.stp.im.ObservableAttribute;
import org.eclipse.stp.im.ProcessCollection;
import org.eclipse.stp.im.Property;
import org.eclipse.stp.im.RouterControl;
import org.eclipse.stp.im.Service;
import org.eclipse.stp.im.ServiceBinding;
import org.eclipse.stp.im.SplitControl;
import org.eclipse.stp.im.Step;
import org.eclipse.stp.im.StpIntermediateModel;
import org.eclipse.stp.im.Transition;
import org.eclipse.stp.im.constants.ImConstants;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.cfg.Configuration;
import org.spagic.metadb.base.ProcessInstanceStateConstants;
import org.spagic.metadb.dbutils.AuditDBUtils;
import org.spagic.metadb.model.ProcessDiagram;
import org.spagic.metadb.model.PropertyProcess;
import org.spagic.metadb.model.PropertyProcessId;
import org.spagic.metadb.model.PropertyStep;
import org.spagic.metadb.model.PropertyStepId;
import org.spagic.metadb.model.Rule;
import org.spagic.metadb.model.TransitionId;
import org.spagic3.bpel.deploy.metadb.processor.IProcessor;
import org.spagic3.bpel.deploy.metadb.processor.RuntimeProcessor;
import org.spagic3.bpel.deploy.ui.dialogs.ChooseRuntimeDialog;
import org.spagic3.utils.ImPreprocessor;
import org.spagic3.utils.SpagicJBIImPreprocessor;
import org.spagic3.utils.SpagicLogger;
import org.spagic3.utils.SpagicUtils;
import org.spagic3.utils.SpagicUtilsActivator;
import org.spagic3.utils.graph.GraphGenerator;
import org.spagic3.utils.preference.SpagicPreferenceConstants;

public class IMPreprocessAction implements IObjectActionDelegate {

	public static final Integer USAGE_EXTRACT_DATA = 0;
	public static final Integer USAGE_ITER_TYPE_DETERMINATION = 1;
	
	public static final String RULE_DESCRIPTION = "RuleForIterTypeDetermination:";
	public static final String ITER_DESCRIPTION = "No description available";
	
	
	private IFile imFile = null;
	
	public IFile getImFile() {
		return imFile;
	}

	public void setImFile(IFile imFile) {
		this.imFile = imFile;
	}

	/**
	 * Constructor for Action1.
	 */
	public IMPreprocessAction() {
		super();
	}

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
	}

	
	
	
	/**
	 * @see IActionDelegate#run(IAction)
	 */
	public void run(IAction action) {
		Shell shell = new Shell();
		Session aSession = null;
		Transaction tx = null;
		
		try{
			ResourceSet resourceSetIM = new ResourceSetImpl();
			URI uri = URI.createPlatformResourceURI(imFile.getFullPath().toString(),true);


			@SuppressWarnings("unused")
			ImPackage imPackage = ImPackage.eINSTANCE;

			// load on demand = true
			Resource resourceIM = resourceSetIM.getResource(uri, true);	
									
			// Get The intermediate model
			StpIntermediateModel imModel= (StpIntermediateModel)resourceIM.getContents().get(0);
			ImPreprocessor preprocess = new SpagicJBIImPreprocessor("runtime.jbi.smx");
			preprocess.process(imFile, imModel);
			
//			System.out.println("-> After Modification [===================================================]");
			for (org.eclipse.stp.im.Process process : imModel.getProcessCollection().getProcesses()){
//				System.out.println("-> Process ["+process.getName()+"]");
				for( Step step : process.getSteps()){
//					System.out.println("---> Step ["+step.getName()+"]");
				}
				for( Transition transition : process.getTransitions()){
					
					if ((transition.getSource() == null) && (transition.getTarget() != null)){
//						System.out.println("#### Transition [NULL] -> ["+transition.getTarget().getName()+"]");
					}
					if ((transition.getSource() != null) && (transition.getTarget() == null)){
//						System.out.println("#### Transition ["+transition.getSource().getName()+"] -> [NULL]");
					}
					
					if ( (transition.getSource() != null) && (transition.getTarget() != null)){
//						System.out.println("#### Transition ["+transition.getSource().getName()+"] -> ["+transition.getTarget().getName()+"]");
					}
					
				}
			}
			resourceIM.save(Collections.EMPTY_MAP);
//			System.out.println("-> After Save OK ");
			
		} catch (Throwable iaex) {
			SpagicLogger.error("IntermediateModel Preprocess", iaex);
			MessageDialog.openError(
					shell,
					"IntermediateModel Preprocess",
					iaex.getMessage());
		} finally {
			
		}
		
		
	}
	
	
	
	
	
	
	
	
	
	public final void selectionChanged(final IAction action,
			final ISelection selection) {
		
		setImFile((IFile) ((StructuredSelection) selection)
				.getFirstElement());
	}
	
	
}
