/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.bpel.deploy.ui.actions;

import java.io.BufferedInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import javax.xml.namespace.QName;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.apache.axiom.om.OMNamespace;
import org.apache.axiom.om.OMText;
import org.apache.axiom.om.util.Base64;
import org.apache.axis2.AxisFault;
import org.apache.ode.axis2.service.ServiceClientUtil;
import org.apache.ode.utils.Namespaces;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;
import org.dom4j.io.XMLWriter;
import org.eclipse.bpel.model.Process;
import org.eclipse.bpel.model.resource.BPELResource;
import org.eclipse.bpel.model.resource.BPELResourceFactoryImpl;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.common.util.URI;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.spagic3.utils.SpagicLogger;
import org.spagic3.utils.SpagicUtilsActivator;
import org.spagic3.utils.preference.SpagicPreferenceConstants;

public class ODEDeployWorkspaceOperation extends WorkspaceModifyOperation{

	private IFile bpelFile = null;
	private Process processToDeploy = null;
	private final String TEMPORARY_FILE_SUFFIX = "Temp";
	
	public ODEDeployWorkspaceOperation(IFile bpelFile, Process processToDeploy){
		this.bpelFile = bpelFile;
		this.processToDeploy = processToDeploy;
	}
	
	@Override
	protected void execute(IProgressMonitor monitor) {
		//save the bpel model modified with a unique identifier name for the activities into a temporary file
		saveBPELToTemporaryFile();
		
		//create the .zip to deploy
		String bpelFilePath = bpelFile.getLocation().toOSString();
    	String deployZipPath = bpelFilePath.substring(0, bpelFilePath.indexOf(".bpel")) + ".zip";
    	
    	if (deploymentDescriptorExist()) {
    		validateDeploymentDescriptor();
	    	createDeploymentZip(deployZipPath);
	    	
			//call the webservice to ODE deploy
			 // Create a factory
	        OMFactory _factory = OMAbstractFactory.getOMFactory();
	        ServiceClientUtil client = new ServiceClientUtil();
	        OMNamespace depns = _factory.createOMNamespace(Namespaces.ODE_DEPLOYAPI_NS, "deployapi");
	        OMElement root = _factory.createOMElement("deploy", depns);
	        OMElement namePart = _factory.createOMElement("name", null);
	        namePart.setText(processToDeploy.getName());
	        OMElement zipPart = _factory.createOMElement("package", null);
	        OMElement zipElmt = _factory.createOMElement("zip", depns);
	
	        // Add the zip to deploy directory
	        File deployZipFile = new File(deployZipPath);
	        try {
	    		BufferedInputStream is = new BufferedInputStream(new FileInputStream(deployZipFile));
		        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
		        for (int b = is.read(); b >= 0; b = is.read()) {
		            outputStream.write((byte) b);
		        }
		        String base64Enc = Base64.encode(outputStream.toByteArray());
		        OMText zipContent = _factory.createOMText(base64Enc, "application/zip", true);
		        root.addChild(namePart);
		        root.addChild(zipPart);
		        zipPart.addChild(zipElmt);
		        zipElmt.addChild(zipContent);
		        client.send(root, getDeploymentServiceUrl());
	        } catch(AxisFault ex) {
	        	String msgError = "Problems during deploy." + ex.getMessage();
				SpagicLogger.error(msgError);
				throw new RuntimeException(msgError);
	        } catch(IOException ioe) {
	        	String msgError = "Problems during deploy." + ioe.getMessage();
				SpagicLogger.error(msgError);
				throw new RuntimeException(msgError);
	        }
			
		} else {
			SpagicLogger.error("Deployment Descriptor does not exist");
			throw new RuntimeException("Deployment Descriptor does not exist. Create a ODE Deployment Descriptor and try again.");
		}
	}
	
	public String getDeploymentServiceUrl() {
		org.eclipse.core.runtime.Preferences prefs = SpagicUtilsActivator.getDefault().getPluginPreferences();
		return prefs.getString(SpagicPreferenceConstants.P_SPAGIC_ODE_URL_DEPLOY);
	}	
	
	public String getProcessManagementUrl() {
		org.eclipse.core.runtime.Preferences prefs = SpagicUtilsActivator.getDefault().getPluginPreferences();
		return prefs.getString(SpagicPreferenceConstants.P_SPAGIC_ODE_URL_PROCESS_MANAGEMENT);
	}	
	
	public String getDeployedVersion() {
		ServiceClientUtil client = new ServiceClientUtil();
		try{
			String bpelFileName = bpelFile.getName();
			String filterString = "name=" + bpelFileName.substring(0, bpelFileName.indexOf(".bpel"));
			OMElement root = client.buildMessage("listProcesses", new String[] {"filter", "orderKeys"}, new String[] {filterString, ""});
			OMElement listProcesses = client.send(root, getProcessManagementUrl());
			
			Iterator it = listProcesses.getFirstElement().getChildElements();
			while (it.hasNext()) {
				OMElement process = (OMElement)it.next();
				OMElement status = process.getFirstChildWithName(new QName(Namespaces.ODE_PMAPI_TYPES_NS, "status"));
				if (status.getText().equals("ACTIVE")){
					OMElement version = process.getFirstChildWithName(new QName(Namespaces.ODE_PMAPI_TYPES_NS, "version"));
					return version.getText();
				}
			}
		} catch(AxisFault af){
			String msgError = "The ODE server could be stopped." + af.getMessage();
			SpagicLogger.error(msgError);
			throw new RuntimeException(msgError);
		}
		throw new RuntimeException("Problem to retrieve deploy info from the ode server");
	}
	
	protected List<IFile> getArtifacts(IContainer project) {
		IResource[] resources = null;
		List<IFile> files = new ArrayList<IFile>();
		
		try {
			resources = project.members();
		}catch(CoreException ce){
			throw new RuntimeException("Problem to retrieve file in the ws:" + ce);
		}
		for (int i = 0; i < resources.length; i++) {
			if (resources[i].getType() == IResource.FILE)
				files.add((IFile)(resources[i]));
		}
		return files;
	}
	
	protected boolean deploymentDescriptorExist(){
		List<IFile> files = getArtifacts(bpelFile.getParent());
		for (IFile file : files) {
			if ( file.getName().equalsIgnoreCase("deploy.xml")) { 		
				return true;
			}
		}
		return false;
	}
	
	private void validateDeploymentDescriptor(){
		String deploymentDescriptorFilePath = getBPLELDirectory().getLocation() + System.getProperty("file.separator") + "deploy.xml";
		SAXReader reader = new SAXReader();
		Document deployDocument = null;
		try{
			deployDocument = reader.read(new File(deploymentDescriptorFilePath));
			Element processElement = deployDocument.getRootElement();
			List<Element> processesList = processElement.elements();
			for (Element element : processesList){
				String processName = element.attributeValue("name");
				if (!processName.endsWith(processToDeploy.getName()))
					element.detach();
			}
			XMLWriter writer = new XMLWriter( new FileWriter( deploymentDescriptorFilePath + TEMPORARY_FILE_SUFFIX) );
		    writer.write(deployDocument);
		    writer.close();
		    String bpelDirectoryName = bpelFile.getParent().getName();
		    IFile file = bpelFile.getProject().getFile(bpelDirectoryName + System.getProperty("file.separator") + "deploy.xml" + TEMPORARY_FILE_SUFFIX);
		    try{
		    file.refreshLocal(IResource.DEPTH_ZERO, null);
		    }catch (CoreException ce){
		    	throw new RuntimeException("unable to modify deploy.xml. ", ce);
		    }
		} catch(DocumentException de){
			throw new RuntimeException("unable to modify deploy.xml. ", de);
		} catch (IOException ioe){
			throw new RuntimeException("unable to modify deploy.xml. ", ioe);
		}
	}
	
	protected boolean isFileOfInterest(IFile file) {
		String fileExtension = file.getFileExtension();
		if (fileExtension.equalsIgnoreCase("bpel" + TEMPORARY_FILE_SUFFIX))
			return true; 
		if (fileExtension.equalsIgnoreCase("wsdl"))
			return true; 
		if (fileExtension.equalsIgnoreCase("xsd"))
			return true; 
		if (fileExtension.equalsIgnoreCase("soap"))
			return true;
		if (fileExtension.equalsIgnoreCase("xml" + TEMPORARY_FILE_SUFFIX))
			return true; 
		else
			return false;
	}
	
	protected boolean isBPELFile(IFile file) {
		String fileExtension = file.getFileExtension();
		if (fileExtension.equalsIgnoreCase("bpel"))
			return true;
		else 
			return false;
	}

	private void createDeploymentZip(String deployZipPath) {
		final int BUFFER = 2048;
		//remove the old .zip file
		if (new File(deployZipPath).exists())
			new File(deployZipPath).delete();
			
		try {
			ZipOutputStream out = new ZipOutputStream(new FileOutputStream(deployZipPath));
			
			List<IFile> files = getArtifacts(bpelFile.getParent());
			byte[] data = new byte[BUFFER];
			for (IFile file : files) {
				if (isFileOfInterest(file)) {
					FileInputStream fi = new FileInputStream(file.getLocation().toString());
					BufferedInputStream buffi = new BufferedInputStream(fi, BUFFER);
					//add a zip entry
					String fileName = file.getName().endsWith(TEMPORARY_FILE_SUFFIX)
										? file.getName().substring(0, file.getName().lastIndexOf(TEMPORARY_FILE_SUFFIX))
										: file.getName();
					ZipEntry entry = new ZipEntry(fileName);
					//add to the zip
					try {
						out.putNextEntry(entry);
					} catch (IOException e) {
						throw new RuntimeException("unable to put an entry in the zip file : " +  entry.getName(), e);
					} 
					
					//writing to the writer stream
					int count;
					try {
						while ((count = buffi.read(data, 0, BUFFER)) != -1) {
							out.write(data, 0, count);
						}
						//close the current entry
						out.closeEntry();
						buffi.close();
					} catch (IOException e) {
						throw new RuntimeException(
								"unable to writing : "+ file.getName());
					} 
				}	
			}
			out.close();
			
			deleteTemporaryBPELFile();
			deleteTemporaryDeploymentDescriptorFile();
			try {
				getBPLELDirectory().refreshLocal(IResource.DEPTH_ONE, null);
			}catch(CoreException ce){
				throw new RuntimeException("unable to refresh workspace: ", ce);
			}
		} catch (FileNotFoundException fe) {
			throw new RuntimeException("unable to find file: ", fe);
		} catch (IOException ioe){
			throw new RuntimeException("unable to create zip file: ", ioe);
		}
	}	

	private void saveBPELToTemporaryFile(){
		String bpelTempFilePath = bpelFile.getFullPath().toOSString() + TEMPORARY_FILE_SUFFIX;
		URI tempURI = URI.createPlatformResourceURI(bpelTempFilePath,true);
		BPELResource resourceBpel = (BPELResource)(new BPELResourceFactoryImpl()).createResource(tempURI);
		resourceBpel.getContents().add(processToDeploy);
		try {
			resourceBpel.save(Collections.EMPTY_MAP);
		} catch (IOException ioe) {
			throw new RuntimeException("unable to save bpelmodel in to temporany file. ", ioe);
		}
	}	
	
	private void deleteTemporaryBPELFile(){
		String bpelTempFilePath = bpelFile.getLocation().toOSString() + TEMPORARY_FILE_SUFFIX;
		if ((new File(bpelTempFilePath)).exists())
			(new File(bpelTempFilePath)).delete();
	}	
	
	private void deleteTemporaryDeploymentDescriptorFile(){
		String  temporaryDeploymentDescriptorFilePath = getBPLELDirectory().getLocation() + System.getProperty("file.separator") + "deploy.xml" + TEMPORARY_FILE_SUFFIX;
		if ((new File(temporaryDeploymentDescriptorFilePath)).exists())
			(new File(temporaryDeploymentDescriptorFilePath)).delete();
	}	
	
	private IContainer getBPLELDirectory(){
		return bpelFile.getParent();
	}
	
}
