/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.bpel.deploy.ui.actions;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.spagic3.bpel.deploy.BPELDeployActivator;
import org.spagic3.utils.SpagicLogger;

public class Unpublish2MetaDBAction implements IObjectActionDelegate {

	private IFile imFile = null;
	
	public IFile getImFile() {
		return imFile;
	}

	public void setImFile(IFile imFile) {
		this.imFile = imFile;
	}

	/**
	 * Constructor for Action1.
	 */
	public Unpublish2MetaDBAction() {
		super();
	}

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
	}

	/**
	 * @see IActionDelegate#run(IAction)
	 */
	public void run(IAction action) {
		Shell shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
		try {
			
			UnpublishIM2MetaDBWorkspace unpublishIM2MetaDBOperation = new UnpublishIM2MetaDBWorkspace(imFile);
			PlatformUI.getWorkbench().getProgressService().run(true, false, unpublishIM2MetaDBOperation);
			
			MessageDialog.openInformation(
					shell, BPELDeployActivator.PLUGIN_ID,
					"Unpublishing completed.");
		} catch (InvocationTargetException x) {
			reportError(shell, x.getTargetException());
		} catch (InterruptedException x) {
			reportError(shell, x.getCause());
		} catch (Throwable e) {
			reportError(shell, e);
		}
	}
	
	public final void selectionChanged(final IAction action,
			final ISelection selection) {
		
		setImFile((IFile) ((StructuredSelection) selection)
				.getFirstElement());
	}
	
	private void reportError(Shell shell, Throwable x) {
		SpagicLogger.error(x.getMessage(), x);
		ErrorDialog.openError(shell, "Error", "Error Occurred", makeStatus(x));
	}

	public static IStatus makeStatus(Throwable x) {
		return new Status(IStatus.ERROR, BPELDeployActivator.PLUGIN_ID, IStatus.ERROR, 
				(x.getMessage() != null) ? x.getMessage()
				: (x.getCause() != null) ? x.getCause().getMessage() : "", null);
	}
	
}
