/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.bpel.deploy.ui.actions;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.spagic3.bpel.deploy.BPELDeployActivator;

public class UnpublishBpel2ImAction implements IObjectActionDelegate {
	private Shell shell = null;
	private IFile bpelFile = null;

	/**
	 * Constructor for Action1.
	 */
	public UnpublishBpel2ImAction() {
		super();
	}

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
	}

	/**
	 * @see IActionDelegate#run(IAction)
	 */
	public void run(IAction action) {
		shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
		try {
			//undeploy on ODE server
			WorkspaceModifyOperation odeDeployOperation = new ODEUndeployWorkspaceOperation(bpelFile);
			PlatformUI.getWorkbench().getProgressService().run(true, false, odeDeployOperation);
			
			//unpublish metaDB
			IFile imFile = getImFileFromBPELFile();
			UnpublishIM2MetaDBWorkspace unpublishIM2MetaDBOperation = new UnpublishIM2MetaDBWorkspace(imFile);
			PlatformUI.getWorkbench().getProgressService().run(true, false, unpublishIM2MetaDBOperation);
			
			MessageDialog.openInformation(
					shell, BPELDeployActivator.PLUGIN_ID,
					"Unpublishing completed.");
			
		} catch (InvocationTargetException x) {
			reportError(shell, x.getTargetException());
		} catch (InterruptedException x) {
			reportError(shell, x.getCause());
		} catch (Throwable e) {
			reportError(shell, e);
		}
			
	}

	private IFile getImFileFromBPELFile(){
//		IWorkspace workspace = ResourcesPlugin.getWorkspace();
//		IWorkspaceRoot root = workspace.getRoot();
//		String projectName = bpelFile.getProject().getName();
//		IProject proj = root.getProject(projectName);
		
		
		IPath imFileRelativeToProject = bpelFile.getFullPath().removeFileExtension().addFileExtension("im");
		//String imFileRelativeToProject = bpelFileNameRelativeToProject.replaceFirst(".bpel", ".im");
		IFile imFile = ResourcesPlugin.getWorkspace().getRoot().getFile(imFileRelativeToProject);
		//IFile imFile = bpelFile.getParent().getFile(imFileRelativeToProject);
		return imFile;
	}
	
	private void reportError(Shell shell, Throwable x) {
		ErrorDialog.openError(shell, "Error", "Error Occurred", makeStatus(x));
	}

	public static IStatus makeStatus(Throwable x) {
		return new Status(IStatus.ERROR, BPELDeployActivator.PLUGIN_ID, IStatus.ERROR,
				(x.getMessage() != null) ? x.getMessage()
				: (x.getCause() != null) ? x.getCause().getMessage() : "", null);
	}
	
	public final void selectionChanged(final IAction action,
			final ISelection selection) {
		setBpelFile((IFile) ((StructuredSelection) selection)
				.getFirstElement());
	}
	
	public IFile getBpelFile() {
		return bpelFile;
	}

	public void setBpelFile(IFile bpelFile) {
		this.bpelFile = bpelFile;
	}
	
}
