/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.bpel.deploy.ui.actions;

import groovy.lang.Binding;
import groovy.lang.GroovyShell;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.stp.bpmn.BpmnDiagram;
import org.eclipse.stp.im.BasicProperty;
import org.eclipse.stp.im.ImFactory;
import org.eclipse.stp.im.ProcessCollection;
import org.eclipse.stp.im.StpIntermediateModel;
import org.eclipse.stp.im.constants.ImConstants;
import org.eclipse.stp.im.in.bpmn2im.Bpmn2ImWorkspaceOperation;
import org.eclipse.stp.im.in.bpmn2im.FixBpmnDiagramAction;
import org.eclipse.stp.im.in.bpmn2im.ui.dialogs.ListBpmnPoolsDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.spagic3.bpel.deploy.BPELDeployActivator;
import org.spagic3.utils.SpagicLogger;
import org.spagic3.utils.SpagicUtilsActivator;
import org.spagic3.utils.preference.SpagicPreferenceConstants;

public class UnpublishBpmn2MetaDBAction implements IObjectActionDelegate {

	private IFile bpmnFile = null;
	
	public IFile getBpmnFile() {
		return bpmnFile;
	}

	public void setBpmnFile(IFile bpmnFile) {
		this.bpmnFile = bpmnFile;
	}

	/**
	 * Constructor for Action1.
	 */
	public UnpublishBpmn2MetaDBAction() {
		super();
	}

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
	}

	/**
	 * @see IActionDelegate#run(IAction)
	 */
	public void run(IAction action) {
		Shell shell = new Shell();
		org.eclipse.core.runtime.Preferences prefs = SpagicUtilsActivator.getDefault().getPluginPreferences();
		
		String dbUrl = prefs.getString(SpagicPreferenceConstants.P_BPEL_SPAGIC_DB_URL);
		String dbDriver = prefs.getString(SpagicPreferenceConstants.P_BPEL_SPAGIC_DB_DRIVER);
		String dbUser = prefs.getString(SpagicPreferenceConstants.P_BPEL_SPAGIC_DB_USER);
		String dbPassword = prefs.getString(SpagicPreferenceConstants.P_BPEL_SPAGIC_DB_PASSWORD);
		
		Binding binding = new Binding();
		binding.setVariable("metadb.driver", dbDriver);
		binding.setVariable("metadb.user", dbUser);
		binding.setVariable("metadb.password", dbPassword);
		binding.setVariable("metadb.url", dbUrl);
		binding.setVariable("mode", "standalone");
		try{
			
			IFile imFile = getImFileForBPMNFile(bpmnFile);
			if (!imFile.exists()) {
				ListBpmnPoolsDialog selectPoolsDialog = new ListBpmnPoolsDialog(shell,bpmnFile);
				int status = selectPoolsDialog.open();
				
				if ((status == 1) || ( selectPoolsDialog.getStatus() == 1)){
				
					MessageDialog.openInformation(shell,
							" Information",
							" Operation Cancelled");
					return;
				
				}
				
				List<String> idSelectedPools = selectPoolsDialog.getSelectedPools();
				
				if ((idSelectedPools == null) || (idSelectedPools.isEmpty())) {
					MessageDialog.openInformation(shell,
							" Information",
							" No Pool Selected ");
					return;
				}
				
				WorkspaceModifyOperation operationFixBpmnDiagram = new FixBpmnDiagramAction(bpmnFile, idSelectedPools);
				PlatformUI.getWorkbench().getProgressService().run(false, false, operationFixBpmnDiagram);
				
				WorkspaceModifyOperation createIMOperation = new Bpmn2ImWorkspaceOperation(bpmnFile, idSelectedPools);
				PlatformUI.getWorkbench().getProgressService().run(false, false, createIMOperation);
	
				imFile = getImFileForBPMNFile(bpmnFile);
	
			}
			
			// Retrieve the Intermediate Model that the user selected
			ResourceSet resourceSetIM = new ResourceSetImpl();
			URI uri = URI.createPlatformResourceURI(imFile.getFullPath().toString(),true);
			Resource resourceIM = resourceSetIM.getResource(uri, true);	
			// Get The intermediate model
			StpIntermediateModel imModel= (StpIntermediateModel)resourceIM.getContents().get(0);
			
			// Unpublish all processes within the process collection
			ProcessCollection processCollection = imModel.getProcessCollection();
			List<org.eclipse.stp.im.Process> processList = processCollection.getProcesses();
			List<org.spagic.metadb.model.Process> metaDBProcessList = new ArrayList<org.spagic.metadb.model.Process>(processList.size());
			String initialVersion = "0";
			ImFactory stpModelfactory = ImFactory.eINSTANCE;
			for (org.eclipse.stp.im.Process process : processList) {
				// unpublish the single process 
				unpublishProcess(binding, process, imModel);
				//modify the process version
				BasicProperty versionBasicProperty = stpModelfactory.createBasicProperty();
				versionBasicProperty.setKey(ImConstants.PROCESS_VERSION);
				versionBasicProperty.setValue(initialVersion);
				process.getProperties().put(ImConstants.PROCESS_VERSION, versionBasicProperty);
			}
			//modify the iter version
			BasicProperty iterVersionBasicProperty = stpModelfactory.createBasicProperty();
			iterVersionBasicProperty.setKey(ImConstants.ITER_VERSION);
			iterVersionBasicProperty.setValue(initialVersion);
			imModel.getProperties().put(ImConstants.ITER_VERSION, iterVersionBasicProperty);
			resourceIM.save(Collections.EMPTY_MAP);
			//modify the bpmn resource version 
			ResourceSet resourceSetBpmn = new ResourceSetImpl();
			String imFullPath = imFile.getFullPath().toString();
			String bpmnFullPath = imFullPath.substring(0, imFullPath.indexOf(".")) + ".bpmn";
			URI uriBpmn = URI.createPlatformResourceURI(bpmnFullPath, true);
			Resource resourceBpmn = null;
			try {
				resourceBpmn = resourceSetBpmn.getResource(uriBpmn, true);	
			} catch (Exception ex) {
				SpagicLogger.warning("Unpublish MetaDB Plug-in: file .bpmn doesn't exist");
				MessageDialog.openInformation(
						shell,
						"Unpublish MetaDB Plug-in",
						"Unpublishing from MetaDB was executed.");
				return;
			}
			BpmnDiagram bpmnDiagram = (BpmnDiagram)resourceBpmn.getContents().get(0);
			EAnnotation ea = bpmnDiagram.getEAnnotation(ImConstants.IM_VERSION_ANNOTATION);
			if (ea != null) {
				ea.getDetails().put(ImConstants.PROCESS_VERSION, initialVersion);
				resourceBpmn.save(Collections.EMPTY_MAP);
			}
			
			MessageDialog.openInformation(
					shell,
					"Unpublish MetaDB Plug-in",
					"Unpublishing from MetaDB was executed.");
			
		} catch (IllegalArgumentException iaex) {
			SpagicLogger.error("Unpublish MetaDB Plug-in", iaex);
			MessageDialog.openError(
					shell,
					"Unpublish MetaDB Plug-in",
					iaex.getMessage());
		} catch (Exception e) {
			SpagicLogger.error("Unpublish MetaDB Plug-in", e);
			MessageDialog.openError(
					shell,
					"Unpublish MetaDB Plug-in",
					"Error occurred during unpublishing from MetaDB: " + e);
		} finally {
			
		}
		
		
	}

	
	private IFile getImFileForBPMNFile(IFile bpmnFile){
		IWorkspace workspace = ResourcesPlugin.getWorkspace();
		IWorkspaceRoot root = workspace.getRoot();
		String projectName = bpmnFile.getProject().getName();
		IProject proj = root.getProject(projectName);
		String bpmnFileNameRelativeToProject = bpmnFile.getProjectRelativePath().toPortableString();
//		System.out.println("BPMN File Relative To Project " + bpmnFileNameRelativeToProject);
		String imFileRelativeToProject = bpmnFileNameRelativeToProject.replaceFirst(".bpmn", ".im");
//		System.out.println("IM File Relative To Project " + imFileRelativeToProject);
		IFile imFile = proj.getFile(imFileRelativeToProject);
		return imFile;
	}


	/**
	 * Save a process on meta-db and all related sub-objects (steps, transitions, etc).
	 * @param aSession Hibernate session
	 * @param process Intermediate Model process to save
	 * @return The saved process
	 */
	private void unpublishProcess(Binding binding, org.eclipse.stp.im.Process process, StpIntermediateModel stpIntermediateModel) {
		// Retrieve the processor to apply to this process before saving on the database
		// TODO Dialog richiesta scelta binding
		//String runtimeBinding = retrieveBindingForProcess(process);
		//IProcessor processor = RuntimeProcessor.getInstance().getProcessor(runtimeBinding);
		
		// Apply the processor to the process: this can change the process, but if
		// we dont save the process, the changes are not persisted on the IM (that is
		// the desired behaviour)
		//process = processor.preProcess(process, stpIntermediateModel);
		
		String processName = process.getName();
		//Property propertyVersion = process.getProperties().get(ImConstants.PROCESS_VERSION);
		//String processVersion = (String)propertyVersion.getValue();
		String processVersion = null;
		String input = createInputString(processName, processVersion);
		binding.setVariable("xmlInputMessage",input);
		try {
			FileInputStream inputStreamDynamicData = new FileInputStream(BPELDeployActivator.getFileFromPlugin("/groovy/deleteDynamicData.groovy"));
			FileInputStream inputStreamStaticData = new FileInputStream(BPELDeployActivator.getFileFromPlugin("/groovy/deleteStaticData.groovy"));
			GroovyShell shell = new GroovyShell(binding);
			shell.evaluate(inputStreamDynamicData);
			shell.evaluate(inputStreamStaticData);
		} catch (FileNotFoundException fnfe){
			SpagicLogger.error("File script groovy not found ", fnfe);
		}
	}
	
	/**
	 * Retrieve the binding used by the process
	 * @param process Process to store on database
	 * @return The binding used in the process
	 */
//	private String retrieveBindingForProcess(org.eclipse.stp.im.Process process) {
//		
//			Map<String, String> usedRuntimes = SpagicUtils.getRuntimeUsedByProcess(process);
//			String choosedRuntimeId = null;
//			
//			if (usedRuntimes.values().size() == 1){
//				
//				choosedRuntimeId = usedRuntimes.keySet().iterator().next();
//				MessageDialog.openInformation(new Shell(), "Spagic", "Process ["+process.getName()+"] Only ["+usedRuntimes.values().iterator().next()+"] Runtime Found");
//				
//			}else{
//				
//				ChooseRuntimeDialog aChooseRuntimeDialog = new ChooseRuntimeDialog(new Shell(),usedRuntimes);
//				aChooseRuntimeDialog.open();
//				choosedRuntimeId = aChooseRuntimeDialog.getChoosedRuntimeId();
//			}
//			
//			return choosedRuntimeId;
//	
//	}
	

	public final void selectionChanged(final IAction action,
			final ISelection selection) {
		
		setBpmnFile((IFile) ((StructuredSelection) selection)
				.getFirstElement());
	}
	
	private String createInputString(String processName, String processVersion){
		StringBuffer parameters = new StringBuffer("<parameters");
		if (processName != null)
			parameters.append(" processName='"+processName+"' ");
		
		if (processVersion != null)
			parameters.append(" processVersion='"+processVersion+"' ");
		
		parameters.append("/>");
		return parameters.toString();
	}
	
	
}
