/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.bpel.deploy.ui.actions;

import groovy.lang.Binding;
import groovy.lang.GroovyShell;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.stp.bpmn.BpmnDiagram;
import org.eclipse.stp.im.BasicProperty;
import org.eclipse.stp.im.ImFactory;
import org.eclipse.stp.im.ProcessCollection;
import org.eclipse.stp.im.StpIntermediateModel;
import org.eclipse.stp.im.constants.ImConstants;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.spagic3.bpel.deploy.BPELDeployActivator;
import org.spagic3.utils.SpagicLogger;
import org.spagic3.utils.SpagicUtilsActivator;
import org.spagic3.utils.preference.SpagicPreferenceConstants;

public class UnpublishIM2MetaDBWorkspace extends WorkspaceModifyOperation {

	private IFile imFile = null;
	
	public UnpublishIM2MetaDBWorkspace(IFile imFile) {
		this.imFile = imFile;
	}
	
	@Override
	protected void execute(IProgressMonitor arg0) throws CoreException,
			InvocationTargetException, InterruptedException {
		org.eclipse.core.runtime.Preferences prefs = SpagicUtilsActivator.getDefault().getPluginPreferences();
		
		String dbUrl = prefs.getString(SpagicPreferenceConstants.P_BPEL_SPAGIC_DB_URL);
		String dbDriver = prefs.getString(SpagicPreferenceConstants.P_BPEL_SPAGIC_DB_DRIVER);
		String dbUser = prefs.getString(SpagicPreferenceConstants.P_BPEL_SPAGIC_DB_USER);
		String dbPassword = prefs.getString(SpagicPreferenceConstants.P_BPEL_SPAGIC_DB_PASSWORD);
		
		Binding binding = new Binding();
		binding.setVariable("metadb.driver", dbDriver);
		binding.setVariable("metadb.user", dbUser);
		binding.setVariable("metadb.password", dbPassword);
		binding.setVariable("metadb.url", dbUrl);
		binding.setVariable("mode", "standalone");
		try{
			// Retrieve the Intermediate Model that the user selected
			ResourceSet resourceSetIM = new ResourceSetImpl();
			URI uri = URI.createPlatformResourceURI(imFile.getFullPath().toString(),true);
			Resource resourceIM = resourceSetIM.getResource(uri, true);	
			// Get The intermediate model
			StpIntermediateModel imModel= (StpIntermediateModel)resourceIM.getContents().get(0);
			
			// Unpublish all processes within the process collection
			ProcessCollection processCollection = imModel.getProcessCollection();
			List<org.eclipse.stp.im.Process> processList = processCollection.getProcesses();
			List<org.spagic.metadb.model.Process> metaDBProcessList = new ArrayList<org.spagic.metadb.model.Process>(processList.size());
			String initialVersion = "0";
			ImFactory stpModelfactory = ImFactory.eINSTANCE;
			for (org.eclipse.stp.im.Process process : processList) {
				// unpublish the single process 
				unpublishProcess(binding, process, imModel);
				//modify the process version
				BasicProperty versionBasicProperty = stpModelfactory.createBasicProperty();
				versionBasicProperty.setKey(ImConstants.PROCESS_VERSION);
				versionBasicProperty.setValue(initialVersion);
				process.getProperties().put(ImConstants.PROCESS_VERSION, versionBasicProperty);
			}
			//modify the iter version
			BasicProperty iterVersionBasicProperty = stpModelfactory.createBasicProperty();
			iterVersionBasicProperty.setKey(ImConstants.ITER_VERSION);
			iterVersionBasicProperty.setValue(initialVersion);
			imModel.getProperties().put(ImConstants.ITER_VERSION, iterVersionBasicProperty);
			resourceIM.save(Collections.EMPTY_MAP);
			//modify the bpmn resource version 
			ResourceSet resourceSetBpmn = new ResourceSetImpl();
			String imFullPath = imFile.getFullPath().toString();
			String bpmnFullPath = imFullPath.substring(0, imFullPath.indexOf(".")) + ".bpmn";
			URI uriBpmn = URI.createPlatformResourceURI(bpmnFullPath, true);
			Resource resourceBpmn = null;
			try {
				resourceBpmn = resourceSetBpmn.getResource(uriBpmn, true);	
			
				BpmnDiagram bpmnDiagram= (BpmnDiagram)resourceBpmn.getContents().get(0);
				EAnnotation ea = bpmnDiagram.getEAnnotation(ImConstants.IM_VERSION_ANNOTATION);
				if (ea != null) {
					ea.getDetails().put(ImConstants.PROCESS_VERSION, initialVersion);
					resourceBpmn.save(Collections.EMPTY_MAP);
				}
			} catch (Exception ex) {
				SpagicLogger.info("File bpmn dosn't exist");
			}
			
		} catch (IllegalArgumentException iaex) {
			SpagicLogger.error("Unpublish MetaDB Plug-in", iaex);
			throw new RuntimeException("Error Unpublish MetaDB Plug-in", iaex);

		} catch (Exception e) {
			SpagicLogger.error("Unpublish MetaDB Plug-in", e);
			throw new RuntimeException("Error Unpublish MetaDB Plug-in", e);
		} finally {
			
		}

	}
	
	/**
	 * Save a process on meta-db and all related sub-objects (steps, transitions, etc).
	 * @param aSession Hibernate session
	 * @param process Intermediate Model process to save
	 * @return The saved process
	 */
	private void unpublishProcess(Binding binding, org.eclipse.stp.im.Process process, StpIntermediateModel stpIntermediateModel) {
		String processName = process.getName();
		String processVersion = null;
		String input = createInputString(processName, processVersion);
		binding.setVariable("xmlInputMessage",input);
		try {
			FileInputStream inputStreamDynamicData = new FileInputStream(BPELDeployActivator.getFileFromPlugin("/groovy/deleteDynamicData.groovy"));
			FileInputStream inputStreamStaticData = new FileInputStream(BPELDeployActivator.getFileFromPlugin("/groovy/deleteStaticData.groovy"));
			GroovyShell shell = new GroovyShell(binding);
			shell.evaluate(inputStreamDynamicData);
			shell.evaluate(inputStreamStaticData);
		} catch (FileNotFoundException fnfe){
			SpagicLogger.error("File script groovy not found ", fnfe);
		}
	}
	
	private String createInputString(String processName, String processVersion){
		StringBuffer parameters = new StringBuffer("<parameters");
		if (processName != null)
			parameters.append(" processName='"+processName+"' ");
		
		if (processVersion != null)
			parameters.append(" processVersion='"+processVersion+"' ");
		
		parameters.append("/>");
		return parameters.toString();
	}

}
