/**

    Copyright 2007, 2009 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.bpel.externalserviceextension.ui.properties;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;


import org.eclipse.bpel.common.ui.details.FocusContext;
import org.eclipse.bpel.common.ui.details.IOngoingChange;
import org.eclipse.bpel.common.ui.details.TextIValue;
import org.eclipse.bpel.common.ui.details.IDetailsAreaConstants;
import org.eclipse.bpel.common.ui.details.widgets.DecoratedLabel;
import org.eclipse.bpel.common.ui.details.widgets.StatusLabel2;
import org.eclipse.bpel.common.ui.flatui.FlatFormAttachment;
import org.eclipse.bpel.common.ui.flatui.FlatFormData;
import org.eclipse.bpel.model.BPELPackage;
import org.eclipse.bpel.model.Process;
import org.eclipse.bpel.model.Scope;
import org.eclipse.bpel.model.Variable;
import org.eclipse.bpel.ui.IHelpContextIds;
import org.eclipse.bpel.ui.commands.CompoundCommand;
import org.eclipse.bpel.ui.properties.BPELPropertySection;
import org.eclipse.bpel.ui.properties.ChangeTracker;
import org.eclipse.bpel.ui.properties.EditController;
import org.eclipse.bpel.ui.util.BPELUtil;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.commands.Command;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.PlatformUI;
import org.spagic3.bpel.externalserviceextension.model.ExternalServiceActivity;
import org.spagic3.bpel.externalserviceextension.model.ModelPackage;
import org.spagic3.bpel.externalserviceextension.ui.commands.SetInputVariableCommand;
import org.spagic3.bpel.externalserviceextension.ui.commands.SetOutputVariableCommand;

public class ExternalServiceActivityPropertySection extends BPELPropertySection{
	Text serviceNameText;
	StatusLabel2 serviceNameLabel;
	CCombo inputVariableCombo;
	StatusLabel2 inputVariableLabel;
	CCombo outputVariableCombo;
	StatusLabel2 outputVariableLabel;
	
	EditController serviceNameController;
	ChangeTracker inputTracker, outputTracker;
	
	Composite serviceNameComposite = null;
	Composite inputVariableComposite = null;
	
	public ExternalServiceActivityPropertySection()  {
		super();
	}
	
	protected void createChangeTrackers() {	
		serviceNameController = createEditController();
		serviceNameController.setViewIValue(new TextIValue(serviceNameText));
		serviceNameController.startListeningTo(serviceNameText);
		
		IOngoingChange change = new IOngoingChange() {
			public String getLabel() {
				return "Input Change";
			}
			public Command createApplyCommand() {
				String input = inputVariableCombo.getText();
				CompoundCommand command = new CompoundCommand("Input Change");
				String newName = "".equals(input) ? null : input;
				command.add(new SetInputVariableCommand(getInput(), newName));
				return wrapInShowContextCommand(command);
			}
			public void restoreOldState() {
				updateInputVariableWidgets();
			}
		};
		inputTracker = new ChangeTracker(inputVariableCombo, change, getCommandFramework());
		
		change = new IOngoingChange() {
			public String getLabel() {
				return "Output Change";
			}
			public Command createApplyCommand() {
				String output = outputVariableCombo.getText();
				CompoundCommand command = new CompoundCommand("Output Change");
				String newName = "".equals(output) ? null : output;
				command.add(new SetOutputVariableCommand(getInput(), newName));
				return wrapInShowContextCommand(command);
			}
			public void restoreOldState() {
				updateOutputVariableWidgets();
			}
		};
		inputTracker = new ChangeTracker(outputVariableCombo, change, getCommandFramework());
	}
	
	protected void updateInputVariableWidgets() {
		inputTracker.stopTracking();
		try {
			Object obj = getInput();
			String inputVariableValue = ((ExternalServiceActivity)obj).getInput();
			if (inputVariableValue == null || "".equals(inputVariableValue)) { 
				inputVariableCombo.setText(""); 
			} else if (!inputVariableValue.equals(inputVariableCombo.getText())) {
				if (inputVariableCombo.indexOf(inputVariableValue) == -1) inputVariableValue = ""; 
				inputVariableCombo.setText(inputVariableValue); 
			} 
		} finally {
			inputTracker.startTracking();
		}
	}
	
	protected void updateOutputVariableWidgets() {
		outputTracker.stopTracking();
		try {
			Object obj = getInput();
			String outputVariableValue = ((ExternalServiceActivity)obj).getOutput();
			if (outputVariableValue == null || "".equals(outputVariableValue)) { 
				outputVariableCombo.setText(""); 
			} else if (!outputVariableValue.equals(outputVariableCombo.getText())) {
				if (outputVariableCombo.indexOf(outputVariableValue) == -1) outputVariableValue = ""; 
				outputVariableCombo.setText(outputVariableValue); 
			} 
		} finally {
			outputTracker.startTracking();
		}
	}
	
	@Override
	protected void basicSetInput(EObject newInput) {
		super.basicSetInput(newInput);
		
		if (newInput instanceof ExternalServiceActivity) {
			serviceNameController.setFeature(ModelPackage.eINSTANCE.getExternalServiceActivity_ServiceName());
			serviceNameController.setInput(newInput);
			List<String> variableList = getBuiltinVariables();
			String[] result = variableList.toArray(new String[variableList.size()]);
			inputVariableCombo.setItems(result);
			outputVariableCombo.setItems(result);
			
			String inputVariableValue = ((ExternalServiceActivity)newInput).getInput();
			if (inputVariableValue == null || "".equals(inputVariableValue)) { 
				inputVariableCombo.setText(""); 
			} else if (!inputVariableValue.equals(inputVariableCombo.getText())) {
				if (inputVariableCombo.indexOf(inputVariableValue) == -1) inputVariableValue = ""; 
				inputVariableCombo.setText(inputVariableValue); 
			} 
			
			String outputVariableValue = ((ExternalServiceActivity)newInput).getOutput();
			if (outputVariableValue == null || "".equals(outputVariableValue)) { 
				outputVariableCombo.setText(""); 
			} else if (!outputVariableValue.equals(outputVariableCombo.getText())) {
				if (outputVariableCombo.indexOf(outputVariableValue) == -1) outputVariableValue = ""; 
				outputVariableCombo.setText(outputVariableValue); 
			} 
		} 
	}
	
	protected void createServiceNameWidget (Composite parent) {
		FlatFormData data;
		Composite composite = serviceNameComposite = createFlatFormComposite(parent);
		data = new FlatFormData();
		data.left = new FlatFormAttachment(0, 0);
		data.right = new FlatFormAttachment(100, 0);
		data.top = new FlatFormAttachment(0, IDetailsAreaConstants.VSPACE);
		composite.setLayoutData(data);
		
		DecoratedLabel nameLabel = new DecoratedLabel(composite,SWT.LEFT);
		fWidgetFactory.adapt(nameLabel);		
		nameLabel.setText("Service ID:"); 
		serviceNameLabel = new StatusLabel2( nameLabel );		
		
		serviceNameText = fWidgetFactory.createText(composite, EMPTY_STRING);
		serviceNameText.setData(FocusContext.NAME,"serviceName");
		
		data = new FlatFormData();
		data.left = new FlatFormAttachment(IDetailsAreaConstants.HSPACE, BPELUtil.calculateLabelWidth(nameLabel, STANDARD_LABEL_WIDTH_AVG));
		data.right = new FlatFormAttachment(100,  -2 * IDetailsAreaConstants.HSPACE);
		data.top = new FlatFormAttachment(0, 2);
		serviceNameText.setLayoutData(data);

		data = new FlatFormData();
		data.left = new FlatFormAttachment(0,0 );
		data.right = new FlatFormAttachment(serviceNameText, -2 * IDetailsAreaConstants.HSPACE);
		data.top = new FlatFormAttachment(serviceNameText, 0, SWT.CENTER);
		nameLabel.setLayoutData(data);
	}
	
	protected void createInputVariableWidgets(Composite parent) {
		FlatFormData data;

		Composite composite = inputVariableComposite = createFlatFormComposite(parent);
		data = new FlatFormData();
		data.left = new FlatFormAttachment(0, 0);
		data.right = new FlatFormAttachment(100, 0);
		data.top = new FlatFormAttachment(serviceNameComposite, IDetailsAreaConstants.VSPACE);
		composite.setLayoutData(data);
		
		inputVariableCombo = fWidgetFactory.createCCombo(composite);
		DecoratedLabel nameLabel = new DecoratedLabel(composite,SWT.LEFT);
		fWidgetFactory.adapt(nameLabel);		
		nameLabel.setText("Input:"); 
		inputVariableLabel = new StatusLabel2( nameLabel );	 
		
		data = new FlatFormData();
		data.left = new FlatFormAttachment(IDetailsAreaConstants.HSPACE, BPELUtil.calculateLabelWidth(nameLabel, STANDARD_LABEL_WIDTH_AVG));
		data.right = new FlatFormAttachment(100,  -2 * IDetailsAreaConstants.HSPACE);
		data.top = new FlatFormAttachment(0, 0);		
		inputVariableCombo.setLayoutData(data);
		
		data = new FlatFormData();
		data.left = new FlatFormAttachment(0,0);
		data.right = new FlatFormAttachment(inputVariableCombo, -2 * IDetailsAreaConstants.HSPACE);
		data.top = new FlatFormAttachment(inputVariableCombo, 0, SWT.CENTER);
		nameLabel.setLayoutData(data);
	}
	
	protected void createOutputVariableWidgets(Composite parent) {
		FlatFormData data;

		Composite composite = createFlatFormComposite(parent);
		data = new FlatFormData();
		data.left = new FlatFormAttachment(0, 0);
		data.right = new FlatFormAttachment(100, 0);
		data.top = new FlatFormAttachment(inputVariableComposite, IDetailsAreaConstants.VSPACE);
		composite.setLayoutData(data);
		
		outputVariableCombo = fWidgetFactory.createCCombo(composite);
		DecoratedLabel nameLabel = new DecoratedLabel(composite,SWT.LEFT);
		fWidgetFactory.adapt(nameLabel);		
		nameLabel.setText("Output:"); 
		outputVariableLabel = new StatusLabel2( nameLabel );	 
		
		data = new FlatFormData();
		data.left = new FlatFormAttachment(IDetailsAreaConstants.HSPACE, BPELUtil.calculateLabelWidth(nameLabel, STANDARD_LABEL_WIDTH_AVG));
		data.right = new FlatFormAttachment(100,  -2 * IDetailsAreaConstants.HSPACE);
		data.top = new FlatFormAttachment(0, 0);		
		
		outputVariableCombo.setLayoutData(data);
		
		data = new FlatFormData();
		data.left = new FlatFormAttachment(0,0);
		data.right = new FlatFormAttachment(outputVariableCombo, -2 * IDetailsAreaConstants.HSPACE);
		data.top = new FlatFormAttachment(outputVariableCombo, 0, SWT.CENTER);
		nameLabel.setLayoutData(data);
	}

	protected List<String> getBuiltinVariables() {
		List<String> variablesList = new ArrayList<String>();
		Object object = getInput();
		if (object instanceof EObject) {
			if (object instanceof Process){
				Process process = (Process)object;
				addVariables(variablesList, process.getVariables().getChildren());
			} else {
				EObject cont = ((EObject)object).eContainer();
				while (cont != null) {
					if (cont.eClass() == BPELPackage.eINSTANCE.getScope()){
						Scope scope = (Scope)cont;
						addVariables(variablesList, scope.getVariables().getChildren());
					} else if (cont.eClass() == BPELPackage.eINSTANCE.getProcess()){
						Process process = (Process)cont;
						addVariables(variablesList, process.getVariables().getChildren());
					}
					cont = cont.eContainer();
				}
			}
		}
		return variablesList;
	}
	
	private void addVariables(List<String> variablesList, EList<Variable> objectVariablesList){
		Iterator it = objectVariablesList.iterator();
		while (it.hasNext()){
			String variableName = ((Variable)it.next()).getName();
			if(!variablesList.contains(variableName))
				variablesList.add(variableName);
		}
	}
	
	@Override
	protected void createClient(Composite parent) {
		Composite composite = createFlatFormComposite(parent);
		createServiceNameWidget(composite);
		createInputVariableWidgets(composite);
		createOutputVariableWidgets(composite);
		createChangeTrackers();

		PlatformUI.getWorkbench().getHelpSystem().setHelp(
				composite, IHelpContextIds.PROPERTY_PAGE_NAME);
	}
}