/**

    Copyright 2007, 2009 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.bpel.manualactivitiesextension.ui.properties;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.bpel.common.ui.details.FocusContext;
import org.eclipse.bpel.common.ui.details.IDetailsAreaConstants;
import org.eclipse.bpel.common.ui.details.IOngoingChange;
import org.eclipse.bpel.common.ui.details.TextIValue;
import org.eclipse.bpel.common.ui.details.widgets.DecoratedLabel;
import org.eclipse.bpel.common.ui.details.widgets.StatusLabel2;
import org.eclipse.bpel.common.ui.flatui.FlatFormAttachment;
import org.eclipse.bpel.common.ui.flatui.FlatFormData;
import org.eclipse.bpel.model.BPELPackage;
import org.eclipse.bpel.model.Process;
import org.eclipse.bpel.model.Scope;
import org.eclipse.bpel.model.Variable;
import org.eclipse.bpel.ui.IHelpContextIds;
import org.eclipse.bpel.ui.commands.CompoundCommand;
import org.eclipse.bpel.ui.properties.BPELPropertySection;
import org.eclipse.bpel.ui.properties.ChangeTracker;
import org.eclipse.bpel.ui.properties.EditController;
import org.eclipse.bpel.ui.util.BPELUtil;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.commands.Command;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;
import org.spagic3.bpel.manualactivitiesextension.model.ManualActivity;
import org.spagic3.bpel.manualactivitiesextension.model.ModelPackage;
import org.spagic3.bpel.manualactivitiesextension.ui.commands.SetInputVariableCommand;

public class ManualActivityPropertySection extends BPELPropertySection{
	Text activityNameText;
	StatusLabel2 activityNameLabel;
	CCombo inputVariableCombo;
	StatusLabel2 inputVariableLabel;
	
	
	EditController activityNameController;
	ChangeTracker inputTracker;
	
	Composite activityNameComposite = null;
	//Composite inputVariableComposite = null;
	
	public ManualActivityPropertySection()  {
		super();
	}
	
	protected void createChangeTrackers() {	
		activityNameController = createEditController();
		activityNameController.setViewIValue(new TextIValue(activityNameText));
		activityNameController.startListeningTo(activityNameText);
		
		IOngoingChange change = new IOngoingChange() {
			public String getLabel() {
				return "Input Change";
			}
			public Command createApplyCommand() {
				String input = inputVariableCombo.getText();
				CompoundCommand command = new CompoundCommand("Input Change");
				String newName = "".equals(input) ? null : input;
				command.add(new SetInputVariableCommand(getInput(), newName));
				return wrapInShowContextCommand(command);
			}
			public void restoreOldState() {
				updateInputVariableWidgets();
			}
		};
		inputTracker = new ChangeTracker(inputVariableCombo, change, getCommandFramework());
	}
	
	protected void updateInputVariableWidgets() {
		inputTracker.stopTracking();
		try {
			Object obj = getInput();
			String inputVariableValue = ((ManualActivity)obj).getInput();
			if (inputVariableValue == null || "".equals(inputVariableValue)) { 
				inputVariableCombo.setText(""); 
			} else if (!inputVariableValue.equals(inputVariableCombo.getText())) {
				if (inputVariableCombo.indexOf(inputVariableValue) == -1) inputVariableValue = ""; 
				inputVariableCombo.setText(inputVariableValue); 
			} 
		} finally {
			inputTracker.startTracking();
		}
	}
	
	@Override
	protected void basicSetInput(EObject newInput) {
		super.basicSetInput(newInput);
		
		if (newInput instanceof ManualActivity) {
			activityNameController.setFeature(ModelPackage.eINSTANCE.getManualActivity_ActivityName());
			activityNameController.setInput(newInput);
			List<String> variableList = getBuiltinVariables();
			String[] result = variableList.toArray(new String[variableList.size()]);
			inputVariableCombo.setItems(result);
			
			String inputVariableValue = ((ManualActivity)newInput).getInput();
			if (inputVariableValue == null || "".equals(inputVariableValue)) { 
				inputVariableCombo.setText(""); 
			} else if (!inputVariableValue.equals(inputVariableCombo.getText())) {
				if (inputVariableCombo.indexOf(inputVariableValue) == -1) inputVariableValue = ""; 
				inputVariableCombo.setText(inputVariableValue); 
			} 
		} 
	}
	
	protected void createActivityNameWidget (Composite parent) {
		FlatFormData data;
		Composite composite = activityNameComposite = createFlatFormComposite(parent);
		data = new FlatFormData();
		data.left = new FlatFormAttachment(0, 0);
		data.right = new FlatFormAttachment(100, 0);
		data.top = new FlatFormAttachment(0, IDetailsAreaConstants.VSPACE);
		composite.setLayoutData(data);
		
		DecoratedLabel nameLabel = new DecoratedLabel(composite,SWT.LEFT);
		fWidgetFactory.adapt(nameLabel);		
		nameLabel.setText("Activity Name:"); 
		activityNameLabel = new StatusLabel2( nameLabel );		
		
		activityNameText = fWidgetFactory.createText(composite, EMPTY_STRING);
		activityNameText.setData(FocusContext.NAME,"activityName");
		
		data = new FlatFormData();
		data.left = new FlatFormAttachment(IDetailsAreaConstants.HSPACE, BPELUtil.calculateLabelWidth(nameLabel, STANDARD_LABEL_WIDTH_AVG));
		data.right = new FlatFormAttachment(100,  -2 * IDetailsAreaConstants.HSPACE);
		data.top = new FlatFormAttachment(0, 2);
		activityNameText.setLayoutData(data);

		data = new FlatFormData();
		data.left = new FlatFormAttachment(0,0 );
		data.right = new FlatFormAttachment(activityNameText, -2 * IDetailsAreaConstants.HSPACE);
		data.top = new FlatFormAttachment(activityNameText, 0, SWT.CENTER);
		nameLabel.setLayoutData(data);
	}
	
	protected void createInputVariableWidgets(Composite parent) {
		FlatFormData data;

		Composite composite = createFlatFormComposite(parent);
		data = new FlatFormData();
		data.left = new FlatFormAttachment(0, 0);
		data.right = new FlatFormAttachment(100, 0);
		data.top = new FlatFormAttachment(activityNameComposite, IDetailsAreaConstants.VSPACE);
		composite.setLayoutData(data);
		
		inputVariableCombo = fWidgetFactory.createCCombo(composite);
		DecoratedLabel nameLabel = new DecoratedLabel(composite,SWT.LEFT);
		fWidgetFactory.adapt(nameLabel);		
		nameLabel.setText("Input:"); 
		inputVariableLabel = new StatusLabel2( nameLabel );	 
		
		data = new FlatFormData();
		data.left = new FlatFormAttachment(IDetailsAreaConstants.HSPACE, BPELUtil.calculateLabelWidth(nameLabel, STANDARD_LABEL_WIDTH_AVG));
		data.right = new FlatFormAttachment(100,  -2 * IDetailsAreaConstants.HSPACE);
		data.top = new FlatFormAttachment(0, 0);		
		inputVariableCombo.setLayoutData(data);
		
		data = new FlatFormData();
		data.left = new FlatFormAttachment(0,0);
		data.right = new FlatFormAttachment(inputVariableCombo, -2 * IDetailsAreaConstants.HSPACE);
		data.top = new FlatFormAttachment(inputVariableCombo, 0, SWT.CENTER);
		nameLabel.setLayoutData(data);
	}

	protected List<String> getBuiltinVariables() {
		List<String> variablesList = new ArrayList<String>();
		Object object = getInput();
		if (object instanceof EObject) {
			if (object instanceof Process){
				Process process = (Process)object;
				addVariables(variablesList, process.getVariables().getChildren());
			} else {
				EObject cont = ((EObject)object).eContainer();
				while (cont != null) {
					if (cont.eClass() == BPELPackage.eINSTANCE.getScope()){
						Scope scope = (Scope)cont;
						addVariables(variablesList, scope.getVariables().getChildren());
					} else if (cont.eClass() == BPELPackage.eINSTANCE.getProcess()){
						Process process = (Process)cont;
						addVariables(variablesList, process.getVariables().getChildren());
					}
					cont = cont.eContainer();
				}
			}
		}
		return variablesList;
	}
	
	private void addVariables(List<String> variablesList, EList<Variable> objectVariablesList){
		Iterator it = objectVariablesList.iterator();
		while (it.hasNext()){
			String variableName = ((Variable)it.next()).getName();
			if(!variablesList.contains(variableName))
				variablesList.add(variableName);
		}
	}
	
	@Override
	protected void createClient(Composite parent) {
		Composite composite = createFlatFormComposite(parent);
		createActivityNameWidget(composite);
		createInputVariableWidgets(composite);
		createChangeTrackers();

		PlatformUI.getWorkbench().getHelpSystem().setHelp(
				composite, IHelpContextIds.PROPERTY_PAGE_NAME);
	}
}