/**

    Copyright 2007, 2009 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.bpel.relevantdataextension.model.util;

import javax.xml.namespace.QName;

import org.eclipse.bpel.model.Activity;
import org.eclipse.bpel.model.BPELFactory;
import org.eclipse.bpel.model.Extension;
import org.eclipse.bpel.model.Extensions;
import org.eclipse.bpel.model.Process;
import org.eclipse.bpel.model.extensions.BPELActivitySerializer;
import org.eclipse.bpel.model.resource.BPELWriter;
import org.eclipse.emf.common.util.EList;
import org.spagic3.bpel.relevantdataextension.model.ModelPackage;
import org.spagic3.bpel.relevantdataextension.model.RelevantDataActivity;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

public class RelevantDataActivitySerializer implements
		BPELActivitySerializer {

	public void marshall(QName elementType, Activity activity, Node parentNode,
			Process process, BPELWriter bpelWriter) {
		Document document = parentNode.getOwnerDocument();
		if (activity instanceof RelevantDataActivity){
			// create a new DOM element for our Activity
			Element activityElement = document.createElementNS(elementType.getNamespaceURI(),
					RelevantDataExtensionConstants.ND_EXTERNAL_SERVICE_ACTIVITY);
			activityElement.setPrefix(RelevantDataExtensionUtils.addNamespace(process));

			// handle attributes
			String variable = ((RelevantDataActivity) activity).getVariable();
			if (variable != null)
				activityElement.setAttribute("variable", variable);
			else
				activityElement.setAttribute("variable", "");
			
			String xpath = ((RelevantDataActivity) activity).getXpath();
			if (xpath != null)
				activityElement.setAttribute("xpath", xpath);
			else
				activityElement.setAttribute("xpath", "");
			
			String attributeName = ((RelevantDataActivity) activity).getAttributeName();
			if (attributeName != null)
				activityElement.setAttribute("attributeName", attributeName);
			else
				activityElement.setAttribute("attributeName", "");

			// insert the DOM element into the DOM tree
			parentNode.appendChild(activityElement);
			//add eventually the extensions
			if (process.getExtensions() == null) {
				Extensions extensions = BPELFactory.eINSTANCE.createExtensions();
				process.setExtensions(extensions);
			}
			//add eventually the extension
			Extension extension = BPELFactory.eINSTANCE.createExtension();
			extension.setNamespace(ModelPackage.eNS_URI);
			extension.setMustUnderstand(new Boolean(true));
			boolean containsExtension = false;
			if (!process.getExtensions().getChildren().isEmpty()){
				EList<Extension> extensionList = process.getExtensions().getChildren();
				for (Extension ext : extensionList)
					if (ext.getNamespace().equalsIgnoreCase(ModelPackage.eNS_URI)){
						containsExtension = true;
						break;
					}
			} 
			if (!containsExtension)
				process.getExtensions().getChildren().add(extension);
		}

	}

}

