package org.spagic3.bpel.relevantdataextension.ui.properties;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import org.eclipse.bpel.common.ui.details.IDetailsAreaConstants;
import org.eclipse.bpel.common.ui.details.IOngoingChange;
import org.eclipse.bpel.common.ui.details.TextIValue;
import org.eclipse.bpel.common.ui.details.widgets.DecoratedLabel;
import org.eclipse.bpel.common.ui.details.widgets.StatusLabel2;
import org.eclipse.bpel.common.ui.flatui.FlatFormAttachment;
import org.eclipse.bpel.common.ui.flatui.FlatFormData;
import org.eclipse.bpel.model.BPELPackage;
import org.eclipse.bpel.model.Process;
import org.eclipse.bpel.model.Scope;
import org.eclipse.bpel.model.Variable;
import org.eclipse.bpel.model.util.BPELUtils;
import org.eclipse.bpel.ui.IHelpContextIds;
import org.eclipse.bpel.ui.commands.CompoundCommand;
import org.eclipse.bpel.ui.details.providers.ModelTreeLabelProvider;
import org.eclipse.bpel.ui.details.providers.VariableTypeTreeContentProvider;
import org.eclipse.bpel.ui.details.tree.ITreeNode;
import org.eclipse.bpel.ui.properties.BPELPropertySection;
import org.eclipse.bpel.ui.properties.ChangeTracker;
import org.eclipse.bpel.ui.properties.EditController;
import org.eclipse.bpel.ui.util.BPELUtil;
import org.eclipse.bpel.ui.util.XSDUtils;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.commands.Command;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.ui.PlatformUI;
import org.eclipse.wst.wsdl.internal.impl.MessageImpl;
import org.eclipse.wst.wsdl.internal.impl.PartImpl;
import org.eclipse.xsd.XSDAttributeDeclaration;
import org.eclipse.xsd.XSDElementDeclaration;
import org.eclipse.xsd.XSDNamedComponent;
import org.spagic3.bpel.relevantdataextension.model.ModelPackage;
import org.spagic3.bpel.relevantdataextension.model.RelevantDataActivity;
import org.spagic3.bpel.relevantdataextension.ui.commands.SetInputVariableCommand;

public class RelevantDataPropertySection extends BPELPropertySection {
	Tree fVariableTree;
	TreeViewer fVariableViewer;
	EObject variableType = null; 
	StatusLabel2 treeLabel;
	VariableTypeTreeContentProvider variableTypeContentProvider;
	Shell shell;
	
	Text attributeNameText;
	StatusLabel2 attributeNameLabel;
	EditController attributeNameController;
	CCombo inputVariableCombo;
	StatusLabel2 inputVariableLabel;
	Text xpathText;
	StatusLabel2 xpathLabel;
	EditController xpathController;
	ChangeTracker inputTracker;
	Composite attributeNameComposite = null;
	Composite inputVariableComposite = null;
	Composite treeComposite = null;

	public RelevantDataPropertySection()  {
		super();
	}
	@Override
	protected final void createClient(Composite parent) {
		Composite composite = createFlatFormComposite(parent);
		createAttributeNameWidget(composite);
		createInputVariableWidgets(composite);
		createTreeWidget(composite);
		createXpathWidget(composite);
		
		createChangeTrackers();
		
		

		PlatformUI.getWorkbench().getHelpSystem().setHelp(
				composite, IHelpContextIds.PROPERTY_PAGE_NAME);
	}
	
	protected void createTreeWidget(Composite parent) {
			FlatFormData data;
			
			Composite composite = treeComposite = createFlatFormComposite(parent);
			data = new FlatFormData();
			data.left = new FlatFormAttachment(0, 0);
			data.top = new FlatFormAttachment(inputVariableComposite, 0);
			data.right = new FlatFormAttachment(100, 0);
			data.bottom = new FlatFormAttachment(100, 0);

			fVariableTree = fWidgetFactory.createTree(composite, SWT.RIGHT /* SWT.BORDER */);
			fVariableTree.setLayoutData(data);
			
//			DecoratedLabel nameLabel = new DecoratedLabel(composite,SWT.LEFT);
//			fWidgetFactory.adapt(nameLabel);		
//			nameLabel.setText("Structure:"); 
//			treeLabel = new StatusLabel2( nameLabel );	
//			
//			data = new FlatFormData();
//			data.left = new FlatFormAttachment(IDetailsAreaConstants.HSPACE, BPELUtil.calculateLabelWidth(nameLabel, STANDARD_LABEL_WIDTH_AVG));
//			data.right = new FlatFormAttachment(100,  -2 * IDetailsAreaConstants.HSPACE);
//			data.top = new FlatFormAttachment(0, 0);		
//			fVariableTree.setLayoutData(data);
//			
//			data = new FlatFormData();
//			data.left = new FlatFormAttachment(0,0);
//			data.right = new FlatFormAttachment(fVariableTree, -2 * IDetailsAreaConstants.HSPACE);
//			data.top = new FlatFormAttachment(fVariableTree, 0, SWT.CENTER);
//			nameLabel.setLayoutData(data);
			
			variableTypeContentProvider = new VariableTypeTreeContentProvider(true, true);
			fVariableViewer = new TreeViewer(fVariableTree);
			fVariableViewer.setContentProvider(variableTypeContentProvider);
			fVariableViewer.setLabelProvider(new ModelTreeLabelProvider());
			fVariableViewer.setAutoExpandLevel(3);
			
			fVariableViewer
					.addSelectionChangedListener(new ISelectionChangedListener() {
						public void selectionChanged(SelectionChangedEvent event) {
							updateXPathFieldFromTreeSelection();
						}
					});
	}
	
	protected void createAttributeNameWidget (Composite parent) {
		FlatFormData data;
		Composite composite = attributeNameComposite = createFlatFormComposite(parent);
		data = new FlatFormData();
		data.left = new FlatFormAttachment(0, 0);
		data.right = new FlatFormAttachment(100, 0);
		data.top = new FlatFormAttachment(0, IDetailsAreaConstants.VSPACE);
		composite.setLayoutData(data);
		
		DecoratedLabel nameLabel = new DecoratedLabel(composite,SWT.LEFT);
		fWidgetFactory.adapt(nameLabel);		
		nameLabel.setText("Attribute Name:"); 
		attributeNameLabel = new StatusLabel2( nameLabel );		
		
		attributeNameText = fWidgetFactory.createText(composite, EMPTY_STRING);
		
		data = new FlatFormData();
		data.left = new FlatFormAttachment(IDetailsAreaConstants.HSPACE, BPELUtil.calculateLabelWidth(nameLabel, STANDARD_LABEL_WIDTH_AVG));
		data.right = new FlatFormAttachment(100,  -2 * IDetailsAreaConstants.HSPACE);
		data.top = new FlatFormAttachment(0, 2);
		attributeNameText.setLayoutData(data);

		data = new FlatFormData();
		data.left = new FlatFormAttachment(0,0 );
		data.right = new FlatFormAttachment(attributeNameText, -2 * IDetailsAreaConstants.HSPACE);
		data.top = new FlatFormAttachment(attributeNameText, 0, SWT.CENTER);
		nameLabel.setLayoutData(data);
	}
	
	protected void createXpathWidget (Composite parent) {
		FlatFormData data;
		Composite composite = createFlatFormComposite(parent);
		data = new FlatFormData();
		data.left = new FlatFormAttachment(0, 0);
		data.right = new FlatFormAttachment(100, 0);
		data.top = new FlatFormAttachment(treeComposite, IDetailsAreaConstants.VSPACE);
		composite.setLayoutData(data);
		
		DecoratedLabel nameLabel = new DecoratedLabel(composite,SWT.LEFT);
		fWidgetFactory.adapt(nameLabel);		
		nameLabel.setText("XPath:"); 
		xpathLabel = new StatusLabel2( nameLabel );		
		
		xpathText = fWidgetFactory.createText(composite, EMPTY_STRING);
		
		data = new FlatFormData();
		data.left = new FlatFormAttachment(IDetailsAreaConstants.HSPACE, BPELUtil.calculateLabelWidth(nameLabel, STANDARD_LABEL_WIDTH_AVG));
		data.right = new FlatFormAttachment(100,  -2 * IDetailsAreaConstants.HSPACE);
		data.top = new FlatFormAttachment(0, 2);
		xpathText.setLayoutData(data);

		data = new FlatFormData();
		data.left = new FlatFormAttachment(0,0 );
		data.right = new FlatFormAttachment(xpathText, -2 * IDetailsAreaConstants.HSPACE);
		data.top = new FlatFormAttachment(xpathText, 0, SWT.CENTER);
		nameLabel.setLayoutData(data);
	}
	
	protected void createInputVariableWidgets(Composite parent) {
		FlatFormData data;
		Composite composite = inputVariableComposite = createFlatFormComposite(parent);
		data = new FlatFormData();
		data.left = new FlatFormAttachment(0, 0);
		data.right = new FlatFormAttachment(100, 0);
		data.top = new FlatFormAttachment(attributeNameComposite, IDetailsAreaConstants.VSPACE);
		composite.setLayoutData(data);
		
		inputVariableCombo = fWidgetFactory.createCCombo(composite);
		DecoratedLabel nameLabel = new DecoratedLabel(composite,SWT.LEFT);
		fWidgetFactory.adapt(nameLabel);		
		nameLabel.setText("Variable:"); 
		inputVariableLabel = new StatusLabel2( nameLabel );	 
		
		data = new FlatFormData();
		data.left = new FlatFormAttachment(IDetailsAreaConstants.HSPACE, BPELUtil.calculateLabelWidth(nameLabel, STANDARD_LABEL_WIDTH_AVG));
		data.right = new FlatFormAttachment(100,  -2 * IDetailsAreaConstants.HSPACE);
		data.top = new FlatFormAttachment(0, 0);		
		inputVariableCombo.setLayoutData(data);
		
		data = new FlatFormData();
		data.left = new FlatFormAttachment(0,0);
		data.right = new FlatFormAttachment(inputVariableCombo, -2 * IDetailsAreaConstants.HSPACE);
		data.top = new FlatFormAttachment(inputVariableCombo, 0, SWT.CENTER);
		nameLabel.setLayoutData(data);
	}
	
	protected void basicSetInput(EObject newInput) {
		super.basicSetInput(newInput);
		
		if (newInput instanceof RelevantDataActivity) {
			xpathController.setFeature(ModelPackage.eINSTANCE.getRelevantDataActivity_Xpath());
			xpathController.setInput(newInput);
			attributeNameController.setFeature(ModelPackage.eINSTANCE.getRelevantDataActivity_AttributeName());
			attributeNameController.setInput(newInput);
			List<String> variableList = getBuiltinVariables();
			String[] result = variableList.toArray(new String[variableList.size()]);
			inputVariableCombo.setItems(result);
			String inputVariableValue = ((RelevantDataActivity)newInput).getVariable();
			if (inputVariableValue == null || "".equals(inputVariableValue)) { 
				inputVariableCombo.setText(""); 
			} else if (!inputVariableValue.equals(inputVariableCombo.getText())) {
				if (inputVariableCombo.indexOf(inputVariableValue) == -1) inputVariableValue = ""; 
				inputVariableCombo.setText(inputVariableValue); 
			} 
			
			Variable variable = getVariables(inputVariableValue);
			if (variable != null) {  
				if (variable.getMessageType() != null) {
					variableType = variable.getMessageType();
				}
				if (variable.getType() != null) {
					variableType = variable.getType();
				}
				if (variable.getXSDElement() != null) {
					variableType = variable.getXSDElement();
				}
			} else 
				variableType = null;
			fVariableViewer.setInput(variableType);
		} 
	}
	
	protected List<String> getBuiltinVariables() {
		List<String> variablesList = new ArrayList<String>();
		Object object = getInput();
		if (object instanceof EObject) {
			if (object instanceof Process){
				Process process = (Process)object;
				addVariables(variablesList, process.getVariables().getChildren());
			} else {
				EObject cont = ((EObject)object).eContainer();
				while (cont != null) {
					if (cont.eClass() == BPELPackage.eINSTANCE.getScope()){
						Scope scope = (Scope)cont;
						addVariables(variablesList, scope.getVariables().getChildren());
					} else if (cont.eClass() == BPELPackage.eINSTANCE.getProcess()){
						Process process = (Process)cont;
						addVariables(variablesList, process.getVariables().getChildren());
					}
					cont = cont.eContainer();
				}
			}
		}
		return variablesList;
	}
	
	protected Variable getVariables(String variableInput) {
		Variable variable = null;
		EList<Variable> variablesList = null;
		Iterator it = null;
		Variable variableTemp = null;
		String variableName = null;
		Object object = getInput();
		if (object instanceof EObject) {
			if (object instanceof Process){
				Process process = (Process)object;
				variablesList = process.getVariables().getChildren();
				it = variablesList.iterator();
				while (it.hasNext()){
					variableTemp = (Variable)it.next();
					variableName = variableTemp.getName();
					if(variableName.equals(variableInput)){
						variable = variableTemp;
						break;
					}
				}
			} else {
				EObject cont = ((EObject)object).eContainer();
				while (cont != null) {
					if (cont.eClass() == BPELPackage.eINSTANCE.getScope()){
						Scope scope = (Scope)cont;
						variablesList = scope.getVariables().getChildren();
						it = variablesList.iterator();
						while (it.hasNext()){
							variableTemp = (Variable)it.next();
							variableName = variableTemp.getName();
							if(variableName.equals(variableInput)) {
								variable = variableTemp;
								break;
							}
						}
					} else if (cont.eClass() == BPELPackage.eINSTANCE.getProcess()){
						Process process = (Process)cont;
						variablesList = process.getVariables().getChildren();
						it = variablesList.iterator();
						while (it.hasNext()){
							variableTemp = (Variable)it.next();
							variableName = variableTemp.getName();
							if(variableName.equals(variableInput)) {
								variable = variableTemp;
								break;
							}
						}
					}
					cont = cont.eContainer();
				}
			}
			
		}
		return variable;
	}
	
	private void addVariables(List<String> variablesList, EList<Variable> objectVariablesList){
		Iterator it = objectVariablesList.iterator();
		while (it.hasNext()){
			String variableName = ((Variable)it.next()).getName();
			if(!variablesList.contains(variableName))
				variablesList.add(variableName);
		}
	}

	protected void updateStructureWidgets(String var) {
		Variable variable = getVariables(var);
		if (variable != null) {  
			if (variable.getMessageType() != null) {
				variableType = variable.getMessageType();
			}
			if (variable.getType() != null) {
				variableType = variable.getType();
			}
			if (variable.getXSDElement() != null) {
				variableType = variable.getXSDElement();
			}
		} else 
			variableType = null;
		fVariableViewer.setInput(variableType);
	}

	protected void createChangeTrackers() {	
		IOngoingChange change = new IOngoingChange() {
			public String getLabel() {
				return "Input Change";
			}
			public Command createApplyCommand() {
				String input = inputVariableCombo.getText();
				CompoundCommand command = new CompoundCommand("Input Change");
				String newName = "".equals(input) ? null : input;
				command.add(new SetInputVariableCommand(getInput(), newName));
				updateStructureWidgets(newName);
				cleanXPathWidgets();
				return wrapInShowContextCommand(command);
			}
			public void restoreOldState() {
				updateInputVariableWidgets();
			}
		};
		inputTracker = new ChangeTracker(inputVariableCombo, change, getCommandFramework());
		
		attributeNameController = createEditController();
		attributeNameController.setViewIValue(new TextIValue(attributeNameText));
		attributeNameController.startListeningTo(attributeNameText);
		
		xpathController = createEditController();
		xpathController.setViewIValue(new TextIValue(xpathText));
		xpathController.startListeningTo(xpathText);
	}
	
	protected void updateInputVariableWidgets() {
		inputTracker.stopTracking();
		try {
			Object obj = getInput();
			String inputVariableValue = ((RelevantDataActivity)obj).getVariable();
			if (inputVariableValue == null || "".equals(inputVariableValue)) { 
				inputVariableCombo.setText(""); 
			} else if (!inputVariableValue.equals(inputVariableCombo.getText())) {
				if (inputVariableCombo.indexOf(inputVariableValue) == -1) inputVariableValue = ""; 
				inputVariableCombo.setText(inputVariableValue); 
			} 
		} finally {
			inputTracker.startTracking();
		}
	}
	
	protected void updateXPathFieldFromTreeSelection() {
		IStructuredSelection sel = (IStructuredSelection) fVariableViewer
				.getSelection();
		Object[] path = variableTypeContentProvider.getPathToRoot(sel
				.getFirstElement());
		Process process = getProcess();
		StringBuilder builder = new StringBuilder();
		ArrayList<String> querySegments = new ArrayList<String>();

		for (Object next : path) {

			Object eObj = BPELUtil.resolveXSDObject(BPELUtil.adapt(next,
					ITreeNode.class).getModelObject());
			builder.setLength(0);

			String targetNamespace = null;
			String namespacePrefix = null;
			String namespaceUri = "";
			String prefixUri = "";
			String name = "";
			if (eObj instanceof MessageImpl) {
				MessageImpl message = (MessageImpl)eObj;
				name = message.getElement().getLocalName();
				//String localName = message.getQName().getLocalPart();
//				namespaceUri = message.getQName().getNamespaceURI();
//				prefixUri = BPELUtils.getNamespacePrefix(process, namespaceUri);
				builder.append("/").append(name);
			} if (eObj instanceof PartImpl) {
				PartImpl part = (PartImpl)eObj;
				String partName = part.getName();
				String elementName = part.getElementName().getLocalPart();
				prefixUri = BPELUtils.getNamespacePrefix(process, part.getElementName().getNamespaceURI());
				builder.append("/").append(partName).append("/").append(part.getElementName().getNamespaceURI()).append(":").append(elementName);
			}
			if (eObj instanceof XSDAttributeDeclaration) {

				XSDAttributeDeclaration att = (XSDAttributeDeclaration) eObj;
				targetNamespace = att.getTargetNamespace();
				builder.append("/@");

				if (targetNamespace != null) {

					namespacePrefix = BPELUtil.lookupOrCreateNamespacePrefix(
							fModelObject, targetNamespace, "xsd", shell);
					if (namespacePrefix == null) {
						break;
					}

					builder.append(namespacePrefix).append(":");
				}
				builder.append(att.getName());

			} else if (eObj instanceof XSDElementDeclaration) {

				XSDElementDeclaration elm = (XSDElementDeclaration) eObj;
				targetNamespace = elm.getTargetNamespace();
				int maxOccurs = XSDUtils.getMaxOccurs(elm);

				builder.append("/");
				if (targetNamespace != null) {
//					namespacePrefix = BPELUtils.getNamespacePrefix(process, targetNamespace);
//					if (namespacePrefix == null) {
//						break;
//					}
					builder.append(targetNamespace).append(":");
				}

				builder.append(elm.getName());
				// Unbounded or bounded by something higher then 1.
				if (maxOccurs != 1) {
					builder.append("[1]");
				}
			}

			// If the current builder has length > 0, then there is a query
			// segment for us to put in.
			if (builder.length() > 0) {
				querySegments.add(builder.toString());
			}
		}

		Collections.reverse(querySegments);
		builder.setLength(0);
		for (String s : querySegments) {
			builder.append(s);
		}

		if (builder.length() > 0) {
			builder.deleteCharAt(0);
		}

		try {
			//fChangeHelper.startNonUserChange();
			xpathText.setText(builder.toString());
		} finally {
			//fChangeHelper.finishNonUserChange();
		}
	}
	
	protected void cleanXPathWidgets(){
		xpathText.setText("");
	}
	
	protected Process getProcess(EObject eObject) {
		EObject context = eObject;
		while (context != null) {
			if (context instanceof Process) {
				return (Process)context;
			}
			context = context.eContainer();
		}
		return null;
	}
}
