/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.im.in.bpel.util;

import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import javax.xml.namespace.QName;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.eclipse.bpel.model.Activity;
import org.eclipse.bpel.model.Assign;
import org.eclipse.bpel.model.Catch;
import org.eclipse.bpel.model.CatchAll;
import org.eclipse.bpel.model.Compensate;
import org.eclipse.bpel.model.Copy;
import org.eclipse.bpel.model.Else;
import org.eclipse.bpel.model.ElseIf;
import org.eclipse.bpel.model.Empty;
import org.eclipse.bpel.model.EventHandler;
import org.eclipse.bpel.model.Exit;
import org.eclipse.bpel.model.Expression;
import org.eclipse.bpel.model.ExtensionActivity;
import org.eclipse.bpel.model.FaultHandler;
import org.eclipse.bpel.model.Flow;
import org.eclipse.bpel.model.ForEach;
import org.eclipse.bpel.model.From;
import org.eclipse.bpel.model.If;
import org.eclipse.bpel.model.Invoke;
import org.eclipse.bpel.model.OnAlarm;
import org.eclipse.bpel.model.OnEvent;
import org.eclipse.bpel.model.OnMessage;
import org.eclipse.bpel.model.Pick;
import org.eclipse.bpel.model.Process;
import org.eclipse.bpel.model.Receive;
import org.eclipse.bpel.model.RepeatUntil;
import org.eclipse.bpel.model.Reply;
import org.eclipse.bpel.model.Rethrow;
import org.eclipse.bpel.model.Scope;
import org.eclipse.bpel.model.Sequence;
import org.eclipse.bpel.model.TerminationHandler;
import org.eclipse.bpel.model.Throw;
import org.eclipse.bpel.model.To;
import org.eclipse.bpel.model.Validate;
import org.eclipse.bpel.model.Variable;
import org.eclipse.bpel.model.Variables;
import org.eclipse.bpel.model.Wait;
import org.eclipse.bpel.model.While;
import org.eclipse.bpel.model.adapters.INamespaceMap;
import org.eclipse.bpel.model.messageproperties.Property;
import org.eclipse.bpel.model.partnerlinktype.PartnerLinkType;
import org.eclipse.bpel.model.proxy.IBPELServicesProxy;
import org.eclipse.bpel.model.util.BPELConstants;
import org.eclipse.bpel.model.util.BPELServicesUtility;
import org.eclipse.bpel.model.util.BPELUtils;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.stp.im.BasicProperty;
import org.eclipse.stp.im.ExpressionCondition;
import org.eclipse.stp.im.ExtractDataRule;
import org.eclipse.stp.im.ImFactory;
import org.eclipse.stp.im.ObservableAttribute;
import org.eclipse.stp.im.ProcessCollection;
import org.eclipse.stp.im.Service;
import org.eclipse.stp.im.ServiceBinding;
import org.eclipse.stp.im.ServiceCollection;
import org.eclipse.stp.im.Step;
import org.eclipse.stp.im.StpIntermediateModel;
import org.eclipse.stp.im.Transition;
import org.eclipse.stp.im.TransitionUnderCondition;
import org.eclipse.stp.im.constants.ImConstants;
import org.eclipse.stp.im.runtime.IRuntime;
import org.eclipse.stp.im.runtime.IService;
import org.eclipse.stp.im.runtime.IServiceBinding;
import org.eclipse.stp.im.runtime.ImRuntimeActivator;
import org.eclipse.stp.im.util.ImLogger;
import org.eclipse.wst.wsdl.Definition;
import org.eclipse.wst.wsdl.Operation;
import org.eclipse.xsd.XSDNamedComponent;
import org.spagic3.bpel.externalserviceextension.model.ExternalServiceActivity;
import org.spagic3.bpel.manualactivitiesextension.model.ManualActivity;
import org.spagic3.bpel.relevantdataextension.model.RelevantDataActivity;
import org.spagic3.im.in.bpel2im.Bpel2ImActivator;
import org.w3c.dom.Element;

/**
 * BPELWriter is responsible for serializing the BPEL EMF model to an output
 * stream.
 */

@SuppressWarnings("nls")
public class IntermediateModelGenerator {

	static final String EMPTY_STRING = ""; //$NON-NLS-1$
	static final String prefixStart = "start-";
	static final String prefixEnd = "end-";
	static final String prefixTaskVar = "TaskVar[";
	static final String suffixTaskVar = "]";
	private ImFactory stpModelfactory = null;
	private org.eclipse.stp.im.StpIntermediateModel intermediateModel = null;
	Map<String, Service> imServicesMap = null;
	private Step prevStep = null;
	private ArrayList<String> activityNames = null;
	private Process process = null;
	private boolean stepStartEventExist = false;
	
	public static final String SERVICE_ROUTER = "RouterService";
	public static final String SERVICE_SPLIT = "ParallelSplitService";

	public IntermediateModelGenerator(Process process) {
		this.process = process;
		activityNames = new ArrayList<String>();
	}

	/**
	 * Return the resource used in this writer.
	 * 
	 * @return the resource used in this writer.
	 */

//	public BPELResource getResource() {
//		return fBPELResource;
//	}

	/**
	 * Convert the BPEL model to an StpIntermediateModel
	 * 
	 */
	public StpIntermediateModel generate() throws Exception {
		try {
			stpModelfactory = ImFactory.eINSTANCE;
			intermediateModel  = stpModelfactory.createStpIntermediateModel();
			
			//VERSION
			String currentVersion = "0";
			BasicProperty iterVersionBasicProperty = stpModelfactory.createBasicProperty();
			iterVersionBasicProperty.setKey(ImConstants.ITER_VERSION);
			iterVersionBasicProperty.setValue(currentVersion);
			intermediateModel.getProperties().put(ImConstants.ITER_VERSION, iterVersionBasicProperty);
			
			//ITER AND OBSERVABLE ATTRIBUTES 
			//TODO
			
			//RUNTIME (SERVICE BINDINGS, SERVICE COLLECTION )
			ServiceCollection serviceCollection = stpModelfactory.createServiceCollection();
			
			imServicesMap = new HashMap<String, Service>();
			Map<String, ServiceBinding> imServiceBindingsMap = new HashMap<String, ServiceBinding>();
			
			String runtimeBpelId = "runtime.bpel";
			IRuntime runtime = ImRuntimeActivator.getRuntime(runtimeBpelId);
			Map<String, IServiceBinding> runtimeServiceBinding = runtime.getServiceBindings();
			ServiceBinding aServiceBinding = null;
			for (IServiceBinding serviceBinding : runtimeServiceBinding.values()){
				if (!(imServiceBindingsMap.containsKey(serviceBinding.getName()))){
					aServiceBinding= stpModelfactory.createServiceBinding();
					aServiceBinding.setName(serviceBinding.getName());
					BasicProperty runtimeIdProperty = null;
					runtimeIdProperty = stpModelfactory.createBasicProperty();
					runtimeIdProperty.setKey(ImConstants.IM_STEP_BINDING_RUNTIME_ID);
					runtimeIdProperty.setValue(runtimeBpelId);
					aServiceBinding.getProperties().put(runtimeIdProperty.getKey(),runtimeIdProperty);
					intermediateModel.getServicebindings().add(aServiceBinding);
					imServiceBindingsMap.put(serviceBinding.getName(), aServiceBinding);
				}else{
					ImLogger.warning(Bpel2ImActivator.PLUGIN_ID, " ServiceBinding  [ "+ serviceBinding.getName() + "] already Created in IM");
				}
			}
			
			Service service = null;
			String serviceEntity = null;
			for (IService aService : runtime.getServices()){
				if (!(imServicesMap.containsKey(aService.getName()))){
					serviceEntity = aService.getEmfServiceEntity();
					if (serviceEntity == null)
						service = stpModelfactory.createService();
					else if (serviceEntity.equalsIgnoreCase(ImConstants.SERVICE_SPLIT_CONTROL))
						service = stpModelfactory.createSplitControl();
					else if (serviceEntity.equalsIgnoreCase(ImConstants.SERVICE_JOIN_CONTROL))
						service = stpModelfactory.createJoinControl();
					else if (serviceEntity.equalsIgnoreCase(ImConstants.SERVICE_ROUTER_CONTROL))
						service = stpModelfactory.createRouterControl();
					else if (serviceEntity.equalsIgnoreCase(ImConstants.SERVICE_ITERATION_CONTROL))
						service = stpModelfactory.createIterationControl();
					else service = stpModelfactory.createService();
					service.setServiceName(aService.getName());
					service.setServiceType("BPMN-Service");
					for (String bindingNameForService : aService.getServiceBindingsName()){
						service.getBindings().add(imServiceBindingsMap.get(bindingNameForService));
					}
					
					serviceCollection.getServices().add(service);
					imServicesMap.put(aService.getName(), service);
				} else {
					ImLogger.warning(Bpel2ImActivator.PLUGIN_ID, " Service [ "+ aService.getName() + "] already Created in IM");
				}
			}
			intermediateModel.setServiceCollection(serviceCollection);

			//PROCESSES COLLECTION
			ProcessCollection processCollection = stpModelfactory.createProcessCollection();
			org.eclipse.stp.im.Process imProcess = process2IM(process, currentVersion);
			processCollection.getProcesses().add(imProcess);
			intermediateModel.setProcessCollection(processCollection);
		} catch (Exception ex) {
			ex.printStackTrace();
			throw ex;
		} finally {
			return intermediateModel;
		}
	}

	protected org.eclipse.stp.im.Process process2IM(Process process, String version) {
		//PROCESS
		org.eclipse.stp.im.Process imProcess = stpModelfactory.createProcess();
		imProcess.setName(process.getName());
		BasicProperty versionBasicProperty = stpModelfactory.createBasicProperty();
		versionBasicProperty.setKey(ImConstants.PROCESS_VERSION);
		versionBasicProperty.setValue(version);
		imProcess.getProperties().put(ImConstants.PROCESS_VERSION, versionBasicProperty);

		BasicProperty namespaceBasicProperty = stpModelfactory.createBasicProperty();
		namespaceBasicProperty.setKey(ImConstants.PROPERTY_NAMESPACE);
		namespaceBasicProperty.setValue(process.getTargetNamespace());
		imProcess.getProperties().put(ImConstants.PROPERTY_NAMESPACE, namespaceBasicProperty);

		String runtimeBpelId = "runtime.bpel";
		BasicProperty runtimeIdProperty = null;
		runtimeIdProperty = stpModelfactory.createBasicProperty();
		runtimeIdProperty.setKey(ImConstants.IM_POOL_RUNTIME_ID);
		runtimeIdProperty.setValue(runtimeBpelId);
		imProcess.getProperties().put(ImConstants.IM_POOL_RUNTIME_ID, runtimeIdProperty);
		
		//SWIMLANE 
		//TODO
		
		//VARIABLES (DATA OBJECTS WITH NO ANNOTATION AND NAME SETTED)
		if (process.getVariables() != null
				&& !process.getVariables().getChildren().isEmpty())
			imProcess.getProcessVariables().addAll(variables2IM(process.getVariables(), imProcess));
		
		FaultHandler faultHandler = null;
		if (process.getFaultHandlers() != null)
			faultHandler = process.getFaultHandlers();
		
		EventHandler eventHandler = null;
		if (process.getEventHandlers() != null)
			eventHandler = process.getEventHandlers();
		
		//STEP
		Step step = null;
		if (process.getActivity() != null) {
			//if process is composed by one step we don't have to create the transitions
			//in the other cases, we have a sequence (and the method sequence2IM called, creates the transiction)
			step = activity2IM(process.getActivity(), faultHandler, eventHandler, imProcess, null);
			
			if (step != null)
				imProcess.getSteps().add(step);
		}
		
		return imProcess;
	}

	protected Transition createIMTransition(Step sourceStep, Step targetStep, ExpressionCondition condition){
		Transition transition = null;
			
		if (condition != null)
			transition = stpModelfactory.createTransitionUnderCondition();
		else
			transition = stpModelfactory.createTransition();
				
		transition.setSource(sourceStep);
		transition.setTarget(targetStep);
				
		if (condition != null)
			((TransitionUnderCondition)transition).setCondition(condition);
			
		return transition;
	}
	
	protected void createTransition(org.eclipse.stp.im.Process imProcess, Step sourceStep, Step targetStep, ExpressionCondition condition){
		if (sourceStep != null) {
			Transition transition = createIMTransition(sourceStep, targetStep, condition);
    		if (transition != null) {
    			sourceStep.getSourceTransitions().add(transition);
	    		targetStep.getTargetTransitions().add(transition);
    			imProcess.getTransitions().add(transition);
    		}
		}
	}
	
	protected QName getQName(EObject object) {
		QName qname = null;

		if (object.eIsProxy() && object instanceof IBPELServicesProxy) {
			qname = ((IBPELServicesProxy) object).getQName();
		} else if (object instanceof PartnerLinkType) {
			qname = BPELServicesUtility.getQName((PartnerLinkType) object);
		} else if (object instanceof Property) {
			qname = BPELServicesUtility.getQName((Property) object);
		}

		return qname;
	}

	protected String getNamespace(EObject object) {
		String namespace = null;
		if (object instanceof IBPELServicesProxy) {
			return ((IBPELServicesProxy) object).getQName().getNamespaceURI();
		} else if (object instanceof PartnerLinkType
				|| object instanceof Property) {
			Definition def = ((org.eclipse.wst.wsdl.ExtensibilityElement) object)
					.getEnclosingDefinition();
			if (def != null) {
				namespace = def.getTargetNamespace();
			}
		} else if (object instanceof XSDNamedComponent) {
			return ((XSDNamedComponent) object).getTargetNamespace();
		} else {
			for (Iterator<?> featureIt = object.eClass().getEAllAttributes()
					.iterator(); featureIt.hasNext() && namespace == null;) {
				EAttribute attr = (EAttribute) featureIt.next();
				if (attr.getName().equals("qName")) {
					QName qName = (QName) object.eGet(attr);
					if (qName != null)
						namespace = qName.getNamespaceURI();
				}
			}
		}
		return namespace;
	}

	protected String getOperationSignature(Operation op) {
		String signature = "";
		if (op != null) {
			signature = op.getName();
		}
		return signature;
	}


	protected Collection<org.eclipse.stp.im.Variable> variables2IM(Variables variables, org.eclipse.stp.im.Process imProcess) {
		Collection<org.eclipse.stp.im.Variable> imVariables = new ArrayList<org.eclipse.stp.im.Variable>(); 

		for (Object next : variables.getChildren()) {
			Variable variable = (Variable) next;
			imVariables.add(variable2IM(variable, imProcess));
		}

		return imVariables;
	}

	protected org.eclipse.stp.im.Variable variable2IM(Variable variable, org.eclipse.stp.im.Process imProcess) {
		org.eclipse.stp.im.Variable imVar = ImFactory.eINSTANCE.createVariable();
		String varSchema = "";
		String varName = null;
		try{
		if (variable.getName() != null) {
			varName = variable.getName();
			imVar.setName(varName);
			Element element = null;
			if (variable.getType() != null){
				varSchema = variable.getType().getQName();
			} else if (variable.getMessageType() != null){
				element = variable.getMessageType().getElement();
			} else if (variable.getElement() != null){
				element = variable.getElement();
				//TODO
			}
			
			if (element != null) {
				TransformerFactory transFactory = TransformerFactory.newInstance();
				Transformer transformer = transFactory.newTransformer();
				StringWriter buffer = new StringWriter();
				transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
				transformer.transform(new DOMSource(element),
				      new StreamResult(buffer));
				varSchema = buffer.toString();
			}
			BasicProperty varSchemaBasicProperty = stpModelfactory.createBasicProperty();
			varSchemaBasicProperty.setKey("Schema[" + varName + "]");
			varSchemaBasicProperty.setValue(varSchema);
			imProcess.getProperties().put("Schema[" + varName + "]", varSchemaBasicProperty);
		}
		}catch(Exception ex){
			
		}
	
		return imVar;
	}

//
//	protected Element messageExchanges2XML(MessageExchanges messageExchanges) {
//		// If there are no messageExchanges then skip creating Element
//		if (messageExchanges.getChildren().isEmpty())
//			return null;
//
//		Element messageExchangesElement = createIMStep("messageExchanges");
//
//		for (Object next : messageExchanges.getChildren()) {
//			messageExchangesElement
//					.appendChild(messageExchange2XML((MessageExchange) next));
//		}
//
//		// serialize local namespace prefixes to XML
//		serializePrefixes(messageExchanges, messageExchangesElement);
//		extensibleElement2XML(messageExchanges, messageExchangesElement);
//
//		return messageExchangesElement;
//	}
//
//	protected Element messageExchange2XML(MessageExchange messageExchange) {
//		Element messageExchangeElement = createIMStep("messageExchange");
//
//		if (messageExchange.getName() != null)
//			messageExchangeElement.setAttribute("name", messageExchange
//					.getName());
//
//		// serialize local namespace prefixes to XML
//		serializePrefixes(messageExchange, messageExchangeElement);
//		extensibleElement2XML(messageExchange, messageExchangeElement);
//
//		return messageExchangeElement;
//	}
//
//	protected Element fromParts2XML(FromParts fromParts) {
//		Element fromPartsElement = createIMStep("fromParts");
//
//		for (Object next : fromParts.getChildren()) {
//			FromPart fromPart = (FromPart) next;
//			fromPartsElement.appendChild(fromPart2XML(fromPart));
//		}
//
//		extensibleElement2XML(fromParts, fromPartsElement);
//
//		// serialize local namespace prefixes to XML
//		serializePrefixes(fromParts, fromPartsElement);
//		extensibleElement2XML(fromParts, fromPartsElement);
//
//		return fromPartsElement;
//	}
//
//	protected Element toParts2XML(ToParts toParts) {
//		Element toPartsElement = createIMStep("toParts");
//
//		for (Object next : toParts.getChildren()) {
//			ToPart toPart = (ToPart) next;
//			toPartsElement.appendChild(toPart2XML(toPart));
//		}
//
//		extensibleElement2XML(toParts, toPartsElement);
//
//		// serialize local namespace prefixes to XML
//		serializePrefixes(toParts, toPartsElement);
//		extensibleElement2XML(toParts, toPartsElement);
//
//		return toPartsElement;
//	}
//
//	protected Element correlations2XML(Correlations correlations) {
//		Element correlationsElement = createIMStep("correlations");
//
//		Iterator<?> it = correlations.getChildren().iterator();
//		while (it.hasNext()) {
//			Correlation correlation = (Correlation) it.next();
//			correlationsElement.appendChild(correlation2XML(correlation));
//		}
//
//		// serialize local namespace prefixes to XML
//		serializePrefixes(correlations, correlationsElement);
//		extensibleElement2XML(correlations, correlationsElement);
//
//		return correlationsElement;
//	}
//
//	protected Element correlation2XML(Correlation correlation) {
//		Element correlationElement = createIMStep("correlation");
//
//		if (correlation.getSet() != null
//				&& correlation.getSet().getName() != null)
//			correlationElement.setAttribute("set", correlation.getSet()
//					.getName());
//
//		if (correlation.isSetInitiate())
//			correlationElement.setAttribute("initiate", correlation
//					.getInitiate());
//
//		if (correlation.isSetPattern())
//			correlationElement.setAttribute("pattern", correlation.getPattern()
//					.toString());
//
//		// serialize local namespace prefixes to XML
//		serializePrefixes(correlation, correlationElement);
//		extensibleElement2XML(correlation, correlationElement);
//
//		return correlationElement;
//	}
//

//	protected Element compensationHandler2XML(
//			CompensationHandler compensationHandler) {
//		Element compensationHandlerElement = createIMStep("compensationHandler");
//
//		if (compensationHandler.getActivity() != null) {
//			Element activityElement = activity2XML(compensationHandler
//					.getActivity());
//			compensationHandlerElement.appendChild(activityElement);
//		}
//
//		// serialize local namespace prefixes to XML
//		serializePrefixes(compensationHandler, compensationHandlerElement);
//		extensibleElement2XML(compensationHandler, compensationHandlerElement);
//		return compensationHandlerElement;
//	}
//
	protected void terminationHandler2IM(
			TerminationHandler terminationHandler, org.eclipse.stp.im.Process imProcess, Step endScope) {
		Step terminationHandlerStep = createIMStep("terminationHandler");
		createTransition(imProcess, prevStep, terminationHandlerStep, null);
		imProcess.getSteps().add(terminationHandlerStep);
		prevStep = terminationHandlerStep;

		if (terminationHandler.getActivity() != null) {
			Step step = activity2IM(terminationHandler.getActivity(), imProcess, null);
			if (prevStep != null && step != null) {
				createTransition(imProcess, prevStep, step, null);
				createTransition(imProcess, step, endScope, null);
	        } else //in the sequence the prevStep is the last step read
	        	createTransition(imProcess, prevStep, endScope, null);
			
			if (step != null)
				imProcess.getSteps().add(step);
		}
			
		return;
	}

	protected void eventHandler2IM(EventHandler eventHandler, org.eclipse.stp.im.Process imProcess, Step endSequence) {
		Step tempPrevStep = prevStep;
		for (Iterator<?> it = eventHandler.getEvents().iterator(); it.hasNext();) {
			OnEvent onEvent = (OnEvent) it.next();
			onEvent2IM(onEvent, imProcess, endSequence);
		}
		prevStep = tempPrevStep;
		for (Iterator<?> it = eventHandler.getAlarm().iterator(); it.hasNext();) {
			OnAlarm onAlarm = (OnAlarm) it.next();
			onAlarm2IM(onAlarm, imProcess, endSequence);
		}
		return;
	}
	
	public Step activity2IM(Activity activity, FaultHandler faultHandler, EventHandler eventHandler, org.eclipse.stp.im.Process imProcess, ExpressionCondition condition) {
		Step step = null;
		if (activity instanceof Sequence)
			sequence2IM((Sequence) activity, faultHandler, eventHandler, imProcess, condition);
		else 
			step = activity2IM(activity, imProcess, condition);
		return step;
	}


	public Step activity2IM(Activity activity, org.eclipse.stp.im.Process imProcess, ExpressionCondition condition) {
		Step step = null;

		if (activity instanceof Empty)
			step = empty2IM((Empty) activity);
		else if (activity instanceof Invoke)
			step = invoke2IM((Invoke) activity);
		else if (activity instanceof Assign)
			step = assign2IM((Assign) activity);
		else if (activity instanceof Reply)
			step = reply2IM((Reply) activity);
		else if (activity instanceof Receive)
			step = receive2IM((Receive) activity);
		else if (activity instanceof Wait)
			step = wait2IM((Wait) activity);
		else if (activity instanceof Throw)
			step = throw2IM((Throw) activity);
		else if (activity instanceof Exit)
			step = exit2IM((Exit) activity);
		else if (activity instanceof Flow)
			flow2IM((Flow) activity, imProcess);
		else if (activity instanceof If)
			if2IM((If) activity, imProcess);
		else if (activity instanceof While)
			while2IM((While) activity, imProcess);
		else if (activity instanceof Sequence)
			sequence2IM((Sequence) activity, null, null, imProcess, condition);
		else if (activity instanceof Pick)
			pick2IM((Pick) activity, imProcess);
		else if (activity instanceof Scope)
			scope2IM((Scope) activity, imProcess);
		else if (activity instanceof Compensate)
			step = compensate2IM((Compensate) activity);
		else if (activity instanceof Rethrow)
			step = rethrow2IM((Rethrow) activity);
		else if (activity instanceof ForEach)
			forEach2IM((ForEach) activity, imProcess);
		else if (activity instanceof RepeatUntil)
			repeatUntil2IM((RepeatUntil) activity, imProcess);
		else if (activity instanceof Validate)
			step = validate2IM((Validate) activity);
		else if (activity instanceof ExtensionActivity)
			step = extensionActivity2IM((ExtensionActivity) activity);

		return step;
	}

	protected Step empty2IM(Empty empty) {
		Step emptyStep = createIMStep(empty);
		return emptyStep;
	}

	protected void forEach2IM(ForEach forEach, org.eclipse.stp.im.Process imProcess) {

		Step startForEachStep = createIMStep(prefixStart + forEach.getName());
		
		if (forEach.getParallel() != null)
			startForEachStep.getProperties().put("parallel", createIMStepProperty("parallel", BPELUtils
					.boolean2XML(forEach.getParallel())));
		if (forEach.getCounterName() != null) {
			startForEachStep.getProperties().put("counterName", createIMStepProperty("counterName", forEach
					.getCounterName().getName()));
		}
		if (forEach.getStartCounterValue() != null) {
			startForEachStep.getProperties().put("startCounterValue", createIMStepProperty("startCounterValue", 
											expression2String(forEach.getStartCounterValue())));
		}
		if (forEach.getFinalCounterValue() != null) {
			startForEachStep.getProperties().put("finalCounterValue", createIMStepProperty("finalCounterValue", 
					expression2String(forEach.getFinalCounterValue())));
		}
		createTransition(imProcess, prevStep, startForEachStep, null);
		prevStep = startForEachStep;
		imProcess.getSteps().add(startForEachStep);
		
		Step endForEachStep = createIMStep(prefixEnd + forEach.getName());
		imProcess.getSteps().add(endForEachStep);

		if (forEach.getActivity() != null) {
			Step step = activity2IM(forEach.getActivity(), imProcess, null);
			if (prevStep != null && step != null) {
				createTransition(imProcess, prevStep, step, null);
				createTransition(imProcess, step, endForEachStep, null);
	        } else //in the sequence the prevStep is the last step read
	        	createTransition(imProcess, prevStep, endForEachStep, null);
			
			if(step!= null)
				imProcess.getSteps().add(step);
		}

		prevStep = endForEachStep;
		return;
	}

//	protected Element completionCondition2XML(
//			CompletionCondition completionCondition) {
//		Element completionConditionElement = createIMStep("completionCondition");
//
//		if (completionCondition.getBranches() != null) {
//			Element branchesElement = branches2XML(completionCondition
//					.getBranches());
//			completionConditionElement.appendChild(branchesElement);
//		}
//
//		return completionConditionElement;
//	}

	protected Step rethrow2IM(Rethrow activity) {
		Step rethrowStep = null;
		if (activity.getName() != null)
			rethrowStep = createIMStep(activity.getName());
		else
			rethrowStep = createIMStep("rethrow");
		return rethrowStep;
	}

	protected Step validate2IM(Validate activity) {
		Step validateStep = createIMStep(activity);

		StringBuilder variablesList = new StringBuilder();
		Iterator<?> variables = activity.getVariables().iterator();
		while (variables.hasNext()) {
			Variable variable = (Variable) variables.next();
			variablesList.append(variable.getName());
			if (variables.hasNext())
				variablesList.append(" ");
		}
		if (variablesList.length() > 0) {
			validateStep.getProperties().put("variables", createIMStepProperty("variables", variablesList.toString()));
		}
		return validateStep;
	}

//	protected Element extensionactivity2IM(ExtensionActivity activity) {
//
//		Element activityElement = createIMStep("" +
//				"");
//
//		String localName = activity.eClass().getName();
//		String namespace = activity.eClass().getEPackage().getNsURI();
//		QName qName = new QName(namespace, localName);
//		BPELActivitySerializer serializer = extensionRegistry
//				.getActivitySerializer(qName);
//		if (serializer != null) {
//			DocumentFragment fragment = document.createDocumentFragment();
//			serializer.marshall(qName, activity, fragment, getProcess(), this);
//			Element child = (Element) fragment.getFirstChild();
//			activityElement.appendChild(child);
//			// Standard attributes
//			addStandardAttributes(child, activity);
//			// Standard elements
//			addStandardElements(child, activity);
//		}
//
//		return activityElement;
//	}

	protected Step invoke2IM(Invoke activity) {

		Step invokeStep = createIMStep(activity);

		if (activity.getPartnerLink() != null)
			if (activity.getPartnerLink() != null)
				invokeStep.getProperties().put("partnerLink", createIMStepProperty("partnerLink", activity
																		.getPartnerLink().getName()));
		if (activity.getPortType() != null)
			invokeStep.getProperties().put("portType", createIMStepProperty("portType", qNameToString(activity,
					activity.getPortType().getQName())));
		
		if (activity.getOperation() != null)
			invokeStep.getProperties().put("operation", createIMStepProperty("operation", getOperationSignature(activity.getOperation())));
		
		if (activity.getInputVariable() != null)
			invokeStep.getProperties().put("inputVariable", createIMStepProperty("inputVariable", activity.getInputVariable().getName()));
		
		if (activity.getOutputVariable() != null)
			invokeStep.getProperties().put("outputVariable", createIMStepProperty("outputVariable", activity.getOutputVariable().getName()));

		return invokeStep;
	}

	protected Step receive2IM(Receive activity) {

		Step receiveStep = createIMStep(activity);
		BasicProperty isStartEventProperty = stpModelfactory.createBasicProperty();
		isStartEventProperty.setKey(ImConstants.IS_STEP_START_EVENT);
		isStartEventProperty.setValue("true");
		receiveStep.getProperties().put(ImConstants.IS_STEP_START_EVENT, isStartEventProperty);
		stepStartEventExist = true;
		
		if (activity.getPartnerLink() != null)
			receiveStep.getProperties().put("partnerLink", createIMStepProperty("partnerLink", activity
																	.getPartnerLink().getName()));
		if (activity.getPortType() != null)
			receiveStep.getProperties().put("portType", createIMStepProperty("portType", qNameToString(activity,
																	activity.getPortType().getQName())));
		if (activity.getOperation() != null)
			receiveStep.getProperties().put("operation", createIMStepProperty("operation", getOperationSignature(activity.getOperation())));

		if (activity.getVariable() != null)
			receiveStep.getProperties().put("variable", createIMStepProperty("variable", activity.getVariable()
																	.getName()));
		if (activity.isSetCreateInstance())
			receiveStep.getProperties().put("createInstance", createIMStepProperty("createInstance", BPELUtils
																			.boolean2XML(activity.getCreateInstance())));
		if (activity.getMessageExchange() != null)
			receiveStep.getProperties().put("messageExchange", createIMStepProperty("messageExchange", activity
																			.getMessageExchange().getName()));

		return receiveStep;
	}

	protected Step reply2IM(Reply activity) {
		Step replyStep = createIMStep(activity);
		BasicProperty isEndEventProperty = stpModelfactory.createBasicProperty();
		isEndEventProperty.setKey(ImConstants.IS_STEP_END_EVENT);
		isEndEventProperty.setValue("true");
		replyStep.getProperties().put(ImConstants.IS_STEP_END_EVENT, isEndEventProperty);
		
		if (activity.getPartnerLink() != null)
			replyStep.getProperties().put("partnerLink", createIMStepProperty("partnerLink", activity
																		.getPartnerLink().getName()));
		if (activity.getPortType() != null)
			replyStep.getProperties().put("portType", createIMStepProperty("portType", qNameToString(activity,
																	activity.getPortType().getQName())));
		if (activity.getOperation() != null)
				replyStep.getProperties().put("operation", createIMStepProperty("operation", getOperationSignature(activity.getOperation())));
		if (activity.getVariable() != null)
			replyStep.getProperties().put("variable", createIMStepProperty("variable", activity.getVariable()
																		.getName()));
		if (activity.getFaultName() != null) 
			replyStep.getProperties().put("faultName", createIMStepProperty("faultName", qNameToString(activity,
																		activity.getFaultName())));
		if (activity.getMessageExchange() != null)
			replyStep.getProperties().put("messageExchange", createIMStepProperty("messageExchange", activity
																			.getMessageExchange().getName()));

		return replyStep;
	}

	protected Step assign2IM(Assign activity) {
		Step assignStep = createIMStep(activity);

		if (activity.getValidate() != null)
			assignStep.getProperties().put("validate", createIMStepProperty("validate", BPELUtils
																			.boolean2XML(activity.getValidate())));
		List<?> copies = activity.getCopy();
		int counter = 0; 
		if (!copies.isEmpty()) {
			for (Iterator<?> i = copies.iterator(); i.hasNext();) {
				counter++;
				Copy copy = (Copy) i.next();
				From from = copy.getFrom();
				String fromValue = "";
				if (from.getVariable() != null)
					fromValue = from.getVariable().getName();
				else if (from.getExpression() != null)
					fromValue = expression2String(from.getExpression());
				else if (from.isSetLiteral() && from.getLiteral() != null && !from.getLiteral().equals(""))
					fromValue = from.getLiteral();
				assignStep.getProperties().put("AssignementFrom_" + counter, createIMStepProperty("AssignementFrom_" + counter, fromValue));
				
				To to = copy.getTo();
				String toValue = "";
				if (to.getVariable() != null)
					toValue = to.getVariable().getName();
				else if (to.getExpression() != null)
					toValue = expression2String(to.getExpression());
				assignStep.getProperties().put("AssignementTo_" + counter, createIMStepProperty("AssignementTo_" + counter, toValue));
			}
			assignStep.getProperties().put("totalAssignements", createIMStepProperty("totalAssignements", Integer.toString(counter)));
		}

		return assignStep;
	}

	protected Step wait2IM(Wait activity) {
		Step waitStep = createIMStep(activity);
		//Duration or Date
		if (activity.getFor() != null) {
			waitStep.getProperties().put("mode", createIMStepProperty("mode", "Duration"));
			waitStep.getProperties().put("value", createIMStepProperty("value",activity.getFor().getBody().toString()));
		}
		if (activity.getUntil() != null) {
			waitStep.getProperties().put("mode", createIMStepProperty("mode", "Date"));
			waitStep.getProperties().put("value", createIMStepProperty("value",activity.getUntil().getBody().toString()));
		}
		return waitStep;
	}

	protected Step throw2IM(Throw activity) {
		Step throwStep = createIMStep(activity);

		if (activity.getFaultVariable() != null
				&& activity.getFaultVariable().getName() != null) {
			throwStep.getProperties().put("faultVariable", createIMStepProperty("faultVariable", activity
																				.getFaultVariable().getName()));
		}
		if (activity.getFaultName() != null) {
			throwStep.getProperties().put("faultName", createIMStepProperty("faultName", qNameToString(activity,
																			activity.getFaultName())));
		}
		return throwStep;
	}

	protected Step exit2IM(Exit activity) {
		Step exitStep = null;
		if (activity.getName() != null && !activity.getName().equals(""))
			createIMStep(activity);
		else
			exitStep = createIMStep("exit");
		return exitStep;
	}

	protected void addActivities(List<?> listOfActivities, org.eclipse.stp.im.Process imProcess, ExpressionCondition condition) {
		Step step = null;
		for (Object next : listOfActivities) {
			step = activity2IM((Activity) next, imProcess, condition);
			if(prevStep != null /*&& !( (Activity) next instanceof Sequence)
								&&	!( (Activity) next instanceof If)
								&&	!( (Activity) next instanceof Flow)*/
								&& step != null) {
				createTransition(imProcess, prevStep, step, condition);
				//condition setted only on the first branch (valide for If)
				condition = null;
			}
			
			if (step != null) {
				prevStep = step;
				imProcess.getSteps().add(step);
			}
		}
	}
	
	protected void addFlowActivities(org.eclipse.stp.im.Process imProcess, List<?> listOfActivities, Step endFlowStep) {
		Step step = null;
		Step startFlow = prevStep;
		for (Object next : listOfActivities) {
			step = activity2IM((Activity) next, imProcess, null);
			if (prevStep != null && step != null) {
				createTransition(imProcess, prevStep, step, null);
				createTransition(imProcess, step, endFlowStep, null);
	        } else //in the sequence the prevStep is the last step read
	        	createTransition(imProcess, prevStep, endFlowStep, null);
			
			//reset the prev step that could be changed if the step was a sequence
			prevStep = startFlow;
			if (step != null)
				imProcess.getSteps().add(step);
		}
	}
	
	protected void flow2IM(Flow activity, org.eclipse.stp.im.Process imProcess) {
		//created an exclusive gateway step starting if
		String activityName = activity.getName();
		String uniqueName = getUniqueActivityName(activityName);
		Step startFlowStep = createIMStep(prefixStart + uniqueName);
		startFlowStep.getProperties().put("originalName", createIMStepProperty("originalName", prefixStart +activityName ));
		activity.setName(uniqueName);
		//Step startFlowStep = createIMStep(prefixStart + activity.getName());
		BasicProperty isParallelGateway = stpModelfactory.createBasicProperty();
		isParallelGateway.setKey(ImConstants.IS_PARALLEL_GATEWAY);
		isParallelGateway.setValue("true");
		startFlowStep.getProperties().put(ImConstants.IS_PARALLEL_GATEWAY, isParallelGateway);
		startFlowStep.setServiceModel(imServicesMap.get(SERVICE_SPLIT));
		createTransition(imProcess, prevStep, startFlowStep, null);
		imProcess.getSteps().add(startFlowStep);
		
		//created an exclusive gateway step ending the if
		Step endFlowStep = createIMStep(prefixEnd + uniqueName);
		endFlowStep.getProperties().put("originalName", createIMStepProperty("originalName", prefixEnd +activityName));
		isParallelGateway = stpModelfactory.createBasicProperty();
		isParallelGateway.setKey(ImConstants.IS_PARALLEL_GATEWAY);
		isParallelGateway.setValue("true");
		endFlowStep.getProperties().put(ImConstants.IS_PARALLEL_GATEWAY, isParallelGateway);
		endFlowStep.setServiceModel(imServicesMap.get(SERVICE_SPLIT));
		imProcess.getSteps().add(endFlowStep);
		
		prevStep = startFlowStep;
		addFlowActivities(imProcess, activity.getActivities(), endFlowStep);

		prevStep = endFlowStep;
		return;
	}

	protected void if2IM(If activity, org.eclipse.stp.im.Process imProcess) {
		//created an exclusive gateway step starting if
		String activityName = activity.getName();
		String uniqueName = getUniqueActivityName(activityName);
		Step startIfStep = createIMStep(prefixStart + uniqueName);
		startIfStep.getProperties().put("originalName", createIMStepProperty("originalName", prefixStart + activityName));
		activity.setName(uniqueName);
		
		BasicProperty isExclusiveGateway = stpModelfactory.createBasicProperty();
		isExclusiveGateway.setKey(ImConstants.IS_EXCLUSIVE_GATEWAY);
		isExclusiveGateway.setValue("true");
		startIfStep.getProperties().put(ImConstants.IS_EXCLUSIVE_GATEWAY, isExclusiveGateway);
		startIfStep.setServiceModel(imServicesMap.get(SERVICE_ROUTER));
		createTransition(imProcess, prevStep, startIfStep, null);
		prevStep = startIfStep;
		imProcess.getSteps().add(startIfStep);
		
		//created an exclusive gateway step ending the if
		Step endIfStep = createIMStep(prefixEnd + uniqueName);
		endIfStep.getProperties().put("originalName", createIMStepProperty("originalName", prefixEnd + activityName));
		isExclusiveGateway = stpModelfactory.createBasicProperty();
		isExclusiveGateway.setKey(ImConstants.IS_EXCLUSIVE_GATEWAY);
		isExclusiveGateway.setValue("true");
		endIfStep.getProperties().put(ImConstants.IS_EXCLUSIVE_GATEWAY, isExclusiveGateway);
		endIfStep.setServiceModel(imServicesMap.get(SERVICE_ROUTER));
		imProcess.getSteps().add(endIfStep);
		
		//created if
		Step ifStep = null;
		ExpressionCondition condition = null;
		if (activity.getCondition() != null) {
			condition = stpModelfactory.createExpressionCondition();
			condition.setExpressionLanguage("XPath");
			condition.setExpression(expression2String(activity.getCondition()));
		}

		if (activity.getActivity() != null) {
			ifStep = activity2IM(activity.getActivity(), imProcess, condition);
			if (prevStep != null && ifStep != null) {
				createTransition(imProcess, prevStep, ifStep, condition);
				createTransition(imProcess, ifStep, endIfStep, null);
	        } else 
	        	createTransition(imProcess, prevStep, endIfStep, null);
			
			if(ifStep!= null)
				imProcess.getSteps().add(ifStep);
		}
		//reset the prev step that could be changed if the ifStep was a sequence
		prevStep = startIfStep;
		
		//created the n-elseif
		List<?> elseIfs = activity.getElseIf();
		if (!elseIfs.isEmpty()) {
			for (Object next : elseIfs) {
				ElseIf elseIf = (ElseIf) next;
				Step elseIfStep = elseIf2IM(elseIf, imProcess, endIfStep);
				//reset the prev step that could be changed if the ifStep was a sequence
				prevStep = startIfStep;
			}
		}
		
		//reset the prev step that could be changed if the ifStep was a sequence
		prevStep = startIfStep;
		//created the else
		Else _else = activity.getElse();
		Step elseStep = null;
		if (_else != null) {
			elseStep = else2IM(_else, imProcess, endIfStep);
		}

		prevStep = endIfStep;
		return;
	}

	protected Step elseIf2IM(ElseIf elseIf, org.eclipse.stp.im.Process imProcess, Step endIfStep) {
		Step elseIfStep = null;
		if (elseIf.getActivity() != null) {
			ExpressionCondition condition = null;
			if (elseIf.getCondition() != null) {
				condition = stpModelfactory.createExpressionCondition();
				condition.setExpressionLanguage("XPath");
				condition.setExpression(expression2String(elseIf.getCondition()));
			}
			elseIfStep = activity2IM(elseIf.getActivity(), imProcess, condition);
			if (prevStep != null && elseIfStep != null) {
				createTransition(imProcess, prevStep, elseIfStep, condition);
				createTransition(imProcess, elseIfStep, endIfStep, null);
	        } else //in the sequence the prevStep is the last step read
	        	createTransition(imProcess, prevStep, endIfStep, null);
			
			//prevStep = elseIfStep;
			if (elseIfStep != null)
				imProcess.getSteps().add(elseIfStep);
		}

		return elseIfStep;
	}

	protected Step else2IM(Else _else, org.eclipse.stp.im.Process imProcess, Step endIfStep) {
		Step elseStep = null;
		if (_else.getActivity() != null) {
			elseStep = activity2IM(_else.getActivity(), imProcess, null);
			if (prevStep != null && elseStep != null) {
				createTransition(imProcess, prevStep, elseStep, null);
				createTransition(imProcess, elseStep, endIfStep, null);
	        } else //in the sequence the prevStep is the last step read
	        	createTransition(imProcess, prevStep, endIfStep, null);
			
			if (elseStep != null)
				imProcess.getSteps().add(elseStep);
		}
		return elseStep;
	}

	protected void while2IM(While activity, org.eclipse.stp.im.Process imProcess) {
		loop2IM(activity, imProcess, false);
	}
	protected void loop2IM(Activity activity, org.eclipse.stp.im.Process imProcess, boolean executeAtLeastOnce) {
		String activityName = activity.getName();
		String uniqueName = getUniqueActivityName(activityName);
		Step loopStartStep = createIMStep(prefixStart + uniqueName);
		loopStartStep.getProperties().put("originalName", createIMStepProperty("originalName", prefixStart + activityName));
		activity.setName(uniqueName);
		
		While whileActivity = null;
		RepeatUntil repeatActivity = null;
		if (executeAtLeastOnce){
			repeatActivity = (RepeatUntil)activity;
			loopStartStep.getProperties().put("ExecuteAtLeastOnce", createIMStepProperty("ExecuteAtLeastOnce", "true"));
			loopStartStep.getProperties().put("XPathCondition", createIMStepProperty("XPathCondition", expression2String(repeatActivity.getCondition())));
		} else {
			whileActivity = (While)activity;
			loopStartStep.getProperties().put("ExecuteAtLeastOnce", createIMStepProperty("ExecuteAtLeastOnce", "false"));
			loopStartStep.getProperties().put("XPathCondition", createIMStepProperty("XPathCondition", expression2String(((While)activity).getCondition())));
		}
		createTransition(imProcess, prevStep, loopStartStep, null);
		prevStep = loopStartStep;
		imProcess.getSteps().add(loopStartStep);
		
		Step loopEndStep = createIMStep(prefixEnd + uniqueName);
		loopEndStep.getProperties().put("originalName", createIMStepProperty("originalName", prefixEnd + activityName));
		imProcess.getSteps().add(loopEndStep);

		Step step = null;
		Activity loopActivity = null;
		if (whileActivity != null && whileActivity.getActivity() != null)
			loopActivity = whileActivity.getActivity();
		else if (repeatActivity != null && repeatActivity.getActivity() != null) 
			loopActivity = repeatActivity.getActivity();
		if (loopActivity != null){
			step = activity2IM(loopActivity, imProcess, null);
			if (prevStep != null && step != null) {
				createTransition(imProcess, prevStep, step, null);
				createTransition(imProcess, step, loopEndStep, null);
	        } else 
	        	createTransition(imProcess, prevStep, loopEndStep, null);
			if(step != null)
				imProcess.getSteps().add(step);
		}
		
		prevStep = loopEndStep;
		return;
	}

	protected void repeatUntil2IM(RepeatUntil activity, org.eclipse.stp.im.Process imProcess) {
		loop2IM(activity, imProcess, true);
	}

	protected String expression2String(Expression expression) {
		String expression2String = "";
		if (expression.getBody() != null) 
			expression2String = expression.getBody().toString();
		
		return expression2String;
	}

	protected void sequence2IM(Sequence activity, FaultHandler faultHandler, EventHandler eventHandler, org.eclipse.stp.im.Process imProcess, ExpressionCondition condition) {
		//sequence activity isn't mapped to a IM step 
		//except when there's an FaultHandler associated to the process
		Step startSeqStep = null;
		Step endSeqStep = null;
		if (faultHandler != null || eventHandler != null){
			String activityName = activity.getName();
			String uniqueName = getUniqueActivityName(activityName);
			startSeqStep = createIMStep(prefixStart + uniqueName);
			startSeqStep.getProperties().put("originalName", createIMStepProperty("originalName", prefixStart + activityName));
			activity.setName(uniqueName);
			createTransition(imProcess, prevStep,startSeqStep, null);
			prevStep = startSeqStep;
			imProcess.getSteps().add(startSeqStep);
			
			//created an step ending the scope
			endSeqStep = createIMStep(prefixEnd + uniqueName);
			endSeqStep.getProperties().put("originalName", createIMStepProperty("originalName", prefixEnd + activityName));
			imProcess.getSteps().add(endSeqStep);
			imProcess.getSteps().add(endSeqStep);
		}
		
		addActivities(activity.getActivities(), imProcess, condition);
		
        if (endSeqStep != null)
        	createTransition(imProcess, prevStep, endSeqStep, null);
		
		
		if (faultHandler != null) {
			prevStep = startSeqStep;
			faultHandlers2IM(faultHandler, imProcess, endSeqStep);
		}
		
		if (eventHandler != null) {
			prevStep = startSeqStep;
			eventHandler2IM(eventHandler, imProcess, endSeqStep);
		}
		
		if (endSeqStep != null)
			prevStep = endSeqStep;
		return;
	}

	protected void onMessage2IM(OnMessage onMsg, org.eclipse.stp.im.Process imProcess, Step endPickStep) {
		String onMessageStepName = "onMessage" + ((onMsg.getOperation() != null) ? getOperationSignature(onMsg.getOperation()) : "");
		Step onMessageStep = createIMStep(onMessageStepName);
		onMessageStep.getProperties().put("originalName", createIMStepProperty("originalName", onMessageStepName));
		if (onMsg.getPartnerLink() != null
				&& onMsg.getPartnerLink().getName() != null) 
			onMessageStep.getProperties().put("partnerLink", createIMStepProperty("partnerLink", onMsg.getPartnerLink()
															.getName()));
		if (onMsg.getPortType() != null
				&& onMsg.getPortType().getQName() != null) 
			onMessageStep.getProperties().put("portType", createIMStepProperty("portType", qNameToString(onMsg,
															onMsg.getPortType().getQName())));
		if (onMsg.getOperation() != null) 
			onMessageStep.getProperties().put("operation", createIMStepProperty("operation", getOperationSignature(onMsg.getOperation())));
		
		if (onMsg.getVariable() != null
				&& onMsg.getVariable().getName() != null) 
			onMessageStep.getProperties().put("variable", createIMStepProperty("variable", onMsg.getVariable()
															.getName()));
		if (onMsg.getMessageExchange() != null)
			onMessageStep.getProperties().put("messageExchange", createIMStepProperty("messageExchange", onMsg
					.getMessageExchange().getName()));
		
		createTransition(imProcess, prevStep, onMessageStep, null);
		imProcess.getSteps().add(onMessageStep);
		prevStep = onMessageStep;
		
		Activity activity = onMsg.getActivity();
		if (activity != null){
			Step step = activity2IM(activity, imProcess, null); 
			if (prevStep != null && step != null) {
				createTransition(imProcess, prevStep, step, null);
				createTransition(imProcess, step, endPickStep, null);
	        } else //in the sequence the prevStep is the last step read
	        	createTransition(imProcess, prevStep, endPickStep, null);
			
			if (step != null)
				imProcess.getSteps().add(step);
		}

		return;
	}

	protected void onEvent2IM(OnEvent onEvent, org.eclipse.stp.im.Process imProcess, Step endScope) {
		String onEventStepName = "onEvent" + ((onEvent.getOperation() != null) ? getOperationSignature(onEvent.getOperation()) : "");
		Step onEventStep = createIMStep(onEventStepName);
		onEventStep.getProperties().put("originalName", createIMStepProperty("originalName", onEventStepName));
		if (onEvent.getPartnerLink() != null
				&& onEvent.getPartnerLink().getName() != null) {
			onEventStep.getProperties().put("partnerLink", createIMStepProperty("partnerLink", onEvent.getPartnerLink()
											.getName()));
		}
		if (onEvent.getPortType() != null
				&& onEvent.getPortType().getQName() != null) {
			onEventStep.getProperties().put("portType", createIMStepProperty("portType", qNameToString(onEvent,
					onEvent.getPortType().getQName())));
		}
		if (onEvent.getOperation() != null) {
			onEventStep.getProperties().put("operation", createIMStepProperty("operation", getOperationSignature(onEvent.getOperation())));
		}
		if (onEvent.getVariable() != null
				&& onEvent.getVariable().getName() != null) {
			onEventStep.getProperties().put("variable", createIMStepProperty("variable", onEvent.getVariable()
					.getName()));
		}
		if (onEvent.getMessageExchange() != null)
			onEventStep.getProperties().put("messageExchange", createIMStepProperty("messageExchange", onEvent
					.getMessageExchange().getName()));
			
		if (onEvent.getMessageType() != null) {
			onEventStep.getProperties().put("messageType", createIMStepProperty("messageType", qNameToString(onEvent,
					onEvent.getMessageType().getQName())));
		}

		createTransition(imProcess, prevStep, onEventStep, null);
		imProcess.getSteps().add(onEventStep);
		prevStep = onEventStep;
		
		Activity activity = onEvent.getActivity();
		if (activity != null){
			Step step = activity2IM(activity, imProcess, null); 
			if (prevStep != null && step != null) {
				createTransition(imProcess, prevStep, step, null);
				createTransition(imProcess, step, endScope, null);
	        } else //in the sequence the prevStep is the last step read
	        	createTransition(imProcess, prevStep, endScope, null);
			
			if (step != null)
				imProcess.getSteps().add(step);
		}

		return;
	}

	protected void onAlarm2IM(OnAlarm onAlarm, org.eclipse.stp.im.Process imProcess, Step endScope) {
		Step onAlarmStep = createIMStep("onAlarm");
		if (onAlarm.getFor() != null) {
			onAlarmStep.getProperties().put("for", createIMStepProperty("for", expression2String(onAlarm.getFor())));
		}
		if (onAlarm.getUntil() != null) {
			onAlarmStep.getProperties().put("until", createIMStepProperty("until", expression2String(onAlarm.getUntil())));
		}
		if (onAlarm.getRepeatEvery() != null) {
			onAlarmStep.getProperties().put("repeatEvery", createIMStepProperty("repeatEvery", expression2String(onAlarm.getRepeatEvery())));
		}
		createTransition(imProcess, prevStep, onAlarmStep, null);
		imProcess.getSteps().add(onAlarmStep);
		prevStep = onAlarmStep;
		
		Activity activity = onAlarm.getActivity();
		if (activity != null){
			Step step = activity2IM(activity, imProcess, null); 
			if (prevStep != null && step != null) {
				createTransition(imProcess, prevStep, step, null);
				createTransition(imProcess, step, endScope, null);
	        } else //in the sequence the prevStep is the last step read
	        	createTransition(imProcess, prevStep, endScope, null);
			
			if (step != null)
				imProcess.getSteps().add(step);
		}

		return;
	}

	protected void pick2IM(Pick activity, org.eclipse.stp.im.Process imProcess) {
		String activityName = activity.getName();
		String uniqueName = getUniqueActivityName(activityName);
		Step startPickStep = createIMStep(prefixStart + uniqueName);
		startPickStep.getProperties().put("originalName", createIMStepProperty("originalName", prefixStart + activityName));
		activity.setName(uniqueName);

		if (activity.isSetCreateInstance()) {
			startPickStep.getProperties().put("createInstance", createIMStepProperty("createInstance", BPELUtils
					.boolean2XML(activity.getCreateInstance())));
		}
		//if the receive doesn't exist the pick became the start step
		if (!stepStartEventExist){
			BasicProperty isStartEventProperty = stpModelfactory.createBasicProperty();
			isStartEventProperty.setKey(ImConstants.IS_STEP_START_EVENT);
			isStartEventProperty.setValue("true");
			startPickStep.getProperties().put(ImConstants.IS_STEP_START_EVENT, isStartEventProperty);
		}
		createTransition(imProcess, prevStep,startPickStep, null);
		prevStep = startPickStep;
		imProcess.getSteps().add(startPickStep);
		
		//created an step ending the pick
		Step endPickStep = createIMStep(prefixEnd + uniqueName);
		endPickStep.getProperties().put("originalName", createIMStepProperty("originalName", prefixEnd + activityName));
		imProcess.getSteps().add(endPickStep);
		
		for (Object next : activity.getMessages()) {                                                                                                                                             
			onMessage2IM((OnMessage) next, imProcess, endPickStep);
			prevStep = startPickStep;
		}
		
		prevStep = endPickStep;
		return;
	}

	protected void scope2IM(Scope activity, org.eclipse.stp.im.Process imProcess) {
		String scopeActivityName = (activity.getName() != null) ? activity.getName() : "scope";
		Step startScopeStep = null;
		Step endScopeStep = null;
		
		if(activity.getEventHandlers() != null 
				|| activity.getFaultHandlers() != null 
				|| activity.getTerminationHandler() != null
				|| activity.getVariables() != null) {
			String activityName = activity.getName();
			String uniqueName = getUniqueActivityName(activityName);
			startScopeStep = createIMStep(prefixStart + uniqueName);
			
			startScopeStep.getProperties().put("originalName", createIMStepProperty("originalName", prefixStart + activityName));
			activity.setName(uniqueName);
			
			if (activity.isSetIsolated())
				startScopeStep.getProperties().put("isolated", createIMStepProperty("isolated", BPELUtils
					.boolean2XML(activity.getIsolated())));
			if (activity.isSetExitOnStandardFault())
				startScopeStep.getProperties().put("exitOnStandardFault", createIMStepProperty("exitOnStandardFault", BPELUtils
						.boolean2XML(activity.getExitOnStandardFault())));
			if (activity.getVariables() != null && !activity.getVariables().getChildren().isEmpty()){
				for (Object next : activity.getVariables().getChildren()) {
					Variable variable = (Variable) next;
					String varKey = prefixTaskVar + variable.getName() + suffixTaskVar;
					startScopeStep.getProperties().put(varKey, createIMStepProperty(varKey, variable.getName()));
				}
			}
			createTransition(imProcess, prevStep,startScopeStep, null);
			
			prevStep = startScopeStep;
			imProcess.getSteps().add(startScopeStep);
			
			//created an step ending the scope
			endScopeStep = createIMStep(prefixEnd + uniqueName);
			endScopeStep.getProperties().put("originalName", createIMStepProperty("originalName", prefixEnd + activityName));
			imProcess.getSteps().add(endScopeStep);
		}
		
		if (activity.getActivity() != null) {
			Step step = activity2IM(activity.getActivity(), imProcess, null);
			if (prevStep != null && step != null) {
				createTransition(imProcess, prevStep, step, null);
				if (endScopeStep != null)
					createTransition(imProcess, step, endScopeStep, null);
	        } else if (endScopeStep != null)//in the sequence the prevStep is the last step read
	        	createTransition(imProcess, prevStep, endScopeStep, null);
			
			if(step!= null) {
				imProcess.getSteps().add(step);
				prevStep = step;
			}
		}
		//reset the prev step that could be changed if the step was a sequence
		if (startScopeStep != null)
			prevStep = startScopeStep;
		
		if (activity.getFaultHandlers() != null){
			prevStep = startScopeStep;
			faultHandlers2IM(activity.getFaultHandlers(), imProcess, endScopeStep);
		}
	
		
		if (activity.getTerminationHandler() != null){
			prevStep = startScopeStep;
			terminationHandler2IM(activity.getTerminationHandler(), imProcess, endScopeStep);
		}
		
		if (activity.getEventHandlers() != null){
			prevStep = startScopeStep;
			eventHandler2IM(activity.getEventHandlers(), imProcess, endScopeStep);
		}
		
		if (endScopeStep != null)
			prevStep = endScopeStep;
		return;
	}
	
	protected void faultHandlers2IM(FaultHandler faultHandler, org.eclipse.stp.im.Process imProcess, Step endScope) {
		Iterator<?> catches = faultHandler.getCatch().iterator();
		while (catches.hasNext()) {
			Catch _catch = (Catch) catches.next();
			catch2IM(_catch, imProcess, endScope);
		}
		
		if (faultHandler.getCatchAll() != null) 
			catchAll2IM(faultHandler.getCatchAll(), imProcess, endScope);
		
	}
	
	protected void catch2IM(Catch _catch, org.eclipse.stp.im.Process imProcess, Step endScope) {
		Step tempPrevStep = prevStep;
		Step startCatchStep = null;
		if(_catch.getFaultName() != null) {
			startCatchStep = createIMStep(_catch.getFaultName().getLocalPart());
			startCatchStep.getProperties().put("namespace", createIMStepProperty("namespace",_catch
					.getFaultName().getNamespaceURI()));
		} else 
			startCatchStep =  createIMStep("catch");
		
		
		if(_catch.getFaultVariable() != null) {
			startCatchStep.getProperties().put("faultVariable", createIMStepProperty("faultVariable",_catch
				.getFaultVariable().getName()));
		}
		createTransition(imProcess, prevStep, startCatchStep, null);
		prevStep = startCatchStep;
		imProcess.getSteps().add(startCatchStep);
		
		Step catchStep = activity2IM(_catch.getActivity(), imProcess, null);
		if (prevStep != null && catchStep != null) {
			createTransition(imProcess, prevStep, catchStep, null);
			createTransition(imProcess, catchStep, endScope, null);
        } else //in the sequence the prevStep is the last step read
        	createTransition(imProcess, prevStep, endScope, null);
		
		if (catchStep != null)
			imProcess.getSteps().add(catchStep);
	
		prevStep = tempPrevStep;
		return;
	}

	protected void catchAll2IM(CatchAll catchAll, org.eclipse.stp.im.Process imProcess, Step endScope) {
		Step catchAllStep = createIMStep("catchAll");
		createTransition(imProcess, prevStep, catchAllStep, null);
		imProcess.getSteps().add(catchAllStep);
		prevStep = catchAllStep;
		
		Activity activity = catchAll.getActivity();
		if (activity != null){
			Step step = activity2IM(activity, imProcess, null); 
			if (prevStep != null && step != null) {
				createTransition(imProcess, prevStep, step, null);
				createTransition(imProcess, step, endScope, null);
	        } else //in the sequence the prevStep is the last step read
	        	createTransition(imProcess, prevStep, endScope, null);
			
			if (step != null)
				imProcess.getSteps().add(step);
		}

		return;
	}
//
//	protected Element compensateScope2XML(CompensateScope activity) {
//		Element activityElement = createIMStep("compensateScope");
//
//		Activity scopeOrInvoke = activity.getTarget();
//		if (scopeOrInvoke != null) {
//			activityElement.setAttribute("scope", scopeOrInvoke.getName());
//		}
//
//		addCommonActivityItems(activityElement, activity);
//		return activityElement;
//	}

	protected Step compensate2IM(Compensate activity) {
		Step compensateStep = createIMStep(activity);
		return compensateStep;
	}
	
	protected Step extensionActivity2IM(ExtensionActivity activity) {

		Step extensionActivityStep = createIMStep(activity);
		if(activity instanceof ExternalServiceActivity){
			ExternalServiceActivity externalServiceActivity = (ExternalServiceActivity)activity;
			extensionActivityStep.getProperties().put("serviceName", createIMStepProperty("serviceName", externalServiceActivity.getServiceName()));
			extensionActivityStep.getProperties().put("input", createIMStepProperty("input", externalServiceActivity.getInput()));
			extensionActivityStep.getProperties().put("output", createIMStepProperty("output", externalServiceActivity.getOutput()));
		} else if (activity instanceof ManualActivity){
			ManualActivity manualActivity = (ManualActivity)activity;
			extensionActivityStep.getProperties().put("activityName", createIMStepProperty("activityName", manualActivity.getActivityName()));
			extensionActivityStep.getProperties().put("input", createIMStepProperty("input", manualActivity.getInput()));
		} else if (activity instanceof RelevantDataActivity){
			RelevantDataActivity relevantDataActivity = (RelevantDataActivity)activity;
			String attributeName = relevantDataActivity.getAttributeName();
			String variableName = relevantDataActivity.getVariable();
			String xpath = relevantDataActivity.getXpath();
			String ruleName = "rule-"  + attributeName + "-" + UUID.randomUUID();
			
			ObservableAttribute obsAttribute = stpModelfactory.createObservableAttribute();
			ExtractDataRule extractDataRule = stpModelfactory.createExtractDataRule();
			extractDataRule.setName(ruleName);
			extractDataRule.setExpression(xpath);
			extractDataRule.setExpressionLanguage("XPath");
			extractDataRule.setVariableName(variableName);
			//extractDataRule.set
			obsAttribute.setName(attributeName);
			obsAttribute.setObservableAttributeExtractRule(extractDataRule);
			extensionActivityStep.getObservableAttributes().add(obsAttribute);
		}
		return extensionActivityStep;
	}
	
	protected QName getQName(org.eclipse.wst.wsdl.ExtensibilityElement element,
			String localName) {
		EObject container = null;
		for (container = element.eContainer(); container != null
				&& !(container instanceof Definition);) {
			container = container.eContainer();
		}
		if (container == null) {
			return null;
		}
		return new QName(((Definition) container).getTargetNamespace(),
				localName);
	}


	/**
	 * Get process from the resource 
	 * @return the Process
	 */
//	private Process getProcess() {
//		return getResource().getProcess();
//	}

	/**
	 * Convert a BPEL ExtensibilityElement to XML
	 */
//	protected Element extensibilityElement2XML(
//			ExtensibilityElement extensibilityElement) {
//
//		BPELExtensionSerializer serializer = null;
//		QName qname = extensibilityElement.getElementType();
//		try {
//			serializer = (BPELExtensionSerializer) extensionRegistry
//					.querySerializer(ExtensibleElement.class, qname);
//		} catch (WSDLException e) {
//			return null;
//		}
//
//		// Deserialize the DOM element and add the new Extensibility element to
//		// the parent
//		// ExtensibleElement
//		DocumentFragment fragment = document.createDocumentFragment();
//		try {
//			serializer
//					.marshall(ExtensibleElement.class, qname,
//							extensibilityElement, fragment, getProcess(),
//							extensionRegistry, this);
//			return (Element) fragment.getFirstChild();
//		} catch (WSDLException e) {
//			throw new WrappedException(e);
//		}
//	}
	
	protected Step createIMStep(Activity activity) {
		String activityName = activity.getName();
		String uniqueName = getUniqueActivityName(activityName);
		Step step = createIMStep(uniqueName);
		step.getProperties().put("originalName", createIMStepProperty("originalName", activityName));
		activity.setName(uniqueName);
		return step;
	}

	protected Step createIMStep(String tagName) {
		Step step = stpModelfactory.createStep();
		step.setName(tagName);
		return step;
	}
	
	protected String getUniqueActivityName(String activityName){
		int indexDuplicateName = 1;
		if (!activityNames.contains(activityName)) {
			activityNames.add(activityName);
			return activityName;
		} else {
			while (activityNames.contains(activityName + Integer.toString(indexDuplicateName)))
				indexDuplicateName++;
			String newNameActivity = activityName + Integer.toString(indexDuplicateName);
			activityNames.add(newNameActivity);
			return newNameActivity;
		}
	}
	
	protected BasicProperty createIMStepProperty(String key, String value){
		BasicProperty stepProperty = stpModelfactory.createBasicProperty();
		stepProperty.setKey(key);
		stepProperty.setValue(value);
		return stepProperty;
	}
	
	/**
	 * 
	 * @return the namespace prefix manager.
	 */

	// public NamespacePrefixManager getNamespacePrefixManager() {
	// return bpelNamespacePrefixManager;
	// }
//	private void serializePrefixes(EObject eObject, Element context) {
//		INamespaceMap<String, String> nsMap = BPELUtils
//				.getNamespaceMap(eObject);
//		if (!nsMap.isEmpty()) {
//			for (Iterator<String> i = nsMap.keySet().iterator(); i.hasNext();) {
//				String prefix = i.next();
//				String namespace = nsMap.get(prefix);
//				if (prefix == "")
//					context.setAttributeNS(XSDConstants.XMLNS_URI_2000,
//							"xmlns", namespace);
//				else
//					context.setAttributeNS(XSDConstants.XMLNS_URI_2000,
//							"xmlns:" + prefix, namespace);
//			}
//		}
//	}

	private String addNewRootPrefix(String basePrefix, String namespace) {
		INamespaceMap<String, String> nsMap = BPELUtils
				.getNamespaceMap(process);

		List<String> prefixes = nsMap.getReverse(namespace);
		if (prefixes.isEmpty()) {
			int i = 0;
			String prefix = basePrefix;
			while (nsMap.containsKey(prefix)) {
				prefix = basePrefix + i;
				i++;
			}
			nsMap.put(prefix, namespace);
			return prefix;
		} else {
			return prefixes.get(0);
		}
	}

	private String qNameToString(EObject eObject, QName qname) {
		EObject context = eObject;
		List<String> prefixes = null;
		String namespace = qname.getNamespaceURI();

		if (namespace == null || namespace.length() == 0) {
			return qname.getLocalPart();
		}

		// Transform BPEL namespaces to the latest version so that
		// references to the old namespace are not serialized.
		if (BPELConstants.isBPELNamespace(namespace)) {
			namespace = BPELConstants.NAMESPACE;
		}
		while (context != null) {
			INamespaceMap<String, String> prefixNSMap = BPELUtils
					.getNamespaceMap(context);
			prefixes = prefixNSMap.getReverse(namespace);
			if (!prefixes.isEmpty()) {
				String prefix = prefixes.get(0);
				if (!prefix.equals(""))
					return prefix + ":" + qname.getLocalPart();
				else
					return qname.getLocalPart();
			}
			context = context.eContainer();
		}
		// if a prefix is not found for the namespaceURI, create a new
		// prefix
		return addNewRootPrefix("ns", namespace) + ":" + qname.getLocalPart();
	}

}
