/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.im.in.bpel2im;

import java.io.IOException;
import java.util.Collections;

import org.eclipse.bpel.model.Process;
import org.eclipse.bpel.model.resource.BPELResource;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.stp.im.StpIntermediateModel;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.spagic3.im.in.bpel.util.IntermediateModelGenerator;
import org.spagic3.utils.SpagicLogger;

public class BPEL2IMWorkspaceOperation extends WorkspaceModifyOperation {

	private IFile bpelFile = null;
	private IFile imFile = null;
	private Process process = null;
	
	public BPEL2IMWorkspaceOperation(IFile bpelFile){
		this.bpelFile = bpelFile;
	}
	
	@Override
	protected void execute(IProgressMonitor arg0){
		try{
			BPELResource resourceBpel = retrieveBPELResource();
			process = (Process)resourceBpel.getContents().get(0);
			IntermediateModelGenerator imGenerator = new IntermediateModelGenerator(process);
			StpIntermediateModel intermediateModel = imGenerator.generate();
			saveIntermediateModel(intermediateModel);
		} catch (Throwable e) {
			SpagicLogger.error("Exception in IM generation ", e);
			throw new RuntimeException("Problems during IM generation", e);
		}
	}

	private BPELResource retrieveBPELResource(){
		ResourceSet resourceSet = new ResourceSetImpl();
		URI uri = URI.createPlatformResourceURI(bpelFile.getFullPath().toString(),true);
		BPELResource resourceBpel = (BPELResource)resourceSet.getResource(uri, true);
		return resourceBpel;
	}
	
	private void saveIntermediateModel(StpIntermediateModel intermediateModel) throws IOException {
		//write intermediateModel
		String destinationStpFileName = bpelFile.getName();
	    destinationStpFileName = destinationStpFileName.substring(0, destinationStpFileName.indexOf("."));
		destinationStpFileName += ".im";
		IContainer container = bpelFile.getParent();
		imFile = container.getFile(new Path(destinationStpFileName));
		URI destinationStpFileURI = URI.createPlatformResourceURI(imFile.getFullPath().toString(),true);

		// Create a resource for this file.
		ResourceSet resourceSetStpIm = new ResourceSetImpl();
		Resource resourceStpIm = resourceSetStpIm.createResource(destinationStpFileURI);
		resourceStpIm.getContents().add(intermediateModel);
	    resourceStpIm.save(Collections.EMPTY_MAP);
	}
	
	public IFile getImFile() {
		return imFile;
	}

	public void setImFile(IFile imFile) {
		this.imFile = imFile;
	}

	public Process getProcess() {
		return process;
	}
	
}
