/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.im.in.bpel2im;

import java.util.ArrayList;
import java.util.Collections;

import org.eclipse.bpel.model.Activity;
import org.eclipse.bpel.model.Flow;
import org.eclipse.bpel.model.Process;
import org.eclipse.bpel.model.Sequence;
import org.eclipse.bpel.model.resource.BPELResource;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.spagic3.utils.SpagicLogger;

public class ValidateBPELWorkspaceOperation extends WorkspaceModifyOperation {
	
	private IFile bpelFile = null;
	private ArrayList<String> activityNames = null;
	
	public ValidateBPELWorkspaceOperation(IFile bpelFile){
		this.bpelFile = bpelFile;
		activityNames = new ArrayList<String>();
	}
	
	protected void execute(IProgressMonitor arg0){
		try {
			ResourceSet resourceSet = new ResourceSetImpl();
			URI uri = URI.createPlatformResourceURI(bpelFile.getFullPath().toString(),true);
			BPELResource resourceBpel = (BPELResource)resourceSet.getResource(uri, true);
			Process process = (Process)resourceBpel.getContents().get(0);
			
			if (process.getActivity() != null) 
				validateActivity(process.getActivity());
			
			Resource resource = resourceSet.createResource(uri);
			resource.getContents().add(0, process);
		    resource.save(Collections.EMPTY_MAP);
		} catch (Throwable e) {
			SpagicLogger.error("Exception in BPEL validation ", e);
			throw new RuntimeException("Problems during BPEL validation", e);
		}
	}
	
	protected void validateActivity(Activity activity){
		if (activity instanceof Sequence)
			validateSequence(activity);
		else if (activity instanceof Flow)
			validateFlow(activity);
		else validateActivityName(activity);
	}
	
	protected void validateSequence(Activity activity){
		validateActivityName(activity);
		EList<Activity> listOfActivities = ((Sequence)activity).getActivities();
		for (Object next : listOfActivities) {
			validateActivity((Activity) next);
		}
	}
	
	protected void validateFlow(Activity activity){
		validateActivityName(activity);
		EList<Activity> listOfActivities = ((Flow)activity).getActivities();
		for (Object next : listOfActivities) {
			validateActivity((Activity) next);
		}
	}
	
	protected void validateActivityName(Activity activity){
		int indexDuplicateName = 1;
		String activityName = activity.getName();
		if (!activityNames.contains(activityName))
			activityNames.add(activityName);
		else {
			while (activityNames.contains(activityName + Integer.toString(indexDuplicateName)))
				indexDuplicateName++;
			String newNameActivity = activityName + Integer.toString(indexDuplicateName);
			activityNames.add(newNameActivity);
			activity.setName(newNameActivity);
		}
	}
}
