/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.im.out.jbpm.actions;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.jet.JET2Platform;
import org.eclipse.stp.im.ImPackage;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.spagic3.utils.SpagicLogger;
import org.spagic3.utils.SpagicUtilsActivator;
import org.spagic3.utils.preference.SpagicPreferenceConstants;

public class Im2JbpmWorkspaceOperation extends WorkspaceModifyOperation {

	private IFile imFile = null;

	public IFile getImFile() {
		return imFile;
	}

	public void setImFile(IFile imFile) {
		this.imFile = imFile;
	}

	public Im2JbpmWorkspaceOperation(IFile imFile) {
		this.imFile = imFile;
	}

	protected void execute(IProgressMonitor monitor) {
		ResourceSet resourceSetIM = new ResourceSetImpl();
		URI uri = URI.createPlatformResourceURI(imFile.getFullPath().toString(), true);

		// Force Package Registration
		ImPackage imPackage = ImPackage.eINSTANCE;

		// Load On Demand = true
		Resource resourceIM = resourceSetIM.getResource(uri, true);

		Map<String, Object> variables = new HashMap<String, Object>();
		String imFileName = imFile.getName();
		imFileName = imFileName.substring(0, imFileName.lastIndexOf("."));
		variables.put("org.eclipse.jet.resource.project.name", imFile.getProject().getName());
		IWorkspace workspace = ResourcesPlugin.getWorkspace();
		IWorkspaceRoot root = workspace.getRoot();
		variables.put("projectFullPath", root.getLocation() + imFile.getProject().getFullPath().toString());
		variables.put("imFileName", imFileName);
		variables.put("graphVizExeLocation", getGraphVizExeLocation());
		variables.put("graphVizTempFolder", getGraphVizTempFolder());
		variables.put("imFile", imFile);

		try {
			IStatus status = JET2Platform.runTransformOnObject("org.spagic3.im.out.jbpm", resourceIM, variables,
					new NullProgressMonitor());
			if (!status.isOK()) {
				System.out.println(status.getMessage());
				if (status.getException() != null)
					status.getException().printStackTrace(System.out);
				System.out.println(status.toString());
			}
			imFile.getProject().refreshLocal(IProject.DEPTH_INFINITE, null);

		} catch (Throwable e) {
			SpagicLogger.error(e.getMessage(), e);
			throw new RuntimeException(" Error Generating JPDL From IM", e);
		} finally {
			if (monitor != null)
				monitor.done();
		}
	}

	/**
	 * @return The location of the Graphviz executable.
	 */
	public String getGraphVizExeLocation() {
		org.eclipse.core.runtime.Preferences prefs = SpagicUtilsActivator.getDefault().getPluginPreferences();
		return prefs.getString(SpagicPreferenceConstants.P_GRAPH_VIZ_DOT_EXE);
	}

	/**
	 * @return The location of the temporary folder used for generating graphs.
	 */
	public String getGraphVizTempFolder() {
		org.eclipse.core.runtime.Preferences prefs = SpagicUtilsActivator.getDefault().getPluginPreferences();
		return prefs.getString(SpagicPreferenceConstants.P_GRAPH_VIZ_TEMP_FOLDER);
	}

}
