/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.metadata.actions;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.stp.im.BasicProperty;
import org.eclipse.stp.im.ImPackage;
import org.eclipse.stp.im.ProcessCollection;
import org.eclipse.stp.im.Step;
import org.eclipse.stp.im.StpIntermediateModel;
import org.eclipse.ui.actions.WorkspaceModifyOperation;

public class DeployProcessSchemaWorkspaceOperation extends
		WorkspaceModifyOperation {

	private static final String FORM_SCHEMA = "FormSchema";

	private IFile imFile = null;

	private List<java.net.URI> schemaFiles = new ArrayList<java.net.URI>();
	
	
	public List<java.net.URI> getSchemaFiles() {
		return schemaFiles;
	}

	public void setSchemaFiles(List<java.net.URI> schemaFiles) {
		this.schemaFiles = schemaFiles;
	}

	public IFile getImFile() {
		return imFile;
	}

	public void setImFile(IFile imFile) {
		this.imFile = imFile;
	}

	public DeployProcessSchemaWorkspaceOperation(IFile imFile)
	{
		this.imFile = imFile;
	}
	
	/**
	 * Retrieve the Intermediate Model that the user selected.
	 * 
	 * @return The Intermediate Model that the user selected.
	 */
	private StpIntermediateModel retrieveIMModel() {
		// Read the bpmn emf model
		ResourceSet resourceSetIM = new ResourceSetImpl();
		URI uri = URI.createPlatformResourceURI(imFile.getFullPath().toString(), true);

		// Force Package Registration
		@SuppressWarnings("unused")
		ImPackage imPackage = ImPackage.eINSTANCE;

		// load on demand = true
		Resource resourceIM = resourceSetIM.getResource(uri, true);

		// Get The intermediate model
		StpIntermediateModel imModel = (StpIntermediateModel) resourceIM.getContents().get(0);
		return imModel;
	}

	public IFile getFile(IFile aFile, String relativePath){
		IWorkspace workspace = ResourcesPlugin.getWorkspace();
		IWorkspaceRoot root = workspace.getRoot();
		String projectName = aFile.getProject().getName();
		IProject proj = root.getProject(projectName);
		IFile imFile = proj.getFile(relativePath);
		return imFile;
	}
	
	protected void execute(IProgressMonitor monitor) throws CoreException,
			InvocationTargetException, InterruptedException {
		
		// Retrieve the Intermediate Model that the user selected
		StpIntermediateModel imModel = retrieveIMModel();

		// Save all processes in process collection
		ProcessCollection processCollection = imModel.getProcessCollection();
		List<org.eclipse.stp.im.Process> processList = processCollection.getProcesses();
		for (org.eclipse.stp.im.Process process : processList) {
			
			// Save the steps
			List<Step> stepList = process.getSteps();
			for (Step step : stepList) {
				BasicProperty formProperty = (BasicProperty) step.getProperties().get(FORM_SCHEMA);
				if (formProperty != null) {
					String schemaValue = formProperty.getValue();
					
					if ((schemaValue != null) && (schemaValue.length() != 0)) {
						IFile schemaFile = getFile(imFile, schemaValue);
						schemaFiles.add(schemaFile.getLocationURI());
					
						/*
						if (schemaFile.exists()) {
							WorkspaceModifyOperation deploySchemaOperation = new DeploySchemaWorkspaceOperation(schemaFile);
							PlatformUI.getWorkbench().getProgressService().run(false, false, deploySchemaOperation);
						}
						*/
						
					}

				}
			}			
		}		
	}

}
