/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.ui;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.URL;
import java.util.List;
import java.util.prefs.PreferenceChangeEvent;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.io.SAXReader;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.spagic3.utils.SpagicLogger;


/**
 * The activator class controls the plug-in life cycle
 */
public class SpagicUIActivator extends AbstractUIPlugin  {

	// The plug-in ID
	public static final String PLUGIN_ID = "org.spagic3.ui";

	// The shared instance
	private static SpagicUIActivator plugin;
	

	
	public static String HL72XML = "plain2xml";
	public static String XML2HL7 = "xml2plain";
	/**
	 * The constructor
	 */
	public SpagicUIActivator() {
	}

	/**
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		plugin = this;
		
		

		//configureSpagicStudioServiceRegistry(false);
		
		//copy keystore
		copyKeystoreFile("/keystore/keystore.jks", getSpagicStudioConfigurationPath() + "/keystore.jks");
		System.setProperty("jaxr-ebxml.home", getSpagicStudioConfigurationPath());
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
	}

	/**
	 * Returns the shared instance
	 *
	 * @return the shared instance
	 */
	public static SpagicUIActivator getDefault() {
		return plugin;
	}

	/**
	 * Returns an image descriptor for the image file at the given
	 * plug-in relative path
	 *
	 * @param path the path
	 * @return the image descriptor
	 */
	public static ImageDescriptor getImageDescriptor(String path) {
		return imageDescriptorFromPlugin(PLUGIN_ID, path);
	}
	
	public void configureSpagicStudioConfDirectory(){
		File cfgDirectory = new File(getSpagicStudioConfigurationPath());
		if (!cfgDirectory.exists())
			cfgDirectory.mkdirs();
	}
		
		
		
		public static String getSpagicStudioConfigurationPath(){
			IPath path = getDefault().getStateLocation();
			String cfgDirPath = path.toOSString();
			return cfgDirPath;
		}
		
		public static void copyInCfgDirectory(File destination, String bundleRelatedPath){
			BufferedReader sourceReader = null;
			BufferedWriter bw = null;
			try{
			 sourceReader = new BufferedReader (new InputStreamReader(getResourceURL(bundleRelatedPath).openStream()));
			 bw = new BufferedWriter(new FileWriter(destination));
			 String line = null;
			 while ( (line = sourceReader.readLine()) != null){
				 bw.write(line);
				 bw.write("\n");
				 bw.flush();
			 }
			}catch (Exception e) {
				e.printStackTrace();
			}finally{
				try{
					sourceReader.close();
					bw.close();
				}catch (Exception e) {
					e.printStackTrace();
				}
				
			}
		}
		
		public static void copyKeystoreFile(String pathSourceRel, String pathDest){
			try {
				//String pathSource = getResourceURL(pathSourceRel).getPath();
				//File fileSource = new File(pathSource);
				File fileDest = new File(pathDest);
				if (fileDest.exists())
					fileDest.delete();
			
				InputStream in = getResourceURL(pathSourceRel).openStream();
		        OutputStream out = new FileOutputStream(fileDest);
		    
		        // Transfer bytes from in to out
		        byte[] buf = new byte[1024];
		        int len;
		        while ((len = in.read(buf)) > 0) {
		            out.write(buf, 0, len);
		        }
		        in.close();
		        out.close();
			}catch(FileNotFoundException fe){
				fe.printStackTrace();
			}catch(IOException ioe){
				ioe.printStackTrace();
			}
		}
		
		public static URL getResourceURL(String path) throws IOException {
			if (PLUGIN_ID == null || path == null)
	    		throw new IllegalArgumentException();
			Bundle bundle = Platform.getBundle(PLUGIN_ID);
			URL url = FileLocator.find(bundle, new Path(path), null);
			return FileLocator.resolve(url);
		}
		
		/*
		public static File getServiceRegistryFile(){
			return new File(getSpagicStudioConfigurationPath() + "/serviceRegistry.xml");
		}
		
		public static StringBuffer getServiceRegistryProperties(){
			StringBuffer serviceRegistryProperties = new StringBuffer();
			try{
				SAXReader reader = new SAXReader(false);
				Document serviceRegistryDocument = reader.read(SpagicUIActivator.getServiceRegistryFile());
				List<org.dom4j.Node> nodeList = serviceRegistryDocument.selectNodes("/SERVICE_REGISTRIES/SERVICE_REGISTRY");
				
				for (org.dom4j.Node n : nodeList){
					String name = n.valueOf("@name");
					serviceRegistryProperties.append(n.valueOf("@name") + "_" + "inquiryUrl" + "=" + n.valueOf("@inquiryUrl") + "\n");
					serviceRegistryProperties.append(n.valueOf("@name") + "_" + "publishUrl" + "=" + n.valueOf("@publishUrl") + "\n");
					serviceRegistryProperties.append(n.valueOf("@name") + "_" + "user" + "=" + n.valueOf("@user") + "\n");
					serviceRegistryProperties.append(n.valueOf("@name") + "_" + "password" + "=" + n.valueOf("@password") + "\n");
					serviceRegistryProperties.append(n.valueOf("@name") + "_" + "defaultOrganitation" + "=" + n.valueOf("@defaultOrganitation") + "\n");
				}
			}catch(DocumentException de){
				SpagicLogger.error(SpagicUIActivator.PLUGIN_ID, "Error in Service Registry", de);
			}
			//serviceRegistryProperties.append(b)
			return serviceRegistryProperties;
		}
		
		public static void configureSpagicStudioServiceRegistry(boolean overwrite){
			File serviceRegistryFile = getServiceRegistryFile();

			if (overwrite && serviceRegistryFile.exists()){ 
				serviceRegistryFile.delete();
			}
			
			if (!serviceRegistryFile.exists()) 
				copyInCfgDirectory(serviceRegistryFile, "/conf/serviceRegistry.xml");
		}
		

		
		
		
		public void initServiceRegistry(){
			org.eclipse.core.runtime.Preferences pref = SpagicUtilsActivator.getDefault().getPluginPreferences();
			String serviceRegistryName = pref.getString(SpagicPreferenceConstants.P_SERVICE_REGISTRY);
			SAXReader reader = new SAXReader(false);
			try {
				Document serviceRegistryDocument = reader.read(SpagicUIActivator.getServiceRegistryFile());
				Node serviceRegistryConf = serviceRegistryDocument.selectSingleNode("/SERVICE_REGISTRIES/SERVICE_REGISTRY[@name='" + serviceRegistryName + "']");
			
				Properties properties = new Properties();
				properties.setProperty("javax.xml.registry.queryManagerURL", serviceRegistryConf.valueOf("@inquiryUrl"));
				properties.setProperty("javax.xml.registry.lifeCycleManagerURL", serviceRegistryConf.valueOf("@publishUrl"));
				properties.setProperty("registry.username", serviceRegistryConf.valueOf("@user"));
				properties.setProperty("registry.password", serviceRegistryConf.valueOf("@password"));
				
				if (serviceRegistryName.equalsIgnoreCase(SpagicPreferenceConstants.P_UDDI)) {
					properties.setProperty("metadb.db.driver", pref.getString(SpagicPreferenceConstants.P_SPAGIC_DB_DRIVER));
					properties.setProperty("metadb.db.user",pref.getString(SpagicPreferenceConstants.P_SPAGIC_DB_USER) );
					properties.setProperty("metadb.db.password",pref.getString(SpagicPreferenceConstants.P_SPAGIC_DB_PASSWORD));
					properties.setProperty("metadb.db.url", pref.getString(SpagicPreferenceConstants.P_SPAGIC_DB_URL));
				}
				
				serviceRegistry = ServiceRegistryFactory.getServiceRegistry(serviceRegistryName, properties);
			}catch(Throwable  e){
				SpagicLogger.error(SpagicUIActivator.PLUGIN_ID, "Error in Service Registry Initialization ", e);
			}
		}
		*/
		
		

}
