/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.ui.deploymentservice;

import java.net.URI;
import java.net.URL;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;
import org.spagic3.ui.deploymentservice.wizards.ServiceInfoRepository;

/**
 * The activator class controls the plug-in life cycle
 */
public class Activator extends AbstractUIPlugin {

	// The plug-in ID
	public static final String PLUGIN_ID = "org.spagic3.ui.deploymentservice";

	public static final String IMG_PROJECTS = "projects";
	public static final String IMG_CONNECTOR = "connector";
	public static final String IMG_SERVICE = "service";

	// The shared instance
	private static Activator plugin;
	
//	private Map<IProject, ServiceInfoRepository> serviceInfoRepositoryMap;

//	private Comparator<IFile> fileComparator = new Comparator<IFile>() {
//		public int compare(IFile o1, IFile o2) {
//			return o1.getName().compareTo(o2.getName());
//		}
//	};
		
	/**
	 * The constructor
	 */
	public Activator() {
//		serviceInfoRepositoryMap = new HashMap<IProject, ServiceInfoRepository>();
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		plugin = this;
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
	}

	/**
	 * Returns the shared instance
	 *
	 * @return the shared instance
	 */
	public static Activator getDefault() {
		return plugin;
	}
	
	public ServiceInfoRepository getServiceInfoRepository(IProject project) {
//		ServiceInfoRepository repository = serviceInfoRepositoryMap.get(project);
//		if (repository == null) {
//			List<URI> serviceURIs = new LinkedList<URI>();
//			appendServiceURIs(serviceURIs, project);
//			repository = new ServiceInfoRepository(serviceURIs);
//			serviceInfoRepositoryMap.put(project, repository);
//		}
//		return repository;
		return new ServiceInfoRepository(getServiceURIs(project));
	}
	
	public List<URI> getServiceURIs(IProject project) {
		List<URI> serviceURIs = new LinkedList<URI>();
		appendServiceURIs(serviceURIs, project);
		return serviceURIs;
	}
	
	private void appendServiceURIs(List<URI> serviceURIs, IResource resource) {
		if (resource instanceof IFile) {
			final IFile file = (IFile) resource;
			final String extension = file.getFileExtension();
			if ("service".equals(extension) 
					|| "connector".equals(extension)) {
				final URI uri = file.getLocationURI();
				int index = Collections.binarySearch(serviceURIs, uri);
				if (index < 0) {
					serviceURIs.add(-index-1, uri);
			    }
			}
		} else if (resource instanceof IContainer) {
			final IContainer container = (IContainer) resource;
			try {
				for (IResource member : container.members(true)) {
					appendServiceURIs(serviceURIs, member);
				}
			} catch (CoreException e) {}
		}
	}

	public List<URI> getBpmnURIs(IProject project) {
		List<URI> bpmnURIs = new LinkedList<URI>();
		appendBpmnURIs(bpmnURIs, project);
		return bpmnURIs;
	}
	
	private void appendBpmnURIs(List<URI> bpmnURIs, IResource resource) {
		if (resource instanceof IFile) {
			final IFile file = (IFile) resource;
			final String extension = file.getFileExtension();
			if ("bpmn".equals(extension)) {
				final URI uri = file.getLocationURI();
				int index = Collections.binarySearch(bpmnURIs, uri);
				if (index < 0) {
					bpmnURIs.add(-index-1, uri);
			    }
			}
		} else if (resource instanceof IContainer) {
			final IContainer container = (IContainer) resource;
			try {
				for (IResource member : container.members(true)) {
					appendBpmnURIs(bpmnURIs, member);
				}
			} catch (CoreException e) {}
		}
	}

	public List<URI> getResourceURIs(List<String> resourceFilenames, IProject project) {
		List<URI> resourceURIs = new LinkedList<URI>();
		appendResourceURIs(resourceURIs, resourceFilenames, project);
		return resourceURIs;
	}
	
	private void appendResourceURIs(List<URI> resourceURIs, List<String> resourceFilenames, IResource resource) {
		if (resource instanceof IFile) {
			final IFile file = (IFile) resource;
			final String filename = file.getName();
			if (resourceFilenames.contains(filename)) {
				final URI uri = file.getLocationURI();
				int index = Collections.binarySearch(resourceURIs, uri);
				if (index < 0) {
					resourceURIs.add(-index-1, uri);
			    }
			}
		} else if (resource instanceof IContainer) {
			final IContainer container = (IContainer) resource;
			try {
				for (IResource member : container.members(true)) {
					appendResourceURIs(resourceURIs, resourceFilenames, member);
				}
			} catch (CoreException e) {}
		}
	}

	protected void initializeImageRegistry(ImageRegistry registry) {
		registerImage(registry, IMG_PROJECTS, "projects.gif");
		registerImage(registry, IMG_CONNECTOR, "connector.gif");
		registerImage(registry, IMG_SERVICE, "service.gif");
	}

	private void registerImage(
			ImageRegistry registry, 
			String key,
			String fileName) {
		try {
			IPath path = new Path("icons/" + fileName);
			URL url = FileLocator.find(getBundle(), path, null);
			if (url != null) {
				ImageDescriptor desc = ImageDescriptor.createFromURL(url);
				registry.put(key, desc);
			}
		} catch (Exception e) {
		}
	}
	
	public Image getImage(String key) {
		return getImageRegistry().get(key);
	}
	
	public ImageDescriptor getImageDescriptor(String key) {
		return getImageRegistry().getDescriptor(key);
	}

}
