/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.ui.deploymentservice;

import java.net.URI;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.common.util.EMap;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.stp.bpmn.Activity;
import org.eclipse.stp.bpmn.ActivityType;
import org.eclipse.stp.bpmn.BpmnDiagram;
import org.eclipse.stp.bpmn.Pool;
import org.eclipse.stp.bpmn.Vertex;
import org.eclipse.stp.im.constants.ImConstants;
import org.spagic3.ui.deploymentservice.wizards.ServiceInfo;
import org.spagic3.ui.deploymentservice.wizards.ServiceInfoRepository;

public class BpmnResolver {

	private List<URI> bpmnURIs;
	private ServiceInfoRepository infoRepository;
	private List<URI> usedBpmnURIs;
	
	public BpmnResolver(List<URI> bpmnURIs, ServiceInfoRepository infoRepository) {
		this.bpmnURIs = bpmnURIs;
		this.infoRepository = infoRepository;
	}
	
	public List<URI> getBpmnURIs() {
		return bpmnURIs;
	}

	public List<URI> getUsedBpmnURIs() {
		return usedBpmnURIs;
	}

	public List<URI> getServiceURIs(List<String> processIds) {
		List<URI> toBeReturnedURIs = new ArrayList<URI>();
		ResourceSet bpmnResourceSet = new ResourceSetImpl();
		usedBpmnURIs = new ArrayList<URI>();
		for (URI bpmnURI : bpmnURIs) {
			final Resource bpmnResource
					= bpmnResourceSet.getResource(
							org.eclipse.emf.common.util.URI.createURI(
									bpmnURI.toString()), 
							true);	
			final BpmnDiagram bpmnDiagram
					= (BpmnDiagram) bpmnResource.getContents().get(0);
			for (Pool pool : bpmnDiagram.getPools()) {
				final String poolName = pool.getName();
				if (poolName != null 
						&& processIds.contains(poolName)) {
					usedBpmnURIs.add(bpmnURI);
					for (Vertex vertex : pool.getVertices()) {
						if (vertex instanceof Activity) {
							final Activity activity = (Activity) vertex;
							if (ActivityType.TASK_LITERAL.equals(activity.getActivityType())) {
								final EAnnotation annotation = activity.getEAnnotation(
										ImConstants.IM_ANNOTATION);
								if (annotation != null) {
									final EMap<String,String> details = annotation.getDetails();
									if (details != null) {
										final String taskType = annotation.getDetails()
												.get(ImConstants.IM_SERVICE_BINDING_NAME);
										final String serviceType = annotation.getDetails()
												.get("ServiceType");
										final String serviceId = annotation.getDetails()
												.get("Service");
										if ("AutomaticTask".equals(taskType)
												&& "OSGi".equals(serviceType)
												&& serviceId != null 
												&& !"".equals(serviceId)) {
											final ServiceInfo serviceInfo
													= infoRepository.findServiceInfoById(serviceId);
											if (serviceInfo != null) {
												final URI serviceURI = serviceInfo.getURI();
												if (serviceURI != null
														&& !toBeReturnedURIs.contains(serviceURI)) {
													toBeReturnedURIs.add(serviceURI);
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
		return toBeReturnedURIs;
	}
}
