/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.ui.deploymentservice.actions;

import java.lang.reflect.InvocationTargetException;
import java.net.URI;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.spagic3.utils.SpagicLogger;



public class Bpmn2JbpmGeneratorAction implements IObjectActionDelegate {

	private IFile bpmnFile = null;

	/**
	 * Constructor for Action1.
	 */
	public Bpmn2JbpmGeneratorAction() {
		super();
	}

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
	}

	
	
	/**
	 * @see IActionDelegate#run(IAction)
	 */
	public void run(IAction action) {
		
		
		Shell shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
		try {
			
			
			
			PreparePackagingArtifacts preparePackagingArtifacts = new PreparePackagingArtifacts();
			preparePackagingArtifacts.setBpmnFile(bpmnFile);
			preparePackagingArtifacts.setGenerateAll(true);
			List<URI> uris = new ArrayList<URI>();
			preparePackagingArtifacts.doPackaging(uris);

		
			
		
		} catch (Throwable e) {
			reportError(shell, e);
		}
	}

	private void reportError(Shell shell, Throwable x) {
		SpagicLogger.error(x.getMessage(), x);
		ErrorDialog.openError(shell, "Error", "Error Occurred", makeStatus(x));
	}

	public StringBuffer getCauseFromThrowable(Throwable x){
		StringBuffer causeMessage = new StringBuffer();
		if (x.getMessage() != null)
			causeMessage.append(x.getMessage()+"\n");
		
		if (x.getCause() != null){
			if (x.getCause().getMessage() != null){
				causeMessage.append(x.getCause().getMessage()+"\n");
			}
		}
		return causeMessage;
	}
	
	public IStatus makeStatus(Throwable x) {

		StringBuffer causeMessage = new StringBuffer();
		
		if (x instanceof InvocationTargetException){
			Throwable target = ((InvocationTargetException)x).getTargetException();
			causeMessage.append(getCauseFromThrowable(target));
		}else{
			causeMessage = getCauseFromThrowable(x);
		}
		
		return new Status(IStatus.ERROR, 
							org.spagic3.ui.deploymentservice.Activator.PLUGIN_ID, 
							IStatus.ERROR,
				causeMessage.toString(), null);

	}

	
	
	public final void selectionChanged(final IAction action,
			final ISelection selection) {
		
		setBpmnFile((IFile) ((StructuredSelection) selection)
				.getFirstElement());
	}
	public IFile getBpmnFile() {
		return bpmnFile;
	}

	public void setBpmnFile(IFile bpmnFile) {
		this.bpmnFile = bpmnFile;
	}

	
			
}
