/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.ui.deploymentservice.wizards;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URI;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Node;
import org.dom4j.XPath;
import org.dom4j.io.SAXReader;
import org.dom4j.xpath.DefaultXPath;
import org.spagic3.service.model.ServiceModelHelper;

public class ServiceInfo {

	private URI uri;
	private Document document;
	private String serviceId;
	private String factoryName;
	private String target;
	private boolean targetFetched = false;
	private String process;
	private boolean processFetched = false;
	private ServiceModelHelper serviceModelHelper = null;
	
	private static Map<String, String> namespaceMap;
//	private static String[] resourceNames;

	static {
		namespaceMap = new HashMap<String, String>();
		namespaceMap.put("spagic", "urn:org:spagic3");
//		resourceNames = new String[] {"script", "wsdl", "xslt", "schema"};
	}
	
	public ServiceInfo(URI uri) {
		this.uri = uri;
	}

	public boolean equals(Object obj) {
		if (obj instanceof ServiceInfo) {
			final ServiceInfo that = (ServiceInfo) obj;
			return this.getURI().equals(that.getURI());
		}
		return false;
	}

	public URI getURI() {
		return uri;
	}

	public String getServiceId() {
		if (serviceId != null) {
			return serviceId;
		}
		if (document == null) {
			fetchDocument();
		}
		if (document != null) {
			return serviceId = evalXPathAsString(document, 
					"/spagic:component/@spagic.id");
		}
		return null;
	}

	public String getFactoryName() {
		if (factoryName != null) {
			return factoryName;
		}
		if (document == null) {
			fetchDocument();
		}
		if (document != null) {
			return factoryName = evalXPathAsString(document, 
					"/spagic:component/@factory.name");
		}
		return null;
	}

	public String getTarget() {
		if (target != null || targetFetched) {
			return target;
		}
		if (document == null) {
			fetchDocument();
		}
		if (document != null) {
			targetFetched = true;
			return target = evalXPathAsString(document, 
					"/spagic:component/spagic:property[@name=\"target\"]/@value");
		}
		return null;
	}

	public String getProcess() {
		if (process != null || processFetched) {
			return process;
		}
		if (document == null) {
			fetchDocument();
		}
		if (document != null) {
			processFetched = true;
			return process = evalXPathAsString(document, 
					"/spagic:component/spagic:property[@name=\"process\"]/@value");
		}
		return null;
	}

	public List<String> getResourceFilenames() {
		List<String> resources = new ArrayList<String>();
		if (document == null) {
			fetchDocument();
		}
		if (document != null) {
			if (serviceModelHelper == null) {
				try {
					serviceModelHelper = new ServiceModelHelper();
				} catch (Exception e) {
					e.printStackTrace();
					return resources;
				}
			}
			final List<String> resourcePropertyNames
					= serviceModelHelper.getResoursePropertyNames(getFactoryName());
			for (String resourcePropertyName : resourcePropertyNames) {
				final String value = evalXPathAsString(document, 
						"/spagic:component/spagic:property[@name=\"" 
						+ resourcePropertyName + "\"]/@value");
				if (value != null && !"".equals(value)) {
					int schemaIndex = value.indexOf("://");
					resources.add(schemaIndex == -1 
							? value : value.substring(schemaIndex + 3));
				}
			}
		}
		return resources;
	}
	
	private void fetchDocument() {
		final SAXReader xmlReader = new SAXReader();
		try {
			document = xmlReader.read(uri.toURL().openStream());
		} catch (MalformedURLException mue) {
		} catch (DocumentException de) {
		} catch (IOException ioe) {
		}
	}
	
	public static List<Node> evalXPathAsNodes(String xml, String xpath) {
		try {
	        Document document = DocumentHelper.parseText(xml);
			return evalXPathAsNodes(document, xpath);
		} catch (Exception e) {
			return null;
		}
	}
	
	@SuppressWarnings("unchecked")
	public static List<Node> evalXPathAsNodes(Document document, String xpath) {
		try {
			XPath xPath = new DefaultXPath(xpath);
			xPath.setNamespaceURIs(getNamespaceMap());
			return (List<Node>) xPath.selectNodes(document);
		} catch (Exception e) {
			return null;
		}
	}
	
	public static String evalXPathAsString(String xml, String xpath) {
		try {
	        Document document = DocumentHelper.parseText(xml);
			return evalXPathAsString(document, xpath);
		} catch (Exception e) {
			return null;
		}
	}
	
	public static String evalXPathAsString(Document document, String xpath) {
		try {
			XPath xPath = new DefaultXPath(xpath);
			xPath.setNamespaceURIs(namespaceMap);
			return xPath.valueOf(document);
		} catch (Exception e) {
			return null;
		}
	}
	
	public static Map<String, String> getNamespaceMap() {
		return namespaceMap;
	}

	public static void setNamespaceMap(Map<String, String> namespaceMap) {
		ServiceInfo.namespaceMap = namespaceMap;
	}

}
