/*******************************************************************************
 * Copyright (c) {2007, 2008} Engineering Ingegneria Informatica S.p.A.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors: Andrea Zoppello (Engineering) - initial API and implementation
 *******************************************************************************/
package org.spagic3.ui.dnd;

import java.util.HashMap;
import java.util.Map;

import org.dom4j.Document;
import org.dom4j.io.SAXReader;
import org.dom4j.xpath.DefaultXPath;
import org.eclipse.core.resources.IResource;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EModelElement;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.gmf.runtime.diagram.ui.editparts.IGraphicalEditPart;
import org.eclipse.stp.bpmn.Activity;
import org.eclipse.stp.bpmn.ActivityType;
import org.eclipse.stp.bpmn.Graph;
import org.eclipse.stp.bpmn.dnd.file.FileDnDHandler;
import org.eclipse.stp.im.constants.ImConstants;
import org.eclipse.stp.im.runtime.IProperty;
import org.eclipse.stp.im.runtime.IRuntime;
import org.eclipse.stp.im.runtime.ImRuntimeActivator;
import org.eclipse.stp.im.runtime.comboproviders.ComboEntries;
import org.eclipse.stp.im.runtime.comboproviders.IComboProvider;



public class ServiceFileDndHandler extends FileDnDHandler {

	private static final String RUNTIME_ID = "runtime.jbpm";
	private static final String TECNOLOGY_NAME = "JBPM Runtime";
	
	private static final String DEFAULT_SERVICE = "TaskService";
	
	private static final String AUTOMATIC_SERVICE_BINDING = "AutomaticTask";
	private static final String HUMAN_SERVICE_BINDING = "HumanTask";

	private static final String AUTOMATIC_SERVICE = "Service";
	private static final String AUTOMATIC_SERVICE_TYPE = "ServiceType";
	
	private static final String NOTIFICATION_SERVICE = "NotificationService";
	private static final String NOTIFICATION_SERVICE_TYPE = "NotificationServiceType";
	
	private static Map<String, String> namespaceMap;
	
	static {
		namespaceMap = new HashMap<String, String>();
		namespaceMap.put("spagic", "urn:org:spagic3");
	}
	
	
	// This could be avoided but if in the base class is declared protected
	// But now it's private
	protected IResource myFile = null;
	protected String serviceID = null;
	public ServiceFileDndHandler(IResource f) {
		super(f);
		myFile = f;
		setServiceId();
	}
	
	public void setServiceId(){
		try{
			SAXReader reader = new SAXReader();
			Document doc = reader.read(myFile.getLocationURI().toURL().openStream());
			this.serviceID = evalXPathAsString(doc, "/spagic:component/@spagic.id");
		}catch (Exception e) {
			throw new RuntimeException(e);
		}
	}
	
	public static String evalXPathAsString(Document document, String xpath) {
		try {
			org.dom4j.XPath xPath = new DefaultXPath(xpath);
			xPath.setNamespaceURIs(namespaceMap);
			return xPath.valueOf(document);
		} catch (Exception e) {
			return null;
		}
	}
	
	/**
	 * this method returns a command that will create an annotation
	 * on the semantic element of the edit part passed in parameter.
	 */
	public Command getDropCommand(IGraphicalEditPart hoverPart, int index, 
			Point dropLocation) {
	
    	String extension = myFile.getFileExtension();
		
    	if (extension.equalsIgnoreCase("service") ||
    		extension.equalsIgnoreCase("connector") ||
    		extension.equalsIgnoreCase("scrappy")) {

    		CompoundCommand co = new CompoundCommand("Setting Service on Task");
    		EModelElement eModelElement = (EModelElement) hoverPart.resolveSemanticElement();
    		
    		Activity activity = (Activity)eModelElement;
    		Graph g = activity.getGraph();
			
    		// 
    		// Creo l'annotazione tecnologica nel pool se non esiste
    		//
    		EAnnotation technologyAnnotation = g.getEAnnotation(ImConstants.TECHNOLOGY_ANNOTATION);
			if (technologyAnnotation == null){
				Map<String, String> details = new HashMap<String, String>();
    	 		
    	 		details.put(ImConstants.IM_POOL_RUNTIME_ID, RUNTIME_ID);
    	 		details.put(ImConstants.TECHNOLOGY_NAME,TECNOLOGY_NAME);
    	 		
    	 		createProcessProperties(details,RUNTIME_ID);
		   
    	 		Command cmd = createEAnnotationDropCommand(createAnnotation(ImConstants.TECHNOLOGY_ANNOTATION, details), g);
    			co.add(cmd);
			}
			
			// Altrimenti creo solo l'annotazione per il servizio
			co.add(getDropCommand(hoverPart, index, dropLocation, serviceID, extension));
			return co;
    	}else{
    		return super.getDropCommand(hoverPart, index, dropLocation);
    	}

	
	}
	
	
	public Command getDropCommand(IGraphicalEditPart hoverPart, int index,
			Point dropLocation, String idService, String extension) {

		try {
			Map<String, String> details = new HashMap<String, String>(6);
			
			EModelElement eModelElement = (EModelElement) hoverPart.resolveSemanticElement();
			Activity activity = (Activity)eModelElement;
			EAnnotation imAnnotation = activity.getEAnnotation(ImConstants.IM_ANNOTATION);
			if (imAnnotation != null) {
				String stepServiceBindingName = imAnnotation.getDetails().get(ImConstants.IM_SERVICE_BINDING_NAME);
				
				if (HUMAN_SERVICE_BINDING.equals(stepServiceBindingName)) {
					// Human Task
					details.put("NotifyTaskAvailability", "true");
					details.put(ImConstants.IM_SERVICE_BINDING_NAME, HUMAN_SERVICE_BINDING);
					details.put(NOTIFICATION_SERVICE, idService);
					details.put(NOTIFICATION_SERVICE_TYPE, "OSGi");
					
					// Merge existing annotations into new annotations
					for (String key : imAnnotation.getDetails().keySet()) {
						if (!details.containsKey(key)) {
							details.put(key, imAnnotation.getDetails().get(key));
						}
					}
				}
			}
			if (details.size() == 0) {
				// Automatic Task
				details.put(ImConstants.IM_SERVICE_BINDING_NAME, AUTOMATIC_SERVICE_BINDING);
				details.put(AUTOMATIC_SERVICE, idService);
				details.put(AUTOMATIC_SERVICE_TYPE, "OSGi");
			}
			
			// Properties common to automatic and human tasks
			details.put(ImConstants.IM_POOL_RUNTIME_ID, RUNTIME_ID);
			details.put(ImConstants.IM_SERVICE_NAME, DEFAULT_SERVICE);
			
			return createEAnnotationDropCommand(createAnnotation(
					ImConstants.IM_ANNOTATION, details),
					eModelElement);
		} catch (ArrayIndexOutOfBoundsException e) {

			throw new RuntimeException(e);
		}

	}


	public void createProcessProperties(Map<String, String> details, String runtimeId){
		IRuntime imRuntime = ImRuntimeActivator.getRuntime(runtimeId);
		for (IProperty property : imRuntime.getProcessProperties().getDefinedProperties()){
			if ((!property.isMap()) && (!property.isVisibleUnderCondition())){
				String value = (property.getDefaultValue() != null ? property.getDefaultValue() : ""); 
				
				if ((value == null) || (value.trim().length() == 0)){  
					if (property.getPropertyEditor().equalsIgnoreCase("combo")){
						IComboProvider comboProvider = property.getComboProvider();
						if (comboProvider == null){
							comboProvider = imRuntime.getNamedComboProvider(property.getComboProviderName());
						}
						
						
						ComboEntries ce = comboProvider.getComboEntries();
						value = ce.getValues()[0];
					}
				}
				details.put(property.getName(),value);
			}
		}
	}
	
	
	public String getMenuItemLabel(IGraphicalEditPart hoverPart, int index) {
		String menuLabel = null;
		
		EModelElement eModelElement = (EModelElement) hoverPart.resolveSemanticElement();
		Activity activity = (Activity)eModelElement;
		
		EAnnotation imAnnotation = activity.getEAnnotation(ImConstants.IM_ANNOTATION);
		if (imAnnotation != null) {
			String stepServiceBindingName = imAnnotation.getDetails().get(ImConstants.IM_SERVICE_BINDING_NAME);
			
			if (HUMAN_SERVICE_BINDING.equals(stepServiceBindingName)) {
				menuLabel = (serviceID != null ? "Activate Notification for Task Availability with Service["
						+ serviceID + "]" : "No Service ");
			}
		}
		if (menuLabel == null) {
			menuLabel = (serviceID != null ? "Create Automatic Task For Service["
					+ serviceID + "]" : "No Service ");
		}

		return menuLabel;
	}

	/**
	 * the file drop should have the lowest priority.
	 */
	public int getPriority() {
		return 10;
	}

	
	
	public boolean isStartOrEnd(Activity act){
		boolean result = false;
		
    	ActivityType at = act.getActivityType();
    	switch (at.getValue()) {
		case ActivityType.EVENT_START_EMPTY:
			result =true;
			break;
		case ActivityType.EVENT_END_EMPTY: {
			result =true;
			break;
		}
		default:
			break;
		}
		return result;
	}
    @Override
    public boolean isEnabled(IGraphicalEditPart hoverPart, int index) {
    	EModelElement emodelElement = (EModelElement) hoverPart.resolveSemanticElement();
    	if (!(emodelElement instanceof Activity)) {
			return false;		
		}
    	if (isStartOrEnd((Activity)emodelElement)){
    		return false;
    	}
    	
    	if (myFile == null)
    		return false;
    	if (myFile.getType() != IResource.FILE)
    		return false;
    	
    
    	String extension = myFile.getFileExtension();
    	if (extension.equalsIgnoreCase("service") || extension.equals("scrappy")){
    		return true;
    	}
    	if (extension.equalsIgnoreCase("connector")){
		  	return true;
    	}
    	return false;		
    }
}
