/**

    Copyright 2007, 2008, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
 **/
package org.spagic3.ui.embed.webapp.popup.actions;

import java.io.File;
import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.spagic3.ui.embed.webapp.Activator;
import org.spagic3.ui.embed.webapp.ui.IniFileChooser;
import org.spagic3.ui.embed.webapp.utils.ProcessCheckerResult;
import org.spagic3.ui.embed.webapp.utils.ProcessStarter;


public class EmbedSpagic implements IObjectActionDelegate {

	private Shell shell;
	private IProject project;

	/**
	 * Constructor for Action1.
	 */
	public EmbedSpagic() {
		super();
	}

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
	}

	/**
	 * @see IActionDelegate#run(IAction)
	 */
	public void run(IAction action) {

		shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
		ProcessStarter chkr = new ProcessStarter();
		
			ProcessCheckerResult rslt = chkr.checkProject(project);
			
			if(rslt.isAllRight()) {
				File availableIniFiles[] = chkr.getSpagicEmbeddingIniFiles();
				
				IniFileChooser iniSelector = new IniFileChooser(shell, availableIniFiles);
				iniSelector.setBlockOnOpen(true);
				iniSelector.open();
				
				if (iniSelector.getSelection() == -1)
					return;
				
				File selectedIniFile = availableIniFiles[iniSelector.getSelection()];
				
				try {
					WorkspaceModifyOperation operation = new Processor(chkr, selectedIniFile);
					PlatformUI.getWorkbench().getProgressService().run(false, false, operation);
					MessageDialog.openInformation(shell,
							"Information",
							"Embedding of the Service Manager completed");
				} catch (InvocationTargetException e) {
					reportError(shell, e);
				} catch (InterruptedException e) {
					reportError(shell, e);
				} catch (Exception e) {
					reportError(shell, e);
				} 
				
			}
			else {
				MessageDialog.openError(shell, "Webapp", rslt.getReply());
			}
	}

	/**
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		project = (IProject)((StructuredSelection) selection).getFirstElement();

	}
	
	/**
	 * Displays an error that occured during the project creation.
	 * 
	 * @param x
	 *            details on the error
	 */
	private void reportError(Shell shell, Throwable x) {
		ErrorDialog.openError(shell, "Error", "Error Occurred of type " + x.getClass().getSimpleName(), makeStatus(x));
	}

	public static IStatus makeStatus(Throwable x) {

		return new Status(IStatus.ERROR, Activator.PLUGIN_ID, IStatus.ERROR,
				(x.getMessage() != null) ? x.getMessage()
				: (x.getCause() != null) ? x.getCause().getMessage() : "", null);


	}

}
