/**

    Copyright 2007, 2008, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
 **/
package org.spagic3.ui.embed.webapp.popup.actions;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.lang.reflect.InvocationTargetException;
import java.nio.channels.FileChannel;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.spagic3.tools.ConfigIniGenerator;
import org.spagic3.ui.embed.webapp.Activator;
import org.spagic3.ui.embed.webapp.utils.ProcessStarter;
import org.spagic3.ui.embed.webapp.utils.WebXmlUtils;

public class Processor extends WorkspaceModifyOperation {

	private static final String SERVLETBRIDGE_JAR = "servletbridge.jar";
	private static final String SPAGIC_CLIENT_API_JAR = "spagic-client-api.jar";
	private static final String LAUNCH_INI = "launch.ini"; 
	private static final String UPDATER_CONFIG_JAR = "org.eclipse.update.configurator_3.2.100.v20070322.jar";
	private static final String SPAGIC_OSGI_CLIENT = "org.spagic3.client.osgi_1.0.0.jar";
	
	private ProcessStarter processStarter;
	private File selectedIniFile; 
	public Processor(ProcessStarter processStarter, File selectedIniFile) {
		this.processStarter = processStarter;
		this.selectedIniFile = selectedIniFile;
	}

	@Override
	protected void execute(IProgressMonitor arg0) throws CoreException,
			InvocationTargetException, InterruptedException {

		arg0.setTaskName("Initialization");
		File serviceManagerDirFiles[] = processStarter.getServiceManagerPluginDir().listFiles();
		if (serviceManagerDirFiles == null || serviceManagerDirFiles.length == 0) {
			StringBuilder erbuf = new StringBuilder("The contents of directory\n\n");
			erbuf.append(processStarter.getServiceManagerPluginDir().getAbsolutePath());
			erbuf.append("\n\ndo not conform to the expected structure, no plugin file found");
			throw new 
				CorruptedInputData(
						new Throwable() {private static final long serialVersionUID = -3171254238232499263L;}, 
						erbuf.toString());
		}
		arg0.worked(1);
		
		File embeddingFiles[] = processStarter.getSpagicEmbeddedJarDir().listFiles();
		if (embeddingFiles == null || embeddingFiles.length == 0) {
			StringBuilder erbuf = new StringBuilder("The contents of directory\n\n");
			erbuf.append(processStarter.getSpagicEmbeddedJarDir().getAbsolutePath());
			erbuf.append("\n\ndo not conform to the expected structure, no jar file to embed found");
			throw new 
				CorruptedInputData(
						new Throwable() {private static final long serialVersionUID = -3171254238232499263L;}, 
						erbuf.toString());
		}
		arg0.worked(1);
		Map<String, File> embeddingDirFileMap = new HashMap<String, File>();
		
		int fmax = embeddingFiles.length;
		for(int i = 0; i < fmax; i++) {
			embeddingDirFileMap.put(embeddingFiles[i].getName(), embeddingFiles[i]);
		}
		// garbage collector, come here, a cookie for you :)
		embeddingFiles = null;
		
		
		arg0.setTaskName("Preliminary work");
		// create the new eclipse directory
		IFolder webInf = processStarter.getWebInfFolder();
		IFolder eclipse = getExistingIFolder("eclipse", webInf, arg0);
		
		
		// get an handle to the lib and create it if missing.
		IFolder lib = getExistingIFolder("lib", webInf, arg0);
		
		transfer(embeddingDirFileMap, SERVLETBRIDGE_JAR, lib, arg0);
		transfer(embeddingDirFileMap, SPAGIC_CLIENT_API_JAR, lib, arg0);
		transfer(embeddingDirFileMap, LAUNCH_INI, eclipse, arg0);
	
		File iniFileDest = getFileFromIFolder("eclipse.ini", eclipse);
		
		copyFile(selectedIniFile, iniFileDest);
		arg0.worked(1);

		arg0.setTaskName("Plugins copy");

		IFolder plugins = getExistingIFolder("plugins", eclipse, arg0);
		
		recursiveCopy(serviceManagerDirFiles, plugins, arg0);

		transfer(embeddingDirFileMap, SPAGIC_OSGI_CLIENT, plugins, arg0);
		
		arg0.setTaskName("Finishing up...");
		IFolder configuration = getExistingIFolder("configuration", eclipse, arg0);

		copyFile(embeddingDirFileMap.get(UPDATER_CONFIG_JAR),
				getFileFromIFolder(UPDATER_CONFIG_JAR, plugins));
		
		arg0.worked(1);
		
		File configIni = getFileFromIFolder("config.ini", configuration);
		
		ConfigIniGenerator.main(new String[] {
				processStarter.getSpagicSimpleConfiguratorInfo().getAbsolutePath(),
				configIni.getAbsolutePath()
		});
		
		File webXml = getFileFromIFolder("web.xml", webInf);
		
		try {
			if(! webXml.exists()) {
				PrintWriter prw = new PrintWriter(webXml);
				WebXmlUtils.printWholeFile(prw, processStarter.getSpagicDeploymentPath());
				prw.close();
			} 
			else {
				patchIfRequired(webXml);
			}
		
			webInf.getProject().getWorkspace().getRoot().refreshLocal(IResource.DEPTH_INFINITE, arg0);
		} catch (IOException e) {
			StringBuilder buf = new StringBuilder("I/O error accessing ");
			buf.append(webXml.getPath());
			IStatus status = new Status(IStatus.ERROR,Activator.PLUGIN_ID,buf.toString(),e);
			throw new CoreException(status);			
		} 
		
		arg0.done();
	}

	private static final Pattern testForPatchPattern = 
		Pattern.compile("<servlet-class>org.eclipse.equinox.servletbridge.SpagicServlet</servlet-class>");
	
	private void patchIfRequired(File webXml) throws IOException {
		BufferedReader br = new BufferedReader( new FileReader(webXml));
		StringBuilder sb = new StringBuilder(2024);
		
		int ch;
		while( (ch = br.read()) != -1 )
			sb.append((char)ch);
		
		br.close();
		
		Matcher testForPathcMatcher = testForPatchPattern.matcher(sb);
		if ( ! testForPathcMatcher.find() ) {
			int where = sb.lastIndexOf("</web-app>");
			PrintWriter prw = new PrintWriter(webXml);
			prw.println(sb.substring(0,where));
			prw.print(WebXmlUtils.servlet(processStarter.getSpagicDeploymentPath()));
			prw.print(sb.substring(where));
			prw.close();
		}
		
	}

	private static final String NOT_THIS_DIRECTORY_PREFIX = "org.eclipse.equinox.launcher.";
	
	private void recursiveCopy(File sourceFiles[], 
			IFolder destination, 
			IProgressMonitor arg0) throws CoreException {
		
		int max = sourceFiles.length;
		
		for(int i = 0; i < max; i++) {
			File current = sourceFiles[i];
			if(current.isDirectory()) {
				String currentName = current.getName();
				if (! currentName.startsWith(NOT_THIS_DIRECTORY_PREFIX)) {
					// prepare the source
					File content[] = current.listFiles();

					// prepare the destination
					IFolder destFolder = getExistingIFolder(currentName,destination,arg0);
					
					// copy
					recursiveCopy(content, destFolder, arg0);
				}
				arg0.worked(1);
			} 
			else if (current.isFile()) { 
				File destFile = new File(destination.getFile(current.getName()).getLocationURI());
				
				copyFile(current,destFile);

				arg0.worked(1);

			}
		}
	}

	private IFolder getExistingIFolder(String newFolderName, 
			IFolder father, 
			IProgressMonitor arg0) throws CoreException {
		IFolder destFolder = father.getFolder(newFolderName);
		if(! destFolder.exists())
			destFolder.create(IResource.FORCE,true,arg0);
		return destFolder;
	}

	private File getFileFromIFolder(String fileName, IFolder destDir) {
		IFile temp = destDir.getFile(fileName);
		return new File(temp.getLocationURI());
	}
	
	private void transfer(Map<String, File> map, 
			String name, 
			IFolder destDir, 
			IProgressMonitor arg0) throws CoreException, CorruptedInputData {
		
		File dest = getFileFromIFolder(name, destDir);
		
		File src = map.get(name);
		
		if (src == null || ! src.canRead()) {
			StringBuilder erbuf = new StringBuilder("Can not find or read the file to copy in\n\n");
			erbuf.append(dest.getAbsolutePath());
			throw new 
				CorruptedInputData(
						new Throwable() {private static final long serialVersionUID = -3171254238232499263L;}, 
						erbuf.toString());
			
		}
		copyFile(src, dest);
		
		arg0.worked(1);
	}
	
	private void copyFile(File srcFile, File dstFile) throws CoreException {
		try {
			final FileChannel srcChannel = new FileInputStream(srcFile).getChannel();
			final FileChannel dstChannel = new FileOutputStream(dstFile).getChannel();
			dstChannel.transferFrom(srcChannel, 0, srcChannel.size());
			srcChannel.close();
			dstChannel.close();
		}
		catch(IOException ioex) {
			StringBuilder buf = new StringBuilder("I/O error while copying from ");
			buf.append(srcFile.getPath());
			buf.append(" to ");
			buf.append(dstFile.getPath());
			IStatus status = new Status(IStatus.ERROR,Activator.PLUGIN_ID,buf.toString(),ioex);
			throw new CoreException(status);			

		}
	}
}
