/**

    Copyright 2007, 2008, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
 **/
package org.spagic3.ui.embed.webapp.ui;

import java.io.File;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;

public class IniFileChooser extends Dialog {

	File fileList[] = null;
	private int selection = -1;
	
	public int getSelection() {
		return selection;
	}

	public IniFileChooser(Shell parentShell) {
		super(parentShell);
	}

	public IniFileChooser(Shell parentShell, File elenco[]) {
		super(parentShell);
		this.fileList = elenco;
	}
	
	protected Control createContents(Composite parent) {
	    Composite composite = new Composite(parent, SWT.NO_REDRAW_RESIZE);

	    GridLayout glo = new GridLayout(1,true);
	    
	    glo.marginBottom = 5;
	    glo.marginTop = 5;
	    glo.marginLeft = 5;
	    glo.marginRight = 5;
	    
	    composite.setLayout(glo);
	    
	    Label message = new Label(composite, SWT.RIGHT | SWT.SHADOW_NONE);
	    GridData gd = new GridData();
	    gd.horizontalAlignment = GridData.CENTER;
	    message.setLayoutData(gd);
	    message.setText("Choose the appropriate target operative system:");
	    
	    Combo osList = new Combo(composite, SWT.READ_ONLY | SWT.DROP_DOWN);
	    
	    int m = fileList.length;
	    for(int i = 0; i < m; i++) {
	    	String s = fileList[i].getName();
	    	osList.add(s.substring(1 + s.lastIndexOf('.')));
	    }

	    gd = new GridData();
	    gd.horizontalAlignment = GridData.CENTER;
	    gd.grabExcessHorizontalSpace = true;
	    osList.setLayoutData(gd);
	    Composite subComposite = new Composite(composite, 0);
	    
	    GridLayout glo1 = new GridLayout(2,true);
	    glo1.marginLeft = 15;
	    glo1.marginRight = 15;
	    glo1.marginWidth = 10;
	    
	    subComposite.setLayout(glo1);
	    gd = new GridData();
	    gd.horizontalAlignment = GridData.CENTER;
	    subComposite.setLayoutData(gd);
	    
	    SelListener selListener = new SelListener();
	    	
        selListener.setList(osList);
	    
	    Button selectButton = new Button(subComposite,SWT.PUSH);
	    selectButton.setText("  Select  ");
	    selectButton.setToolTipText("Confirms the operating system selected in the list above.");
	    selectButton.addListener(SWT.Selection, selListener);
	    
	    Button abortButton = new Button(subComposite,SWT.PUSH);
	    abortButton.setText("  Abort  ");
	    abortButton.setToolTipText("Cancels the O.S. selection and aborts the embedding of Spagic S.M. in the selected project.");
	    abortButton.addListener(SWT.Selection, new Listener() {

			public void handleEvent(Event arg0) {
				boolean answer =
					MessageDialog.openQuestion(
							getShell(),
							"Confirm",
							"Really abort the embedding of Spagic Service Manager in this project ?");

				if(answer) {
					IniFileChooser.this.selection = -1;
					IniFileChooser.this.close();
				}
			}
	    	
	    });
		return composite;
	}

	class SelListener implements Listener {
		private Combo osList = null;

		public void setList(Combo osList) {
			this.osList = osList;
		}
		
		public void handleEvent(Event event) {

			if(osList.getSelectionIndex() == -1) {
				MessageDialog.openWarning(getShell(), "Select Target O.S.", "Please, select the target O.S. first!");
				return;
			}
			StringBuilder sb = new StringBuilder("Do you confirm ");
			sb.append(osList.getItem(osList.getSelectionIndex()));
			sb.append(" as target Operative System ?");
			boolean answer =
				MessageDialog.openQuestion(
						getShell(),
						"Confirm the selection",
						sb.toString());

			if(answer) {
				IniFileChooser.this.selection = osList.getSelectionIndex();
				IniFileChooser.this.close();

			}
		}
	}
	
}
