/**

    Copyright 2007, 2008, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
 **/
package org.spagic3.ui.embed.webapp.utils;

import java.io.File;
import java.io.FileFilter;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.preference.IPreferenceStore;
import org.spagic3.ui.embed.webapp.Activator;
import org.spagic3.utils.SpagicUtilsActivator;
import org.spagic3.utils.preference.SpagicPreferenceConstants;

/**
 * Describe class ProcessChecker here.
 *
 *
 * Created: Tue Jan  5 10:38:44 2010
 *
 * @author <a href="mailto:saint@eng.it">Gian Uberto Lauri</a>
 * @version $Revision$
 */
public class ProcessStarter {

	private String serviceManagerPath = null;
	private String spagicEmbeddingPath = null;
	private String spagicDeploymentPath = null;

	// these resources will be useful in the continuation...
	private IFolder webInfFolder;
	private File spagicEmbeddingDir = null;
	private File serviceManagerDir = null;
	private File serviceManagerPluginDir = null;
	private File spagicEmbeddedExtraJarDir = null;
	private File spagicEmbeddingIniFiles[] = null;
	private File spagicSimpleConfiguratorInfo = null;
	
	public File getSpagicSimpleConfiguratorInfo() {
		return spagicSimpleConfiguratorInfo;
	}

	public String getSpagicEmbeddingPath() {
		return spagicEmbeddingPath;
	}

	public File getSpagicEmbeddedJarDir() {
		return spagicEmbeddedExtraJarDir;
	}

	public File[] getSpagicEmbeddingIniFiles() {
		return spagicEmbeddingIniFiles;
	}

	public String getSpagicDeploymentPath() {
		return spagicDeploymentPath;
	}

	/**
	 * Creates a new instance of <code>ProcessChecker</code> .
	 *
	 */
	public ProcessStarter() {

		IPreferenceStore store = SpagicUtilsActivator.getDefault()
			.getPreferenceStore();

		serviceManagerPath = store.getString(SpagicPreferenceConstants.P_EMBEDDED_SERVICE_MANAGER_SOURCE_DIR);

		spagicEmbeddingPath = store.getString(SpagicPreferenceConstants.P_EXTRA_EMBEDDING_FILES_SOURCE_DIR);
		
		spagicDeploymentPath = store.getString(SpagicPreferenceConstants.P_DEPLOY_SERVICE_TARGET_DIR);
	}

	public ProcessCheckerResult checkProject(IProject project) {
		ProcessCheckerResult returnValue = new ProcessCheckerResult();


		if (serviceManagerPath == null || serviceManagerPath.trim().equals(""))
			returnValue.addClaim("Missing Service Manager Path property. Edit Spagic 3 preferences using Eclipse Preferences");

		else {
			serviceManagerDir = new File(serviceManagerPath);

			if(!(serviceManagerDir != null
				 && serviceManagerDir.exists()
				 && serviceManagerDir.isDirectory()
				 && serviceManagerDir.canRead()) )
				returnValue.addClaim(serviceManagerPath 
						+ " does not point to a readable directory");
	
			StringBuilder temp = new StringBuilder(serviceManagerPath);
			if(!serviceManagerPath.endsWith(File.separator))
				temp.append(File.separator);
			//temp.append("eclipse");
			//temp.append(File.separator);
			int here = temp.length();
			temp.append("plugins");
			serviceManagerPluginDir = new File( temp.toString());
			
			temp.delete(here, temp.length());
			temp.append("configuration");
			temp.append(File.separator);
			temp.append("org.eclipse.equinox.simpleconfigurator");
			temp.append(File.separator);
			temp.append("bundles.info");
			spagicSimpleConfiguratorInfo = new File(temp.toString());
		}

		if (spagicEmbeddingPath == null || spagicEmbeddingPath.trim().equals(""))
			returnValue.addClaim("Missing Spagic Embedding Path property. Edit Spagic 3 preferences using Eclipse Preferences");
		else {
			spagicEmbeddingDir = new File(spagicEmbeddingPath);

			if(!(spagicEmbeddingDir != null
				 && spagicEmbeddingDir.exists()
				 && spagicEmbeddingDir.isDirectory()
				 && spagicEmbeddingDir.canRead()) )
				returnValue.addClaim(spagicEmbeddingPath 
									 + " does not point to a readable directory");
			
			StringBuilder temp = new StringBuilder(spagicEmbeddingPath);
			if(!spagicEmbeddingPath.endsWith(File.separator))
				temp.append(File.separator);
			
			int tempLen = temp.length();
			
			temp.append("embedded-files");
			
			spagicEmbeddedExtraJarDir = new File( temp.toString() );

			if(!(spagicEmbeddedExtraJarDir != null
					 && spagicEmbeddedExtraJarDir.exists()
					 && spagicEmbeddedExtraJarDir.isDirectory()
					 && spagicEmbeddedExtraJarDir.canRead()) )
					returnValue.addClaim(spagicEmbeddedExtraJarDir 
										 + " (calculated from the Spagic embedding path) does not point to a readable directory");

			temp.delete(tempLen, temp.length());
			
			temp.append("ini-files");
			File smIniFilesDir = new File ( temp.toString() );
			
			if(!(smIniFilesDir != null
					 && smIniFilesDir.exists()
					 && smIniFilesDir.isDirectory()
					 && smIniFilesDir.canRead()) )
					returnValue.addClaim(smIniFilesDir 
										 + " (calculated from the Spagic embedding path) does not point to a readable directory");
		
			spagicEmbeddingIniFiles = smIniFilesDir.listFiles( new FileFilter() {
				public boolean accept(File f) {
					return f.getName().startsWith("eclipse.ini.");
				}
			});
			
			if(spagicEmbeddingIniFiles == null || spagicEmbeddingIniFiles.length == 0)
				returnValue.addClaim("No ini file found.");
		}


		try {
			if(! checkProcessDir(project) )
				returnValue.addClaim("Selected process is not a valid web application");
		} catch (CoreException e) {
			IStatus status = e.getStatus();
			StringBuilder buf = new StringBuilder("A CoreException occurred while searching the WEB-INF folder: ");
			buf.append(status.getMessage());
			buf.append("(code=");
			buf.append(status.getCode());
			buf.append("). Reported by plugin: ");
			buf.append(status.getPlugin());
			buf.append(" and caused by exception ");
			buf.append(status.getException().getClass().getName());
			buf.append(": ");
			buf.append(status.getException().getMessage());
			Activator.getDefault().getLog().log(status);
			returnValue.addClaim(buf.toString());
		}

		if (spagicDeploymentPath == null || spagicDeploymentPath.trim().equals(""))
			returnValue.addClaim("Missing Spagic Deployment Directory property. Edit Spagic 3 preferences using Eclipse Preferences");

		else {
			File foobar = new File(spagicDeploymentPath);

			if(!(foobar != null
				 && foobar.exists()
				 && foobar.isDirectory()
				 && foobar.canRead()) )
				returnValue.addClaim(spagicDeploymentPath 
						+ " does not point to a readable directory");
		}

		
		returnValue.setAllRightIfNoClaim();
		return returnValue;
	}

	public File getServiceManagerPluginDir() {
		return serviceManagerPluginDir;
	}

	private boolean checkProcessDir(IProject project) throws CoreException {

		webInfFolder = findWebInf(project);

		return webInfFolder != null;

	}

	private IFolder findWebInf(IContainer resource) throws CoreException {

		IResource members[] = resource.members();
		int count = members.length;

		for(int i = 0; i < count; i++) {
			if(members[i] instanceof IContainer) {
				if(members[i] instanceof IFolder) {
					if ("WEB-INF".equals(members[i].getName()))
						return (IFolder)members[i];
				}
				IFolder rf = findWebInf((IContainer)members[i]);
				if( rf != null)
					return rf;
			}
		}

		return null;
	}


	public IFolder getWebInfFolder() {
		return webInfFolder;
	}

	public File getSpagicEmbeddingDir() {
		return spagicEmbeddingDir;
	}

	public File getServiceManagerDir() {
		return serviceManagerDir;
	}
}
