/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.ui.metadataeditor.editors;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.ui.forms.DetailsPart;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.MasterDetailsBlock;
import org.eclipse.ui.forms.SectionPart;
import org.eclipse.ui.forms.editor.FormPage;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.ScrolledForm;
import org.eclipse.ui.forms.widgets.Section;
import org.spagic3.ui.metadataeditor.Activator;
import org.spagic3.ui.metadataeditor.MetadataEditorInput;
import org.spagic3.ui.metadataeditor.model.ColumnDefinition;
import org.spagic3.ui.metadataeditor.model.FieldDefinition;
import org.spagic3.ui.metadataeditor.model.MetadataDefinition;
import org.spagic3.ui.metadataeditor.model.IModelListener;
import org.spagic3.ui.metadataeditor.model.IModelPart;
import org.spagic3.ui.metadataeditor.model.ItemDefinition;
import org.spagic3.ui.metadataeditor.model.ModelChangeType;
import org.spagic3.ui.metadataeditor.model.NamedModelPart;
import org.spagic3.ui.metadataeditor.model.TableDefinition;
/**
 *
 */
public class ScrolledPropertiesBlock extends MasterDetailsBlock implements IModelListener {
	
	private FormPage page;
	private TreeViewer viewer;
	
	private Button addFieldButton;
	private Button addTableButton;
	private Button addColumnButton;
	private Button removeButton;
	private Button moveUpButton;
	private Button moveDownButton;

	public ScrolledPropertiesBlock(FormPage page) {
		this.page = page;
	}
	
	/**
	 * @param id
	 * @param title
	 */
	class MasterContentProvider implements ITreeContentProvider {

		public Object[] getChildren(Object parentElement) {
			if (parentElement instanceof MetadataDefinition) {
				final MetadataDefinition metadataDefinition = (MetadataDefinition) parentElement;
				return metadataDefinition.getParts();
			} else if (parentElement instanceof TableDefinition) {
				final TableDefinition tableDefinition = (TableDefinition) parentElement;
				return tableDefinition.getColumns();
			}
			return null;
		}

		public Object getParent(Object element) {
			if (element instanceof MetadataDefinition) {
				return null;
			} else {
				final IModelPart part = (IModelPart) element;
				return part.getParent();
			}
		}

		public boolean hasChildren(Object element) {
			if (element instanceof MetadataDefinition) {
				return ((MetadataDefinition) element).getParts().length > 0;
			} else if (element instanceof TableDefinition) {
				return ((TableDefinition) element).getColumns().length > 0;
			} else {
				return false;
			}
		}
		
		public Object[] getElements(Object inputElement) {
			if (inputElement instanceof MetadataEditorInput) {
				//TODO check this way
//				FormEditorInput input = (FormEditorInput) page
//						.getEditor().getEditorInput();
				MetadataEditorInput input = (MetadataEditorInput) inputElement;
				return input.getModel().getParts();
			}
			return new Object[0];
		}
		
		public void dispose() {
		}
		
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		}

	}
	
	class MasterLabelProvider extends LabelProvider {
		
		public Image getImage(Object element) {
			if (element instanceof MetadataDefinition) {
				return Activator.getDefault().getImage(Activator.IMG_METADATA);
			} else if (element instanceof FieldDefinition) {
				return Activator.getDefault().getImage(Activator.IMG_FIELD);
			} else if (element instanceof TableDefinition) {
				return Activator.getDefault().getImage(Activator.IMG_TABLE);
			} else if (element instanceof ColumnDefinition) {
				return Activator.getDefault().getImage(Activator.IMG_COLUMN);
			} else {
				return null;
			}
		}
		
		public String getText(Object element) {
			if (element instanceof MetadataDefinition) {
				final MetadataDefinition metadataDefinition = (MetadataDefinition) element;
				return metadataDefinition.getId();
			} else if (element instanceof NamedModelPart && !(element instanceof TableDefinition)) {
				final NamedModelPart part = (NamedModelPart) element;
				return part.getName();
			}
			return "<anonymous>";
		}
		
	}
	
	protected void createMasterPart(final IManagedForm managedForm,
			Composite parent) {
		//final ScrolledForm form = managedForm.getForm();
		FormToolkit toolkit = managedForm.getToolkit();
		Section section = toolkit.createSection(parent, Section.DESCRIPTION|Section.TITLE_BAR);
		section.setText("Metadata definition parts");
		section.marginWidth = 10;
		section.marginHeight = 5;

		Composite client = toolkit.createComposite(section, SWT.WRAP);
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		layout.marginWidth = 2;
		layout.marginHeight = 2;
		client.setLayout(layout);
		
		Tree tree = toolkit.createTree(client, SWT.NULL);
		GridData gd = new GridData(GridData.FILL_BOTH);
		gd.heightHint = 20;
		gd.widthHint = 100;
		tree.setLayoutData(gd);
		toolkit.paintBordersFor(client);
		
		Composite buttons = toolkit.createComposite(client);
		gd = new GridData(GridData.VERTICAL_ALIGN_BEGINNING);
		buttons.setLayoutData(gd);
		buttons.setLayout(new FillLayout(SWT.VERTICAL));

		section.setClient(client);
		final SectionPart spart = new SectionPart(section);
		managedForm.addPart(spart);

		addFieldButton = toolkit.createButton(buttons, "Add Field", SWT.PUSH);
		addFieldButton.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {
			}
			public void widgetSelected(SelectionEvent e) {
				IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();
				if (selection.size() == 1) {
					final IModelPart selected = (IModelPart) selection.getFirstElement();
					if (selected instanceof MetadataDefinition) {
						final MetadataDefinition metadataDefinition = (MetadataDefinition) selected;
						final FieldDefinition fieldDefinition = new FieldDefinition();
						metadataDefinition.addPart(fieldDefinition);
						metadataDefinition.getModel().fireModelChanged(new Object[]{fieldDefinition, metadataDefinition}, ModelChangeType.CHANGE_ADD);
						viewer.setExpandedState(metadataDefinition, true);
					}
				}
			}
		});
		
		addTableButton = toolkit.createButton(buttons, "Add Table", SWT.PUSH);
		addTableButton.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {
			}
			public void widgetSelected(SelectionEvent e) {
				IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();
				if (selection.size() == 1) {
					final IModelPart selected = (IModelPart) selection.getFirstElement();
					if (selected instanceof MetadataDefinition) {
						final MetadataDefinition metadataDefinition = (MetadataDefinition) selected;
						final TableDefinition tableDefinition = new TableDefinition();
						metadataDefinition.addPart(tableDefinition);
						metadataDefinition.getModel().fireModelChanged(new Object[]{tableDefinition, metadataDefinition}, ModelChangeType.CHANGE_ADD);
						addTableButton.setEnabled(
								shouldEnableAddTableButton(metadataDefinition));
					}
				}
			}
		});

		addColumnButton = toolkit.createButton(buttons, "Add Column", SWT.PUSH);
		addColumnButton.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {
			}
			public void widgetSelected(SelectionEvent e) {
				IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();
				if (selection.size() == 1) {
					final IModelPart selected = (IModelPart) selection.getFirstElement();
					if (selected instanceof TableDefinition) {
						final TableDefinition tableDefinition = (TableDefinition) selected;
						final ColumnDefinition columnDefinition = new ColumnDefinition();
						tableDefinition.addColumn(columnDefinition);
						tableDefinition.getModel().fireModelChanged(new Object[]{columnDefinition, tableDefinition}, ModelChangeType.CHANGE_ADD);
						viewer.setExpandedState(tableDefinition, true);
					}
				}
			}
		});

		removeButton = toolkit.createButton(buttons, "Remove", SWT.PUSH);
		removeButton.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {
			}
			public void widgetSelected(SelectionEvent e) {
				IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();
				if (selection.size() == 1) {
					final IModelPart selected = (IModelPart) selection.getFirstElement();
					if (selected instanceof MetadataDefinition) {
						//do not remove metadata
					} else if (selected instanceof FieldDefinition) {
						final MetadataDefinition parent = (MetadataDefinition) selected.getParent();
						parent.removePart(selected);
						parent.getModel().fireModelChanged(new Object[]{selected, parent}, ModelChangeType.CHANGE_REMOVE);
					} else if (selected instanceof TableDefinition) {
						final MetadataDefinition parent = (MetadataDefinition) selected.getParent();
						parent.removePart(selected);
						parent.getModel().fireModelChanged(new Object[]{selected, parent}, ModelChangeType.CHANGE_REMOVE);
						addTableButton.setEnabled(
								shouldEnableAddTableButton(parent));
					} else if (selected instanceof ColumnDefinition) {
						final TableDefinition parent = (TableDefinition) selected.getParent();
						parent.removeColumn((ColumnDefinition) selected);
						parent.getModel().fireModelChanged(new Object[]{selected, parent}, ModelChangeType.CHANGE_REMOVE);
					}
				}
			}
		});
	
		moveUpButton = toolkit.createButton(buttons, "Move Up", SWT.PUSH);
		moveUpButton.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {
			}
			public void widgetSelected(SelectionEvent e) {
				IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();
				if (selection.size() == 1) {
					final IModelPart selected = (IModelPart) selection.getFirstElement();
					if (selected instanceof MetadataDefinition) {
						//do not move metadata by now
					} else if (selected instanceof FieldDefinition) {
						final MetadataDefinition parent = (MetadataDefinition) selected.getParent();
						parent.movePartUp(selected); //new
						parent.getModel().fireModelChanged(new Object[]{selected, parent}, ModelChangeType.MOVE_UP);
					} else if (selected instanceof TableDefinition) {
						final MetadataDefinition parent = (MetadataDefinition) selected.getParent();
						parent.movePartUp(selected); //new
						parent.getModel().fireModelChanged(new Object[]{selected, parent}, ModelChangeType.MOVE_UP);
//					} else if (selected instanceof ColumnDefinition) {
//						final TableDefinition parent = (TableDefinition) selected.getParent();
//						parent.moveColumnUp((ColumnDefinition) selected); //metodo nuovo
//						parent.getModel().fireModelChanged(new Object[]{selected, parent}, ModelChangeType.MOVE_UP);
					}
				}
			}
		});
	
		moveDownButton = toolkit.createButton(buttons, "Move Down", SWT.PUSH);
		moveDownButton.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {
			}
			public void widgetSelected(SelectionEvent e) {
				IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();
				if (selection.size() == 1) {
					final IModelPart selected = (IModelPart) selection.getFirstElement();
					if (selected instanceof MetadataDefinition) {
						//do not move metadata by now
					} else if (selected instanceof FieldDefinition) {
						final MetadataDefinition parent = (MetadataDefinition) selected.getParent();
						parent.movePartDown(selected); //new
						parent.getModel().fireModelChanged(new Object[]{selected, parent}, ModelChangeType.MOVE_DOWN);
					} else if (selected instanceof TableDefinition) {
						final MetadataDefinition parent = (MetadataDefinition) selected.getParent();
						parent.movePartDown(selected); //new
						parent.getModel().fireModelChanged(new Object[]{selected, parent}, ModelChangeType.MOVE_DOWN);
//					} else if (selected instanceof ColumnDefinition) {
//						final TableDefinition parent = (TableDefinition) selected.getParent();
//						parent.moveColumnDown((ColumnDefinition) selected); //metodo nuovo
//						parent.getModel().fireModelChanged(new Object[]{selected, parent}, ModelChangeType.MOVE_DOWN);
					}
				}
			}
		});
	
		
		
		viewer = new TreeViewer(tree);
		viewer.addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				IStructuredSelection selection = (IStructuredSelection) event.getSelection();
				if (selection.size() == 1) {
					final IModelPart selected = (IModelPart) selection.getFirstElement();
					if (selected instanceof MetadataDefinition) {
						addFieldButton.setEnabled(true);
						addTableButton.setEnabled(
								shouldEnableAddTableButton((MetadataDefinition) selected));
						addColumnButton.setEnabled(false);
						removeButton.setEnabled(false);
						moveDownButton.setEnabled(false);
						moveUpButton.setEnabled(false);
					} else if (selected instanceof FieldDefinition) {
						addFieldButton.setEnabled(false);
						addTableButton.setEnabled(false);
						addColumnButton.setEnabled(false);
						removeButton.setEnabled(true);
						
						moveDownButton.setEnabled(true);
						moveUpButton.setEnabled(true);
					} else if (selected instanceof TableDefinition) {
						addFieldButton.setEnabled(false);
						addTableButton.setEnabled(false);
						addColumnButton.setEnabled(true);
						removeButton.setEnabled(true);
						
						moveDownButton.setEnabled(true);
						moveUpButton.setEnabled(true);
					} else if (selected instanceof ColumnDefinition) {
						addFieldButton.setEnabled(false);
						addTableButton.setEnabled(false);
						addColumnButton.setEnabled(false);
						removeButton.setEnabled(true);
						
						moveDownButton.setEnabled(false);
						moveUpButton.setEnabled(false);
					}
				} else {
					addFieldButton.setEnabled(false);
					addTableButton.setEnabled(false);
					addColumnButton.setEnabled(false);
					removeButton.setEnabled(false);
					moveDownButton.setEnabled(false);
					moveUpButton.setEnabled(false);
				}
				managedForm.fireSelectionChanged(spart, event.getSelection());
			}
		});
		viewer.setContentProvider(new MasterContentProvider());
		viewer.setLabelProvider(new MasterLabelProvider());
		viewer.setInput(((MetadataEditor) page.getEditor()).getMetadataEditorInput());
		
		((MetadataEditor) page.getEditor()).getMetadataEditorInput()
				.getModel().addModelListener(this);
	}
	
	private boolean shouldEnableAddTableButton(MetadataDefinition metadataDefinition) {
		boolean containsATable = false;
		for (IModelPart metadataPart : metadataDefinition.getParts()) {
			if (metadataPart instanceof TableDefinition) {
				containsATable = true;
				break;
			}
		}
		return !containsATable;
	}
	
	public void modelChanged(Object[] objects, ModelChangeType type) {
		if ((type != ModelChangeType.CHANGE_PROPERTY 
						&& !(objects[0] instanceof ItemDefinition))
				|| (type == ModelChangeType.CHANGE_PROPERTY 
						&& ("id".equals(objects[1])
							|| "name".equals(objects[1])))) {
						viewer.refresh();
		}
	}
	
	protected void createToolBarActions(IManagedForm managedForm) {
		final ScrolledForm form = managedForm.getForm();
		Action haction = new Action("hor", Action.AS_RADIO_BUTTON) {
			public void run() {
				sashForm.setOrientation(SWT.HORIZONTAL);
				form.reflow(true);
			}
		};
		haction.setChecked(true);
		haction.setToolTipText("flip horizontal");
		haction.setImageDescriptor(Activator.getDefault()
				.getImageDescriptor(Activator.IMG_HORIZONTAL));
		Action vaction = new Action("ver", Action.AS_RADIO_BUTTON) {
			public void run() {
				sashForm.setOrientation(SWT.VERTICAL);
				form.reflow(true);
			}
		};
		vaction.setChecked(false);
		vaction.setToolTipText("flip vertical");
		vaction.setImageDescriptor(Activator.getDefault()
				.getImageDescriptor(Activator.IMG_VERTICAL));
		form.getToolBarManager().add(haction);
		form.getToolBarManager().add(vaction);
	}
	
	protected void registerPages(DetailsPart detailsPart) {
		detailsPart.registerPage(MetadataDefinition.class, new MetadataDefinitionDetailPage());
		detailsPart.registerPage(FieldDefinition.class, new InputModelPartDetailsPage());
		detailsPart.registerPage(TableDefinition.class, new NoPropertiesDetailsPage());
		detailsPart.registerPage(ColumnDefinition.class, new InputModelPartDetailsPage());
	}
	
}