/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.ui.resources;

import java.util.List;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Node;
import org.dom4j.XPath;
import org.dom4j.xpath.DefaultXPath;
import org.eclipse.stp.im.resources.IImResource;
import org.eclipse.stp.im.resources.IImResourceConfigurator;
import org.eclipse.stp.im.resources.datasources.DataSourceImResource;
import org.spagic3.utils.SpagicUtilsActivator;
import org.spagic3.utils.preference.SpagicPreferenceConstants;

public class Spagic3ResourceConfigurator extends Spagic3ResourceManager implements IImResourceConfigurator {
	
	public Spagic3ResourceConfigurator(){
		initNamespaces();
	}
	
	public String deploy(List<IImResource> resourceList, String resourceType) {
		String deploymentPaylod =  prepareDeploymentPayload(resourceList);
		String result = invoke(deploymentPaylod);
		return normalizeResult(result);
	}
	
	public String normalizeResult(String result){
		StringBuffer sb = new StringBuffer();
		sb.append("Deployment Results \n");
		sb.append("\n");
		
		try{
			Document dom = DocumentHelper.parseText(result);
			
			XPath xPathExpression = new DefaultXPath("/soapenv:Envelope/soapenv:Body/DEPLOYMENT_RESULT/DATASOURCE_DEPLOY_RESULT");
			xPathExpression.setNamespaceURIs(this.namespaces);
			List<Node> dsNodes = xPathExpression.selectNodes(dom);
      
			for (Node dsNode : dsNodes){
				sb.append("Datasource ["+dsNode.valueOf("@name")+"] - ["+normalizeStatus(dsNode.valueOf("@status"))+"]\n");
				sb.append("\n");
			}
			
		}catch (Exception e) {
			sb.append("Errors ["+e.getMessage()+"]\n");
		}
		
		return sb.toString();
	}

	protected String normalizeStatus(String aString){
		return aString.replace("_", " ");
	}

	public String getConfiguration(List<IImResource> resourceList,
			String resourceType) {
		return notYetImplemented();
	}

	protected String notYetImplemented(){
		return "<!-- NOT YET IMPLEMENTED -->";
	}
	
	public boolean isDeploySupported() {
		return true;
	}

	

	protected String getURL() {
		String strURL = SpagicUtilsActivator.getDefault().getPluginPreferences().getString(SpagicPreferenceConstants.P_SERVICE_MANAGER_RESOURCE_DEPLOY_SERVICE_URL);
		return strURL;
	}
	
	protected String prepareDeploymentPayload(List<IImResource> resourceList){
		StringBuffer sb = new StringBuffer();
		sb.append("<DEPLOY_DATASOURCES>");
		for (IImResource res :resourceList){
			if (res instanceof DataSourceImResource){
				sb.append("<DATASOURCE \n");
				sb.append("name=\""+((DataSourceImResource)res).getId()+"\"\n");
				sb.append("url=\""+res.getProperty(DataSourceImResource.DATASOURCE_URL)+"\"\n");
				sb.append("driver=\""+res.getProperty(DataSourceImResource.DATASOURCE_DRIVER)+"\"\n");
				sb.append("user=\""+res.getProperty(DataSourceImResource.DATASOURCE_USER)+"\"\n");
				sb.append("password=\""+res.getProperty(DataSourceImResource.DATASOURCE_PASSWORD)+"\"\n");
				sb.append("/>");
			}
		}
		sb.append("</DEPLOY_DATASOURCES>");
		return sb.toString();
	}
	
}
