/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.ui.resources;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.RequestEntity;
import org.apache.commons.httpclient.methods.StringRequestEntity;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Node;
import org.dom4j.XPath;
import org.dom4j.xpath.DefaultXPath;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.stp.im.resources.IImResource;
import org.eclipse.stp.im.resources.datasources.DataSourceImResource;
import org.eclipse.swt.widgets.Shell;
import org.spagic3.ui.SpagicUIActivator;
import org.spagic3.utils.SpagicLogger;
import org.spagic3.utils.SpagicUtilsActivator;
import org.spagic3.utils.preference.SpagicPreferenceConstants;

public abstract class Spagic3ResourceManager {
	
	protected Map<String, String> namespaces = null;
	
	protected void initNamespaces(){
		
		this.namespaces = new HashMap<String, String>();
		this.namespaces.put("jencks","http://jencks.org/2.0"); 
		this.namespaces.put("amq","http://activemq.org/config/1.0");
		this.namespaces.put("amqra","http://activemq.org/ra/1.0");
		this.namespaces.put("util","http://www.springframework.org/schema/util");
		this.namespaces.put("soapenv","http://schemas.xmlsoap.org/soap/envelope/");
		this.namespaces.put("xsd","http://www.w3.org/2001/XMLSchema"); 
		this.namespaces.put("xsi","http://www.w3.org/2001/XMLSchema-instance");
	
	}
	
	
	
	private String getSpagicPayload(String xmlString){
		return "<spagic:payload xmlns:spagic=\"urn:eng:spagic\">"+xmlString+"</spagic:payload>";
	}

	private String getSOAP(String xmlString){
		StringBuffer sb = new StringBuffer();
		sb.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
		sb.append("<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">");
		sb.append("<soapenv:Body>");
		sb.append(getSpagicPayload(xmlString));
		sb.append("</soapenv:Body>");
		sb.append("</soapenv:Envelope>");
		return sb.toString();
	}
	
	protected abstract String getURL();
	
	
	
	protected String getDeployURL(){
		String strURL = SpagicUtilsActivator.getDefault().getPluginPreferences().getString(SpagicPreferenceConstants.P_SERVICE_MANAGER_RESOURCE_DEPLOY_SERVICE_URL);
		return strURL;
	}
	
	protected String invoke(String xmlString){
		PostMethod post = new PostMethod(getURL());
		
		
		String soapResponse = null;
        RequestEntity entity = new StringRequestEntity(getSOAP(xmlString));
        post.setRequestEntity(entity);
        // consult documentation for your web service
        post.setRequestHeader("SOAPAction", "");
        // Get HTTP client
        org.apache.commons.httpclient.HttpClient httpclient = new org.apache.commons.httpclient.HttpClient();
        // Execute request
        try {
            int result = httpclient.executeMethod(post);
             soapResponse = post.getResponseBodyAsString();
        }catch (Exception e) {
        	MessageDialog.openWarning(new Shell(), "Spagic3ResourceManager", " Error in Spagic Service Manager Conversation ");
			SpagicLogger.error(SpagicUIActivator.PLUGIN_ID,e.getMessage(), e);
        } finally {
            // Release current connection to the connection pool once you are done
            post.releaseConnection();
        }
        return soapResponse;
	}
	
	public List<IImResource> getDatasourceListFromResponse(String response){
		
		List<IImResource> listDsBean = new ArrayList<IImResource>();
		try{
			Document dom = DocumentHelper.parseText(response);
    
    
			XPath xPathExpression = new DefaultXPath("/soapenv:Envelope/soapenv:Body/DATASOURCES/DATASOURCE");
		
			xPathExpression.setNamespaceURIs(this.namespaces);
		
			List<Node> dsNodes = xPathExpression.selectNodes(dom);
       
			IImResource res = null;
      
			for (Node dsNode : dsNodes){
				res = new DataSourceImResource(dsNode.valueOf("@name"));
				res.setProperty(DataSourceImResource.DATASOURCE_URL,dsNode.valueOf("@url"));
				res.setProperty(DataSourceImResource.DATASOURCE_DRIVER, dsNode.valueOf("@driver"));
				res.setProperty(DataSourceImResource.DATASOURCE_USER,dsNode.valueOf("@user"));
				res.setProperty(DataSourceImResource.DATASOURCE_PASSWORD,dsNode.valueOf("@password"));
				listDsBean.add(res);
			}
		}catch (Exception e) {
			MessageDialog.openWarning(new Shell(), "Spagic3ResourceManager", " Error in Parsing Response From Service Manager ");
			SpagicLogger.error(SpagicUIActivator.PLUGIN_ID,e.getMessage(), e);
		}
   
        return listDsBean;
	}
	
}
