package org.spagic3.ui.serviceeditor.actions;


import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jet.JET2Platform;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.IEditorActionDelegate;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.spagic3.ui.serviceeditor.editors.ConnectorEditor;


/**
 * Class <code>GenerateWSDLAction</code> Generates a WSDL for a given
 * HTTP Server connector
 *
 * @author <a href="mailto:andrea.zoppello@eng.it">Andrea Zoppello</a>
 * @author <a href="mailto:saint@eng.it">Gian Uberto Lauri</a>
 * @version $Revision$
 */
public class GenerateWSDLAction extends Action implements IEditorActionDelegate {

	// these are fixed

	static final String GENERATE_WSDL = "Generate WSDL";
	static final String ACTION_ID = "org.spagic3.ui.serviceeditor.actions.genWsdl";

	@Override
	public String getId() {

		return ACTION_ID;
	}

	// This holds as long as the HTTP server have
	// spagic3.httpserverfactory as factory.
	private static final Pattern matchHttpServerConnector =
		Pattern.compile("<spagic:component\\s*xmlns:spagic=\"([^\"]+)\"\\s*xmlns=\"([^\"]+)\"\\s*spagic.id=\"([^\"]*)\"\\s*factory.name=\"spagic3.httpserverfactory\"[^>]*>");

	// These hold as long as the url is in locationURI property, MEP in mep property and
	// the WSDL file in the wsdl property. The latter has no grouping as we will replace the
	// entire tag...
	private static final Pattern getURL = 
		Pattern.compile("<property\\s+name=\"locationURI\"\\s+value=\"([^\"]*)\"[^>]*/>");

	private static final Pattern getMep =
		Pattern.compile("<property\\s+name=\"mep\"\\s+value=\"([^\"]*)\"[^>]*/>");


	private static final Pattern getWsdl = 
		Pattern.compile("<property\\s+name=\"wsdl\"\\s+value=\"[^\"]*\"[^>]*/>");

	/** the selected file. */
	private IFile imFile = null;


	public IFile getImFile() {
		return imFile;
	}

	public void setImFile(IFile imFile) {
		this.imFile = imFile;
	}

	ConnectorEditor editor;
	/**
	 * Constructor for GenerateJarFileAction.
	 */
	public GenerateWSDLAction() {
		super();
	}

	/**
	 * the getter fileselected function.
	 * 
	 * @return IFile
	 */
	public final IFile getFileSelected() {
		return imFile;
	}

	/**
	 * The setter file selected function.
	 * 
	 * @param filePath :
	 *            the file
	 */
	public final void setFileSelected(final IFile filePath) {
		this.imFile = filePath;
	}

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 * @param action
	 *            the action proxy that handles the presentation portion of the
	 *            action
	 * @param targetPart
	 *            the new part target
	 */
	public final void setActivePart(final IAction action,
			final IWorkbenchPart targetPart) {
	}


	/**
	 * @see IActionDelegate#run(IAction)
	 */
	public void run(IAction action) {
		final Shell shell = editor.getShell();

		//		IProgressService ps = PlatformUI.getWorkbench().getProgressService();
		IProgressMonitor monitor = new NullProgressMonitor();
		try {
			WorkspaceModifyOperation wmop = new WorkspaceModifyOperation() {


				@Override
				protected void execute(IProgressMonitor monitor) throws CoreException,
				InvocationTargetException, InterruptedException {
					monitor.beginTask("WSDL Generation", 5);

					// 1) read the file
					String xmlnsSpagic=null;
					String xmlns = null;
					String connectorId = null;
					String connectorUrl = null;
					String connectorMep = null;
					StringBuilder wsdlFileName = null;
					StringBuilder sb = null;

					sb = new StringBuilder(editor.getXmlText());

					if(sb.indexOf("<property name=\"isSoap\" value=\"true\"/>") < 0) {
						MessageDialog.openError(shell, "Spagic", "The selected is not a valid HTTP Server connector, or the SOAP checkbox has not be checked.");
						return;
					}

					Matcher matcher = matchHttpServerConnector.matcher(sb);
					// 2) check if factory.name attribute value is "spagic3.httpserverfactory"
					if(matcher.find()) {
						xmlnsSpagic = matcher.group(1);
						xmlns = matcher.group(2);

						// 3) extract variables:
						//    URL <- locationuri
						//    mepValue <- mep
						//    portName <- id

						connectorId = matcher.group(3);
						wsdlFileName = new StringBuilder("wsdl://");
						wsdlFileName.append(connectorId);
						wsdlFileName.append(".wsdl");
						if(connectorId == null || connectorId.trim().equals("")) {
							MessageDialog.openError(shell, "Spagic", "The selected is not a valid HTTP Server connector, missing ID value");
							return;
						}

						matcher = getURL.matcher(sb);

						if(matcher.find()) {
							connectorUrl = matcher.group(1);

							if(connectorUrl == null ||
									connectorUrl.trim().equals("")) {
								MessageDialog.openError(shell, "Spagic", "The selected is not a valid HTTP Server connector, missing URL value");
								return;
							}

						} else {
							MessageDialog.openError(shell, "Spagic", "The selected is not a valid HTTP Server connector, missing URL value");
							return;
						} 


						matcher = getMep.matcher(sb);

						if(matcher.find()) {
							connectorMep = matcher.group(1);

							if(connectorMep == null ||
									connectorMep.trim().equals("")) {
								MessageDialog.openError(shell, "Spagic", "The selected is not a valid HTTP Server connector, missing MEP value");
								return;
							}

						} else {
							MessageDialog.openError(shell, "Spagic", "The selected is not a valid HTTP Server connector, missing MEP value");
							return;
						} 
						monitor.worked(1);
					}
					else {
						//    2.bis) if not return, not an HTTP Server connector
						MessageDialog.openError(shell, "Spagic", "The selected connector is not an HTTP Server.");
						return;
					}

					// Read the bpmn emf model
					//			ResourceSet resourceSetIM = new ResourceSetImpl();
					//			URI uri = URI.createPlatformResourceURI(imFile.getFullPath().toString(),true);
					//
					//			// Load On Demand = true 
					//			Resource resourceIM = resourceSetIM.getResource(uri, true);

					Map<String, String> variables = new HashMap<String, String>();

					// this is right, we need to know the process name to generate
					// the wsdl in the right position.
					variables.put("org.eclipse.jet.resource.project.name",editor.getFile().getProject().getName());

					// Let's add the values we got from the connector
					variables.put("org.eclipse.jet.resource.connector.xmlnsSpagic", xmlnsSpagic);
					variables.put("org.eclipse.jet.resource.connector.xmlns", xmlns);
					variables.put("org.eclipse.jet.resource.connector.connectorId", connectorId);
					variables.put("org.eclipse.jet.resource.connector.connectorUrl", connectorUrl);
					variables.put("org.eclipse.jet.resource.connector.connectorMep",connectorMep);
					variables.put("org.eclipse.jet.resource.connector.wsdlFileName", wsdlFileName.toString());
					xmlnsSpagic=null;
					xmlns = null;
					connectorId = null;
					connectorUrl = null;
					connectorMep = null;

					IStatus status = JET2Platform.runTransformOnObject("org.spagic3.im.out.wsdl", 
							null, variables, monitor);
					monitor.worked(1);
					switch (status.getSeverity()) {
					case IStatus.OK:
					case IStatus.INFO:

						StringBuilder pw = new StringBuilder(sb.length()+100);
						matcher = getWsdl.matcher(sb);

						int start;
						int end;
						if(matcher.find()) {
							String toBeReplaced = matcher.group();
							start = sb.indexOf(toBeReplaced);
							end = start + toBeReplaced.length();
						}
						else {
							start = sb.indexOf("</spagic:component>");
							end = start;
						}

						pw.append(sb.substring(0,start));
						if(end == start)
							pw.append("\t");
						pw.append("<property name=\"wsdl\" value=\"");
						pw.append(wsdlFileName);
						pw.append("\"/>");
						if(end == start)
							pw.append("\n");
						pw.append(sb.substring(end));
						editor.update(pw.toString(), monitor);

						MessageDialog.openInformation(shell, "Spagic", "WSDL Generation Completed ");
						monitor.worked(1);
						break;
					case IStatus.WARNING:
						MessageDialog.openWarning(shell, "Spagic", "WSDL generation reported the warning:\n" + status.getMessage());
						break;
					case IStatus.ERROR:
						MessageDialog.openError(shell, "Spagic", "WSDL generation reported the error:\n" + status.getMessage());
						break;
					case IStatus.CANCEL:
						MessageDialog.openInformation(shell, "Spagic", "WSDL Generation Cancelled by user ");
						break;
					}
				}
			};
			wmop.run(monitor);

		} catch (InvocationTargetException e) {
			MessageDialog.openError(shell, "Spagic", "Plugin invovation problems:\n" + e.getMessage());
			return;
		} catch (InterruptedException e) {
			MessageDialog.openError(shell, "Spagic", "Plugin work interrupted:\n" + e.getMessage());
			return;
		}
		finally {
			monitor.done();
		}

	}



	public final void selectionChanged(final IAction action,
			final ISelection selection) {

		//		setImFile((IFile) ((StructuredSelection) selection)
		//				.getFirstElement());
	}

	@Override
	public boolean isEnabled() {
		return true;
	}

	//	@Override
	//	public int getStyle() { 
	//		return AS_DROP_DOWN_MENU;	
	//	}

	@Override
	public boolean isHandled() { 
		return true; 
	}

	public void setActiveEditor(IAction arg0, IEditorPart arg1) {
		editor = (ConnectorEditor)arg1;

	}

}
