/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.ui.serviceeditor.model;

import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Properties;


public class ServiceModel implements IServiceModel {
	
	private String spagicId;
	private String factoryName;
	private Properties properties = null;
	private Map<String, Map<String, Properties>> mapProperties = null;

	
	public ServiceModel() {
		properties = new Properties();
		mapProperties = new LinkedHashMap<String, Map<String,Properties>>();
	}

	
	public String getSpagicId() {
		return spagicId;
	}

	
	public void setSpagicId(String spagicId) {
		this.spagicId = spagicId;
	}


	public String getFactoryName() {
		return factoryName;
	}


	public void setFactoryName(String factoryName) {
		this.factoryName = factoryName;
	}
	
	
	public String get(String propertyName) {
		return (String)properties.get(propertyName);
	}
	
	
	//
	// Handling simple Properties 
	// 
	public void addProperty(String propertyName, String propertyValue) {
		this.properties.put(propertyName, propertyValue);
	}
	
	
	public void removeProperty(String propertyName) {
		this.properties.remove(propertyName);
	}

	
	//
	// Handling Map Properties
	// !!!! ATTENTION - CREATING A MAP PROPERTY DOES NOT IMPLY TO HAVE ENTRIES 
	//      FOR EXAMPLE - CREATE A QUERY PARAMETER MAP DOES NOT MEAN I'VE A PARAMETER
	//
	public void addPropertyMap(String mapPropertyName) {
			mapProperties.put(mapPropertyName, new LinkedHashMap<String, Properties>());
	}
	
	
	public void removePropertyMap(String mapPropertyName) {
		mapProperties.remove(mapPropertyName);
	}
	
	
	public void addEntryToPropertyMap(String mapPropertyName, String key,
			Properties props) {
		Map<String, Properties> toUpdate = mapProperties.get(mapPropertyName);
		toUpdate.put(key, props);	
	}
	
	
	public void removeEntryFromPropertyMap(String mapPropertyName, String key) {
		Map<String, Properties> toUpdate = mapProperties.get(mapPropertyName);
		toUpdate.remove(key);
	}

	
	public Properties getEntryForPropertyMap(String mapPropertyName, String key) {
		Map<String, Properties> mapProperty = mapProperties.get(mapPropertyName);
		return mapProperty == null ? null : mapProperty.get(key);
	}

	
	public Map<String, Map<String, Properties>> getMapProperties() {
		return mapProperties;
	}

	
	public Properties getProperties() {
		return properties;
	}

}
