/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.ui.wizards.bpmn;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.OperationHistoryFactory;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.xmi.XMIResource;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.gmf.runtime.common.core.command.CommandResult;
import org.eclipse.gmf.runtime.diagram.core.services.ViewService;
import org.eclipse.gmf.runtime.diagram.ui.resources.editor.util.DiagramFileCreator;
import org.eclipse.gmf.runtime.emf.commands.core.command.AbstractTransactionalCommand;
import org.eclipse.gmf.runtime.emf.core.GMFEditingDomainFactory;
import org.eclipse.gmf.runtime.notation.Diagram;
import org.eclipse.jface.operation.IRunnableContext;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.stp.bpmn.ActivityType;
import org.eclipse.stp.bpmn.BpmnDiagram;
import org.eclipse.stp.bpmn.Pool;
import org.eclipse.stp.bpmn.diagram.BpmnDiagramMessages;
import org.eclipse.stp.bpmn.diagram.edit.parts.ActivityEditPart;
import org.eclipse.stp.bpmn.diagram.edit.parts.PoolEditPart;
import org.eclipse.stp.bpmn.diagram.generation.impl.BPMNVisual2ProcessGenerator;
import org.eclipse.stp.bpmn.diagram.part.BpmnDiagramEditorPlugin;
import org.eclipse.stp.bpmn.diagram.part.BpmnDiagramEditorUtil;
import org.eclipse.stp.bpmn.policies.BpmnDiagramXYLayoutEditPolicy;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchWindow;
public class SpagicBPMNEditorUtil extends BpmnDiagramEditorUtil {
	
	
	    public static IFile createAndOpenDiagram(
	            DiagramFileCreator diagramFileCreator, IPath containerPath,
	            String fileName, InputStream initialContents, String kind,
	            IWorkbenchWindow window, IProgressMonitor progressMonitor,
	            boolean openEditor, boolean saveDiagram) {
	    	
	        	IFile diagramFile = SpagicBPMNEditorUtil.createNewDiagramFile(
	                diagramFileCreator, containerPath, fileName, initialContents,
	                kind, window.getShell(), progressMonitor);
	        	
	        return diagramFile;
	    }
	 
	/**
	* <p>
	* This method should be called within a workspace modify operation since it
	* creates resources.
	* </p>
	*
	* @notgenerated
	* @return the created file resource, or <code>null</code> if the file was
	* not created
	*/
	    @SuppressWarnings("unchecked") //$NON-NLS-1$
	public static IFile createNewDiagramFile(
	            DiagramFileCreator diagramFileCreator, IPath containerFullPath,
	            String fileName, InputStream initialContents, String kind,
	            Shell shell, IProgressMonitor progressMonitor) {
	        TransactionalEditingDomain editingDomain = GMFEditingDomainFactory.INSTANCE
	                .createEditingDomain();
	        ResourceSet resourceSet = editingDomain.getResourceSet();
	        progressMonitor.beginTask("Creating diagram and model files", 4); //$NON-NLS-1$
	        final IProgressMonitor subProgressMonitor = new SubProgressMonitor(
	                progressMonitor, 1);
	        final IFile diagramFile = diagramFileCreator.createNewFile(
	                containerFullPath, fileName, initialContents, shell,
	                new IRunnableContext() {
	                    public void run(boolean fork, boolean cancelable,
	                            IRunnableWithProgress runnable)
	                            throws InvocationTargetException,
	                            InterruptedException {
	                        runnable.run(subProgressMonitor);
	                    }
	                });
	        final Resource diagramResource = resourceSet
	                .createResource(URI.createPlatformResourceURI(diagramFile
	                        .getFullPath().toString()));
	        List affectedFiles = new ArrayList();
	        affectedFiles.add(diagramFile);
	 
	        IPath modelFileRelativePath = diagramFile.getFullPath()
	                .removeFileExtension().addFileExtension("bpmn"); //$NON-NLS-1$
	        IFile modelFile = diagramFile.getParent().getFile(
	                new Path(modelFileRelativePath.lastSegment()));
	        final Resource modelResource = resourceSet.createResource(URI
	                .createPlatformResourceURI(modelFile.getFullPath().toString()));
	        affectedFiles.add(modelFile);
	 
	        final String fNameWithoutExtension = fileName.substring(0, fileName.indexOf("."));
	        final String kindParam = kind;
	        AbstractTransactionalCommand command = new AbstractTransactionalCommand(
	                editingDomain, "Creating diagram and model", affectedFiles) { //$NON-NLS-1$
	            protected CommandResult doExecuteWithResult(
	                    IProgressMonitor monitor, IAdaptable info)
	                    throws ExecutionException {
	            
	             BpmnDiagram model = createInitialModelGen();
	             modelResource.getContents().add(model);
	             Diagram diagram = ViewService.createDiagram(model, kindParam,
	                        BpmnDiagramEditorPlugin.DIAGRAM_PREFERENCES_HINT);
	             diagramResource.getContents().add(diagram);
	                diagram.setName(diagramFile.getName());
	                diagram.setElement(model);
	             BPMNVisual2ProcessGenerator generator =
	             new BPMNVisual2ProcessGenerator(diagram);
	                Pool p = generator.addPool(fNameWithoutExtension,
	                 BpmnDiagramXYLayoutEditPolicy.DEFAULT_POOL_X_COORD,
	                 BpmnDiagramXYLayoutEditPolicy.DEFAULT_POOL_X_COORD,
	                 PoolEditPart.POOL_WIDTH,
	                 PoolEditPart.POOL_HEIGHT);
	                generator.addActivity(p, BpmnDiagramMessages.BpmnDiagramEditorUtil_task_default_name, ActivityType.TASK,
	                 ActivityEditPart.ACTIVITY_FIGURE_SIZE.width / 2,
	                 PoolEditPart.POOL_HEIGHT / 2 -
	                 ActivityEditPart.ACTIVITY_FIGURE_SIZE.height / 2,
	                 ActivityEditPart.ACTIVITY_FIGURE_SIZE.width,
	                 ActivityEditPart.ACTIVITY_FIGURE_SIZE.height);
	                generator.generateViews();
	                
	                try {
	                    Map options = new HashMap();
	                    options.put(XMIResource.OPTION_ENCODING, "UTF-8"); //$NON-NLS-1$
	                    modelResource.save(options);
	                    diagramResource.save(Collections.EMPTY_MAP);
	                } catch (IOException e) {
	 
	                    BpmnDiagramEditorPlugin.getInstance().logError(
	                            "Unable to store model and diagram resources", e); //$NON-NLS-1$
	                }
	                return CommandResult.newOKCommandResult();
	            }
	        };
	        
	        
	 
	        try {
	            OperationHistoryFactory.getOperationHistory().execute(command,
	                    new SubProgressMonitor(progressMonitor, 1), null);
	        } catch (ExecutionException e) {
	            BpmnDiagramEditorPlugin.getInstance().logError(
	                    "Unable to create model and diagram", e); //$NON-NLS-1$
	        }
	 
	        try {
	            modelFile.setCharset(
	                    "UTF-8", new SubProgressMonitor(progressMonitor, 1)); //$NON-NLS-1$
	        } catch (CoreException e) {
	            BpmnDiagramEditorPlugin.getInstance().logError(
	                    "Unable to set charset for model file", e); //$NON-NLS-1$
	        }
	        try {
	            diagramFile.setCharset(
	                    "UTF-8", new SubProgressMonitor(progressMonitor, 1)); //$NON-NLS-1$
	        } catch (CoreException e) {
	            BpmnDiagramEditorPlugin.getInstance().logError(
	                    "Unable to set charset for diagram file", e); //$NON-NLS-1$
	        }
	        try{
	        	
		        IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
				String projectName = modelFile.getProject().getName();
				IProject proj = root.getProject(projectName);
				
				
			
				IContainer container = proj.getFolder("Services");
				String bpmGatewayServiceID =  fNameWithoutExtension + "Orchestration";
				String bpmGatewayServiceFileName = bpmGatewayServiceID+".service";
				final IFile file = container.getFile(new Path(bpmGatewayServiceFileName));
				try {
					InputStream stream = openContentStreamForBPMGateway(bpmGatewayServiceID,bpmGatewayServiceFileName,fNameWithoutExtension);
					if (file.exists()) {
						file.setContents(stream, true, true,  new SubProgressMonitor(progressMonitor, 1));
					} else {
						file.create(stream, true,  new SubProgressMonitor(progressMonitor, 1));
					}
					stream.close();
	        } catch (CoreException e) {
	        	BpmnDiagramEditorPlugin.getInstance().logError(
                    "Unable to set charset for diagram file", e); //$NON-NLS-1$
	        }
	        
			} catch (IOException e) {
			}
	        return diagramFile;
	    }

	    
	    private static InputStream openContentStreamForBPMGateway(String bpmGatewayServiceId,String fileName, String poolName) {
			StringBuffer xml = new StringBuffer("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
			xml.append("<spagic:component\n");
			xml.append("\t\txmlns:spagic=\"urn:org:spagic3\"\n");
			xml.append("\t\txmlns=\"urn:org:spagic3\"\n");
			xml.append("\t\tspagic.id=\""+bpmGatewayServiceId+"\"\n");
			xml.append("\t\tfactory.name=\"spagic3.bpmfactory\">\n");

		
			xml.append("\t<property name=\"")
						.append("process")
						.append("\" value=\"")
						.append(poolName)
						.append("\"/>\n");
			
			xml.append("</spagic:component>");
			return new ByteArrayInputStream(xml.toString().getBytes());
		}
		
		private String generateId(String fileName, String factory) {
			Pattern pattern = Pattern.compile("\\..");
		    Matcher matcher = pattern.matcher(fileName);
		    StringBuffer buf = new StringBuffer();
		    boolean found = false;
		    while ((found = matcher.find())) {
		        String replaceStr = matcher.group();
		        replaceStr = replaceStr.substring(1).toUpperCase();
		        matcher.appendReplacement(buf, replaceStr);
		    }
		    matcher.appendTail(buf);
			return buf.toString();
		}

}
