package org.spagic3.service.model;

import java.io.StringReader;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

import javax.xml.XMLConstants;
import javax.xml.namespace.NamespaceContext;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.ISafeRunnable;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.SafeRunner;
import org.spagic3.constants.SpagicConstants;
import org.spagic3.core.servicedefinitions.extensionprovider.IServiceDefinition;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

public class ServiceModelHelper implements IServiceModelHelper {
	
	private static final String ISERVICEDEFINITION_ID = "org.spagic3.extensionpoint.servicedescriptor";

	private static final String CONNECTORS = "/scrappy/connectors";
	private static final String SERVICES = "/scrappy/services";
	private static final String DEFINITIONS = "/scrappy/definitions";
	private static final String COMBOPROVIDERS = "/scrappy/combo-providers";
	private static final String DEPLOYSCHEMAS = "/scrappy/deploy-schemas";
	
	private ComboProviderFactory comboProviderFactory;
	
	private static SpagicNamespaceContext nsContext;
	
    public static class SpagicNamespaceContext implements NamespaceContext
    {
    	private Map<String, String> namespaceMap;
    	
    	public SpagicNamespaceContext() {
			namespaceMap = new HashMap<String, String>();
			namespaceMap.put("spagic", "urn:org:spagic3");
		}

		public String getNamespaceURI(String prefix)
        {
            if (namespaceMap.containsKey(prefix))
                return namespaceMap.get(prefix);
            else
                return XMLConstants.NULL_NS_URI;
        }
        
        public String getPrefix(String namespace)
        {
            if (namespaceMap.containsValue(namespace)) {
            	for (Entry<String, String> e : namespaceMap.entrySet()) {
            		if (e.getValue().equals(namespace))
            			return e.getKey();
            	}
                return null;
            }
            else
                return null;
        }

        public Iterator<String> getPrefixes(String namespace)
        {
            return namespaceMap.keySet().iterator();
        }
    }  
  	
	
	private Document scappyDefDocument;
	
	public ServiceModelHelper() throws Exception {
		URL scrappyDefFile = Activator.getFileURL("/conf/scrappy-def.xml");

		// Parse the XML as a W3C document.
		DocumentBuilderFactory xmlFact = DocumentBuilderFactory.newInstance();
		xmlFact.setNamespaceAware(true);
		DocumentBuilder builder = xmlFact.newDocumentBuilder();
        scappyDefDocument = builder.parse(scrappyDefFile.openStream());
						
		// Load all extensions for service definitions
		comboProviderFactory = new ComboProviderFactory();
		IConfigurationElement[] config = Platform.getExtensionRegistry().getConfigurationElementsFor(ISERVICEDEFINITION_ID);
		for (IConfigurationElement e : config) {
			final Object o = e.createExecutableExtension("class");
			if (o instanceof IServiceDefinition) {
				ISafeRunnable runnable = new ISafeRunnable() {
					
					public void handleException(Throwable exception) {
						exception.printStackTrace();
					}

					public void run() throws Exception {
						IServiceDefinition sd = (IServiceDefinition)o;
						addNodes(getElement(scappyDefDocument, CONNECTORS), sd.getConnectors());
						addNodes(getElement(scappyDefDocument, SERVICES), sd.getServices());
						addNodes(getElement(scappyDefDocument, DEFINITIONS), sd.getDefinitions());
						addNodes(getElement(scappyDefDocument, COMBOPROVIDERS), sd.getComboProviders());
						addNodes(getElement(scappyDefDocument, DEPLOYSCHEMAS), sd.getDeploySchemas());
						
						if (sd.getComboProviderFactoryContribution() != null) {
							comboProviderFactory.addComboProviders(sd.getComboProviderFactoryContribution());
						}
					}
				};
				SafeRunner.run(runnable);
			}
		}
		
	}
	
	public void addNodes(Element el, List<Node> nodes) {
		if (nodes != null) {
			for (Node currentNode : nodes) {
				Node dup = el.getOwnerDocument().importNode(currentNode, true);
				el.appendChild(dup);
			}
		}
	}
	
	public ComboProviderFactory getComboProviderFactory() {
		return comboProviderFactory;
	}

	public Element getElement(Document document, String xpath) {
		try {
			XPath xPath = XPathFactory.newInstance().newXPath();
			xPath.setNamespaceContext(nsContext);
			Node node = (Node) xPath.evaluate(xpath, document,
					XPathConstants.NODE);
			return (Element) node;
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	public Document getScappyDefDocument() {
		return scappyDefDocument;
	}

	public String getComponentName(String factoryName) {
		return evalXPathAsString(scappyDefDocument, 
				"(/scrappy/definitions/def[@factory=\"" + 
				factoryName + 
				"\"]/@name)");
	}
	
	public static String evalXPathAsString(String xml, String xpath) {
		try {
			DocumentBuilderFactory xmlFact = DocumentBuilderFactory
					.newInstance();
			xmlFact.setNamespaceAware(true);
			DocumentBuilder builder = xmlFact.newDocumentBuilder();
			InputSource source = new InputSource(new StringReader(xml));
	        Document document = builder.parse(source);
			return evalXPathAsString(document, xpath);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}
		
	public static String evalXPathAsString(Node node, String xpath) {
		try {
			XPathFactory factory = XPathFactory.newInstance();
			XPath xPath = factory.newXPath();
			xPath.setNamespaceContext(ServiceModelHelper.nsContext);
			String value = (String) xPath.evaluate(xpath, node,
					XPathConstants.STRING);
			return value;
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	public static List<Node> evalXPathAsNodes(String xml, String xpath) {
		try {
	        // Parse the XML as a W3C document.
			DocumentBuilderFactory xmlFact = DocumentBuilderFactory
					.newInstance();
			xmlFact.setNamespaceAware(true);
			DocumentBuilder builder = xmlFact.newDocumentBuilder();
			InputSource source = new InputSource(new StringReader(xml));
	        Document document = builder.parse(source);
			return evalXPathAsNodes(document, xpath);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}
	
	public static List<Node> evalXPathAsNodes(Node node, String xpath) {
		XPath xPath = XPathFactory.newInstance().newXPath();
		xPath.setNamespaceContext(ServiceModelHelper.getNsContext());
		
		NodeList nodes = null;
		try {
			nodes = (NodeList) xPath.evaluate(xpath, node,
					XPathConstants.NODESET);
		} catch (XPathExpressionException e) {
			e.printStackTrace();
			return null;
		}
		List<Node> result = new ArrayList<Node>(nodes.getLength());
		for (int i = 0; i < nodes.getLength(); i++) {
			result.add(nodes.item(i));
		}
		return result;
	}

	public String getComponentType(String factoryName) {
		
		// Search first within connectors
		String componentName = evalXPathAsString(scappyDefDocument, 
				"(/scrappy/connectors/connector[@factory=\"" + 
				factoryName + 
				"\"]/@name)");
		if ((componentName != null) && (componentName.length() > 0)) {
			return SpagicConstants.SERVICE_TYPE_CONNECTOR;
		}

		// Search within services
		componentName = evalXPathAsString(scappyDefDocument, 
				"(/scrappy/services/service[@factory=\"" + 
				factoryName + 
				"\"]/@name)");
		if ((componentName != null) && (componentName.length() > 0)) {
			return SpagicConstants.SERVICE_TYPE_SERVICE;
		}
		
		return SpagicConstants.SERVICE_TYPE_UNDEFINED;
	}

	public static SpagicNamespaceContext getNsContext() {
		return nsContext;
	}
	
	public List<String> getResoursePropertyNames(String factory) {
		List<Node> valueNodes = evalXPathAsNodes(getScappyDefDocument(), 
				"(/scrappy/definitions/def[@factory=\"" + factory + "\"]/property[@resource=\"true\"]/@name)" +
						" | (/scrappy/definitions/def[@factory=\"" + factory + "\"]/when/property[@resource=\"true\"]/@name)");
		List<String> resourcePropertieNames = new ArrayList<String>();
		for (Node valueNode : valueNodes) {
			final String value = ((Attr)valueNode).getValue();
			resourcePropertieNames.add(value);
		}
		return resourcePropertieNames;
	}
	
	private boolean deploySchemasMapsInitialized = false;
	private Map<String, DeploySchemaHelper> deploySchemasMapByName = null;
	private Map<String, DeploySchemaHelper> deploySchemasMapByExtension = null;
	
	private void initDeploySchemasMaps() {
		List<Node> deploySchemaNodes = evalXPathAsNodes(getScappyDefDocument(), "/scrappy/deploy-schemas/deploy-schema");
		deploySchemasMapByName = new HashMap<String, DeploySchemaHelper>();
		deploySchemasMapByExtension = new HashMap<String, DeploySchemaHelper>();
		for (Node deploySchemaNode : deploySchemaNodes) {
			final DeploySchemaHelper helper = new DeploySchemaHelper(deploySchemaNode);
			deploySchemasMapByName.put(helper.getName(), helper);
			for (String extension : helper.getEstensions()) {
				deploySchemasMapByExtension.put(extension, helper);
			}
		}
	}
	
	public DeploySchemaHelper getDeploySchemaByName(String name) {
		if (!deploySchemasMapsInitialized) {
			initDeploySchemasMaps();
		}
		return deploySchemasMapByName.get(name);
	}

	public DeploySchemaHelper getDeploySchemaByExtension(String extension) {
		if (!deploySchemasMapsInitialized) {
			initDeploySchemasMaps();
		}
		return deploySchemasMapByExtension.get(extension);
	}
	
	public class DeploySchemaHelper {
		
		private Node deploySchemaDoc;
		
		public DeploySchemaHelper(Node schema) {
			this.deploySchemaDoc = schema;
		}
		
		public String getName() {
			return evalXPathAsString(deploySchemaDoc, "@name");
		}

		public String getPath() {
			return evalXPathAsString(deploySchemaDoc, "@path");
		}
		
		public Set<String> getEstensions() {
			final List<Node> valueNodes = evalXPathAsNodes(deploySchemaDoc, "extension/@value");
			Set<String> rs = new HashSet<String>();
			for (Node valueNode : valueNodes) {
				rs.add(valueNode.getNodeValue());
			}
			return rs;
		}

	}
	
	static {
		// Instantiate once the namespace context
		nsContext = new SpagicNamespaceContext();		
	}

}
