/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.ui.deploymentservice.wizards;

import java.net.URI;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.resources.IFile;

public class ServiceInfoRepository {
	
	private List<URI> projectServiceURIs;
	private List<URI> projectProcessURIs;
	private Map<String, ServiceInfo> serviceMapById = new HashMap<String, ServiceInfo>();
	private Map<String, List<ServiceInfo>> serviceMapByTarget = new HashMap<String, List<ServiceInfo>>();
	private Map<URI, ServiceInfo> serviceMapByURI = new HashMap<URI, ServiceInfo>();

	public ServiceInfoRepository(List<URI> projectServiceURIs) {
		this.projectServiceURIs = projectServiceURIs;
	}
	
	public List<URI> getProjectServiceURIs() {
		return projectServiceURIs;
	}

	public List<URI> getProjectProcessURIs() {
		if (projectProcessURIs == null) {
			projectProcessURIs = new ArrayList<URI>();
			for (URI service : projectServiceURIs) {
				final String process = getOrCreate(service).getProcess();
				if (process != null && !"".equals(process)) {
					projectProcessURIs.add(service);
				}
			}
		}
		return projectProcessURIs;
	}

	private ServiceInfo getOrCreate(URI uri) {
		ServiceInfo info = serviceMapByURI.get(uri);
		if (info == null) {
			info = new ServiceInfo(uri);
			serviceMapById.put(info.getServiceId(), info);
			final String target = info.getTarget();
			if (target != null && !"".equals(target)) {
				List<ServiceInfo> targeters = serviceMapByTarget.get(target);
				if (targeters == null) {
					targeters = new ArrayList<ServiceInfo>();
					serviceMapByTarget.put(target, targeters);
					targeters.add(info);
				} else {
					if (!targeters.contains(info)) {
						targeters.add(info);
					}
				}
			}
			serviceMapByURI.put(uri, info);
		}
		return info;
	}

	private ServiceInfo getOrFind(String id) {
		ServiceInfo info = serviceMapById.get(id);
		if (info == null) {
			for (URI candidateURI : projectServiceURIs) {
				ServiceInfo candidateInfo = getOrCreate(candidateURI);
				if (id.equals(candidateInfo.getServiceId())) {
					info = candidateInfo;
					break;
				}
			}
			//not found
		}
		return info;
	}

	private List<ServiceInfo> getOrFindByTarget(String target) {
		List<ServiceInfo> info = serviceMapByTarget.get(target);
		if (info == null) {
			for (URI candidateURI : projectServiceURIs) {
				getOrCreate(candidateURI);
			}
			info = serviceMapByTarget.get(target);
			//info == null => not found
		}
		return info;
	}

	public ServiceInfo findServiceInfoById(String id) {
		if (id == null || "".equals(id)) {
			return null;
		}
		return getOrFind(id);
	}

	public List<ServiceInfo> findServiceInfosByTarget(String target) {
		if (target == null || "".equals(target)) {
			return null;
		}
		return getOrFindByTarget(target);
	}

	public ServiceInfo findServiceInfoByURI(URI uri) {
		if (uri == null) {
			return null;
		}
		return getOrCreate(uri);
	}

	public ServiceInfo findServiceInfoByFile(IFile file) {
		return findServiceInfoByURI(file.getLocationURI());
	}

}
