/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.utils.graph;

import java.io.ByteArrayOutputStream;
import java.util.List;

import org.eclipse.stp.im.IterationControl;
import org.eclipse.stp.im.JoinControl;
import org.eclipse.stp.im.RouterControl;
import org.eclipse.stp.im.Service;
import org.eclipse.stp.im.SplitControl;
import org.spagic.graph.util.GraphEndNode;
import org.spagic.graph.util.GraphNode;
import org.spagic.graph.util.GraphStartNode;
import org.spagic.graph.util.JoinNode;
import org.spagic.graph.util.RouterNode;
import org.spagic.graph.util.SplitNode;
import org.spagic3.utils.SpagicLogger;
import org.spagic3.utils.SpagicUtils;

public class GraphGenerator {

	/**
	 * Generates the SVG graph for the given process
	 * @param process Process to render as a graph
	 * @param graphVizExeLocation Location of Graphviz executable, retrieved from preferences
	 * @param graphVizTempFolder Temporary Location for intermediate files, retrieved from preferences
	 * @return SVG graph (XML string)
	
	public static String getGraphForProcess(org.spagic.metadb.model.Process process, String graphVizExeLocation, String graphVizTempFolder) {
		try{
			org.spagic.graph.util.Graph gv = new org.spagic.graph.util.Graph("graph"+System.currentTimeMillis());
			gv.setDotExeLocation(graphVizExeLocation);
			gv.setTempDirLocation(graphVizTempFolder);
			gv.startGraph();
			Set<Step> steps = process.getSteps();
			for (Step step : steps){
				if (step.isIsStart()){
					gv.addNode(new GraphStartNode(String.valueOf(step.getIdStep()), String.valueOf(step.getIdStep())));
				}else if (step.isIsEnd()){
					gv.addNode(new GraphEndNode(String.valueOf(step.getIdStep()), String.valueOf(step.getIdStep())));
				}else{
					String serviceEntityType = getStepPropertyValue(step, ImConstants.SERVICE_ENTITY_PROPERTY_NAME);
					System.out.println(" ServiceEntityType --> " + serviceEntityType);
					if (serviceEntityType.equalsIgnoreCase(ImConstants.SERVICE_SPLIT_CONTROL))
						gv.addNode(new SplitNode(String.valueOf(step.getIdStep()),String.valueOf(step.getIdStep())));
					else if (serviceEntityType.equalsIgnoreCase(ImConstants.SERVICE_JOIN_CONTROL)) 
						gv.addNode(new JoinNode(String.valueOf(step.getIdStep()),String.valueOf(step.getIdStep())));
					else if (serviceEntityType.equalsIgnoreCase(ImConstants.SERVICE_ROUTER_CONTROL)) 
						gv.addNode(new RouterNode(String.valueOf(step.getIdStep()),String.valueOf(step.getIdStep())));
					else if (serviceEntityType.equalsIgnoreCase(ImConstants.SERVICE_ITERATION_CONTROL)) 
						gv.addNode(new GraphNode(String.valueOf(step.getIdStep()),String.valueOf(step.getIdStep())));
					else gv.addNode(new GraphNode(String.valueOf(step.getIdStep()),String.valueOf(step.getIdStep())));
				}
			}
			Set<Transition> transitions = process.getTransitions();
			for (Transition transition : transitions) {
				gv.addFlow(new org.spagic.graph.util.Transition(String.valueOf(transition.getId().getIdStepSource()),
							 String.valueOf(transition.getId().getIdStepTarget())));
			}
			gv.endGraph();
		
			byte[] img = gv.getGraph(gv.getDotSource());
			String svg = null;
		
			ByteArrayOutputStream bos = new ByteArrayOutputStream();
			bos.write(img);
			bos.flush();
			svg = bos.toString();
			bos.close();
		
		
			// Remove the DTD SVG Declartion
			String svgPart = svg.substring(svg.indexOf("<svg"));
			String svgWithoutDTD = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" + svgPart;
			//System.out.println("------------------------------------------------\n");
			//System.out.println(svgWithoutDTD);
			//System.out.println("------------------------------------------------\n");
			return svgWithoutDTD;
		}catch(Exception ex){
			SpagicLogger.error(IM2MetaDBActivator.PLUGIN_ID,"Exception in getting SVG For Process " + process.getName(), ex);
			return null;
		}
	}
	 */
	
	public static String getGraphForProcess(org.eclipse.stp.im.Process process, String graphVizExeLocation, String graphVizTempFolder) {
		final String NODE_LABEL = "###";
		
		try{
			org.spagic.graph.util.Graph gv = new org.spagic.graph.util.Graph("graph"+System.currentTimeMillis());
			gv.setDotExeLocation(graphVizExeLocation);
			gv.setTempDirLocation(graphVizTempFolder);
			gv.startGraph();
			List<org.eclipse.stp.im.Step> steps = process.getSteps();
			
			// Perform 3 different cycles to assign diffent weights to the nodes
			for (org.eclipse.stp.im.Step step : steps){
				if (SpagicUtils.isStartStep(step)){
					// Generate nodes database-independent
					
					String stepName = SpagicUtils.getStepNameInGraph(process, step);
					gv.addNode(new GraphStartNode(dotEncode(stepName), NODE_LABEL));
				}
			}
		
			for (org.eclipse.stp.im.Step step : steps){
				if (!(SpagicUtils.isEndStep(step) || SpagicUtils.isStartStep(step))){
					String stepName = SpagicUtils.getStepNameInGraph(process, step);
					Service stepServiceModel = step.getServiceModel();

					if (stepServiceModel instanceof SplitControl)
						gv.addNode(new SplitNode(dotEncode(stepName),NODE_LABEL));
					else if (stepServiceModel instanceof JoinControl) 
						gv.addNode(new JoinNode(dotEncode(stepName),NODE_LABEL));
					else if (stepServiceModel instanceof RouterControl)
						gv.addNode(new RouterNode(dotEncode(stepName),NODE_LABEL));
					else if (stepServiceModel instanceof IterationControl)
						gv.addNode(new GraphNode(dotEncode(stepName),stepName));
					else gv.addNode(new GraphNode(dotEncode(stepName),NODE_LABEL));
				}
			}
			
			for (org.eclipse.stp.im.Step step : steps) {
				if (SpagicUtils.isEndStep(step)) {
					String stepName = SpagicUtils.getStepNameInGraph(process, step);
					gv.addNode(new GraphEndNode(dotEncode(stepName), NODE_LABEL));
				}
			}
			
			List<org.eclipse.stp.im.Transition> transitions = process.getTransitions();
			for (org.eclipse.stp.im.Transition transition : transitions) {
				String sourceStepName = dotEncode(SpagicUtils.getStepNameInGraph(process, transition.getSource()));
				String targetStepName = dotEncode(SpagicUtils.getStepNameInGraph(process, transition.getTarget()));
				
				/*
				QName sourceStepQName = SpagicUtils.getQNameForStep(process, transition.getSource());
				String sourceStepName = dotEncode(sourceStepQName.getLocalPart());
				QName targetStepQName = SpagicUtils.getQNameForStep(process, transition.getTarget());
				String targetStepName = dotEncode(targetStepQName.getLocalPart());
				*/
				gv.addFlow(new org.spagic.graph.util.Transition(sourceStepName, targetStepName));
			}
			gv.endGraph();
		
			byte[] img = gv.getGraph(gv.getDotSource());
			ByteArrayOutputStream bos = new ByteArrayOutputStream();
			bos.write(img);
			bos.flush();
			String svgTemp = bos.toString();
			bos.close();
		
			// Saint - Fri Mar 12 16:35:31 2010
			// 
			// replace &#160; occurrences with plain blanks...
			StringBuilder svg = new StringBuilder();
			String tgt = "&#160;";
			int tgtLen = tgt.length();

			int index=0;
			for(int where=svgTemp.indexOf(tgt,index); where > -1; where=svgTemp.indexOf(tgt,index)) {
				svg.append(svgTemp.substring(index, where));
				svg.append(" ");
				index = where + tgtLen;
			}
			svg.append(svgTemp.substring(index, svgTemp.length()));
			
			// Remove the DTD SVG Declartion and restore XML header
			return "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" + svg.substring(svg.indexOf("<svg"));
		}catch(Exception ex){
			SpagicLogger.error("Exception in getting SVG For Process " + process.getName(), ex);
			return null;
		}
	}
	
	public static String dotEncode(String id) {
		return "\"" + id + "\"";
	}
	
	
	
	
	
	@SuppressWarnings("unused")
	private static String getStepPropertyValue(org.eclipse.stp.im.Step aStep, String propertyName){
		String value = null;
		org.eclipse.stp.im.Property aProperty = aStep.getProperties().get(propertyName);
		if (aProperty != null) {
			value = (String)aProperty.getValue();
		}
		return value;
	}	

}
