/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.utils.preference;

import org.eclipse.core.runtime.preferences.AbstractPreferenceInitializer;
import org.eclipse.jface.preference.IPreferenceStore;
import org.spagic3.utils.SpagicUtils;
import org.spagic3.utils.SpagicUtilsActivator;

/**
 * Class used to initialize default preference values.
 */
public class SpagicPreferenceInitializer extends AbstractPreferenceInitializer {

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.runtime.preferences.AbstractPreferenceInitializer#initializeDefaultPreferences()
	 */
	public void initializeDefaultPreferences() {
		IPreferenceStore store = SpagicUtilsActivator.getDefault()
				.getPreferenceStore();
		
		// PREFERENCE GENERAL
		store.setDefault(SpagicPreferenceConstants.P_SPAGIC_BASE_NAME_SPACE,"urn:eng:spagic:processes:");
		store.setDefault(SpagicPreferenceConstants.P_SPAGIC_SERVICE_MANAGER_HOST,"localhost");
		
		store.setDefault(SpagicPreferenceConstants.P_GRAPH_VIZ_DOT_EXE, "C:/Programmi/ATT/Graphviz/bin/dot");
		store.setDefault(SpagicPreferenceConstants.P_GRAPH_VIZ_TEMP_FOLDER,"C:/tmp");
		
		store.setDefault(SpagicPreferenceConstants.P_SERVICE_MANAGER_RESOURCE_GET_SERVICE_URL, "http://localhost:9090/getResources/");
		store.setDefault(SpagicPreferenceConstants.P_SERVICE_MANAGER_RESOURCE_DEPLOY_SERVICE_URL, "http://localhost:9090/deployResources/");
		
		
		
		// PREFERENCE PAGE META DB ( H2 )
		store.setDefault(SpagicPreferenceConstants.P_BPEL_SPAGIC_DB_URL, "jdbc:h2:tcp://localhost/~/spagic");
		store.setDefault(SpagicPreferenceConstants.P_BPEL_SPAGIC_DB_DRIVER, "org.h2.Driver");
		store.setDefault(SpagicPreferenceConstants.P_BPEL_SPAGIC_DB_HIB_DIALECT, "org.hibernate.dialect.H2Dialect");
		
		store.setDefault(SpagicPreferenceConstants.P_BPEL_SPAGIC_DB_USER, "spagic");
		store.setDefault(SpagicPreferenceConstants.P_BPEL_SPAGIC_DB_PASSWORD, "spagic");
		store.setDefault(SpagicPreferenceConstants.P_BPEL_SPAGIC_DB_SHOW_SQL, "true");
		
		// Preference PAGE SERVICE REGISTRY
		store.setDefault(SpagicPreferenceConstants.P_SERVICE_REGISTRY_INQUIRY_URL, "http://localhost:8080/juddi/inquiry");
		store.setDefault(SpagicPreferenceConstants.P_SERVICE_REGISTRY_PUBLISH_URL, "http://localhost:8080/juddi/publish");
		store.setDefault(SpagicPreferenceConstants.P_SERVICE_REGISTRY_USER, "eng");
		store.setDefault(SpagicPreferenceConstants.P_SERVICE_REGISTRY_PASSWORD, "eng");
		store.setDefault(SpagicPreferenceConstants.P_SERVICE_REGISTRY_DEFAULT_ORG, "Engineering");
		
		// PREFERENCE PAGE WORKFLOW
		store.setDefault(SpagicPreferenceConstants.P_WF_PROCESS_STATUS_VARIABLE_NAME, "InstanceStatus");
		store.setDefault(SpagicPreferenceConstants.P_WF_CREATE_VARIABLE_FOR_TASK_STATUS, "false");
		store.setDefault(SpagicPreferenceConstants.P_WF_VARIABLE_FOR_TASK_STATUS_PREFIX, "TaskInstanceStatus_");
		store.setDefault(SpagicPreferenceConstants.P_WF_BIND_VARIABLE_FOR_CONDITION, "$ProcessInstanceVariableStatus");
		
		store.setDefault(SpagicPreferenceConstants.P_WF_CREATE_VARIABLE_FOR_SUBPROCESS_STATUS, "false");
		store.setDefault(SpagicPreferenceConstants.P_WF_VARIABLE_FOR_SUBPROCESS_PREFIX, "SubprocessStatus_");

		store.setDefault(SpagicPreferenceConstants.P_WF_NAME_SUFFIX_FOR_DECISION_NODE, "_Decision");
		store.setDefault(SpagicPreferenceConstants.P_WF_NAME_SUFFIX_FOR_FORK_NODE, "_Fork");
	
		
		
//		// PREFERENCE PAGE META DB ( H2 )
//		store.setDefault(SpagicPreferenceConstants.P_WF_DB_URL, "jdbc:h2:tcp://localhost/~/jbpm");
//		store.setDefault(SpagicPreferenceConstants.P_WF_DB_DRIVER, "org.h2.Driver");
//		store.setDefault(SpagicPreferenceConstants.P_WF_DB_HIB_DIALECT, "org.hibernate.dialect.H2Dialect");
//		store.setDefault(SpagicPreferenceConstants.P_WF_DB_USER, "jbpm");
//		store.setDefault(SpagicPreferenceConstants.P_WF_DB_PASSWORD, "jbpm");
		
		
		// Preference BPEL
		store.setDefault(SpagicPreferenceConstants.P_SPAGIC_ODE_URL_DEPLOY, "http://localhost:8080/ode/processes/DeploymentService");
		store.setDefault(SpagicPreferenceConstants.P_SPAGIC_ODE_URL_PROCESS_MANAGEMENT, "http://localhost:8080/ode/processes/ProcessManagement");
		
		// Preference Orbeon, installed in a Tomcat listening on port 8080
		store.setDefault(SpagicPreferenceConstants.P_WF_SPAGIC_ORBEON_FORM_RUNNER_URL, "http://localhost:8080/orbeon/fr");
		store.setDefault(SpagicPreferenceConstants.P_WF_SPAGIC_ORBEON_FORM_REPO_URL, "http://localhost:8080/orbeon/exist/rest/db/");
	}

}
