/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.utils.preference;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.collections.map.HashedMap;
import org.dom4j.Document;
import org.dom4j.Node;
import org.dom4j.io.SAXReader;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.ComboFieldEditor;
import org.eclipse.jface.preference.FieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.spagic3.utils.SpagicUtilsActivator;

/**
 * This class represents a preference page that
 * is contributed to the Preferences dialog. By 
 * subclassing <samp>FieldEditorPreferencePage</samp>, we
 * can use the field support built into JFace that allows
 * us to create a page that is small and knows how to 
 * save, restore and apply itself.
 * <p>
 * This page is used to modify preferences only. They
 * are stored in the preference store that belongs to
 * the main plug-in class. That way, preferences can
 * be accessed directly via the preference store.
 */

public class SpagicWorkflowPreferencePage
	extends FieldEditorPreferencePage
	implements IWorkbenchPreferencePage {

	
	
	
	public SpagicWorkflowPreferencePage() {
		
		super(GRID);
		setPreferenceStore(SpagicUtilsActivator.getDefault().getPreferenceStore());
		setDescription("");
		
		
		
	}
	
	/**
	 * Creates the field editors. Field editors are abstractions of
	 * the common GUI blocks needed to manipulate various types
	 * of preferences. Each field editor knows how to save and
	 * restore itself.
	 */
	public void createFieldEditors() {
		GridLayout layoutPreference = new GridLayout();
		layoutPreference.numColumns = 1;
		
		Composite aComposite = new Composite(getFieldEditorParent(), SWT.NONE);
		aComposite.setLayout(layoutPreference);
		
		
		Group wfGroup = new Group(aComposite,SWT.SHADOW_ETCHED_IN);
		GridData gdwfGroup = new GridData(GridData.FILL_HORIZONTAL);
		wfGroup.setLayoutData(gdwfGroup);
		wfGroup.setText("BPM Gateway Code Generation");
		addField(new BooleanFieldEditor(SpagicPreferenceConstants.P_WF_CREATE_VARIABLE_FOR_PROCESS_STATUS, "Automatic Creation of Process Status Variable Name ", wfGroup));
		addField(new StringFieldEditor(SpagicPreferenceConstants.P_WF_PROCESS_STATUS_VARIABLE_NAME, "Process Status Variable Name ", wfGroup));
		
		addField(new BooleanFieldEditor(SpagicPreferenceConstants.P_WF_CREATE_VARIABLE_FOR_TASK_STATUS, "Automatic Creation of Status Variable for Task  ", wfGroup));
		addField(new StringFieldEditor(SpagicPreferenceConstants.P_WF_VARIABLE_FOR_TASK_STATUS_PREFIX, "Task Status Variable Prefix ", wfGroup));
		
		addField(new BooleanFieldEditor(SpagicPreferenceConstants.P_WF_CREATE_VARIABLE_FOR_SUBPROCESS_STATUS, "Automatic Creation of Status Variable for SubProcess  ", wfGroup));
		addField(new StringFieldEditor(SpagicPreferenceConstants.P_WF_VARIABLE_FOR_SUBPROCESS_PREFIX, "SubProcess Status Variable Prefix ", wfGroup));
		
		addField(new StringFieldEditor(SpagicPreferenceConstants.P_WF_BIND_VARIABLE_FOR_CONDITION, "Name of the Variable In Condition ( If not specified ) ", wfGroup));
	
		
	
		addField(new StringFieldEditor(SpagicPreferenceConstants.P_WF_NAME_SUFFIX_FOR_DECISION_NODE, "Suffix For Decision Node ", wfGroup));
		addField(new StringFieldEditor(SpagicPreferenceConstants.P_WF_NAME_SUFFIX_FOR_FORK_NODE, "Suffix For Fork Node ", wfGroup));
		
		
		
	}

	/** (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
		
	}


	
	@Override
	protected void contributeButtons(Composite parent) {
		super.contributeButtons(parent);
		
		
	}
}