package org.spagic3.utils.wsdl;

import java.io.File;
import java.io.FileWriter;
import java.io.Writer;
import java.net.URI;
import java.util.Iterator;
import java.util.Map;

import javax.wsdl.Binding;
import javax.wsdl.BindingOperation;
import javax.wsdl.Definition;
import javax.wsdl.Message;
import javax.wsdl.Operation;
import javax.wsdl.Port;
import javax.wsdl.PortType;
import javax.wsdl.Service;
import javax.wsdl.Types;
import javax.wsdl.WSDLException;
import javax.wsdl.extensions.ExtensibilityElement;
import javax.wsdl.extensions.soap.SOAPAddress;
import javax.wsdl.extensions.soap12.SOAP12Address;
import javax.wsdl.factory.WSDLFactory;
import javax.wsdl.xml.WSDLReader;
import javax.wsdl.xml.WSDLWriter;
import javax.xml.namespace.QName;

import org.apache.servicemix.common.tools.wsdl.SchemaCollection;
import org.apache.servicemix.common.tools.wsdl.WSDLFlattener;

import com.ibm.wsdl.Constants;
import com.ibm.wsdl.extensions.schema.SchemaImpl;

public class WSDLUtils {
	
	public static final String IN_ONLY_STYLE = "IN_ONLY_STYLE";
	public static final String IN_OUT_STYLE = "IN_OUT_STYLE";
	
	public static Definition readWSDLDefinition(URI wsdlResourceURI) throws WSDLException  {
//		System.out.println(wsdlResourceURI.toString());
		WSDLFactory wsdlFactory = WSDLFactory.newInstance();
		WSDLReader wsdlReader = wsdlFactory.newWSDLReader();

		wsdlReader.setFeature(Constants.FEATURE_VERBOSE, false);
		//wsdlReader.setFeature(Constants.FEATURE_IMPORT_DOCUMENTS, false);
		
		return wsdlReader.readWSDL(wsdlResourceURI.toString());
	}
	
	
	
	public static Definition flatDefinition(Definition def) throws WSDLException {
		try{
			WSDLFlattener wsdlFlattener = new WSDLFlattener(def);
			wsdlFlattener.initialize();
			SchemaCollection schemas = wsdlFlattener.getSchemas();
			if (schemas.getSize() > 0) {
				Types types = def.createTypes();
				for (Iterator it = schemas.getSchemas().iterator(); it.hasNext();) {
					javax.wsdl.extensions.schema.Schema imp = new SchemaImpl();
					imp.setElement(((org.apache.servicemix.common.tools.wsdl.Schema) it.next()).getRoot());
					imp.setElementType(new QName("http://www.w3.org/2001/XMLSchema", "schema"));
					types.addExtensibilityElement(imp);
				}
				def.setTypes(types);
			}
			return def;
		}catch (Exception e) {
			throw new WSDLException(e.getMessage(), e.getMessage(), e);
		}
	}
	
	public static void writeWSDL(Definition def, File wsdlFile) throws WSDLException  {
		try {
			writeWSDL(def, new FileWriter(wsdlFile));
		}catch (Exception e) {
			throw new WSDLException(e.getMessage(), e.getMessage(), e);
		}
	}
	
	public static void writeWSDL(Definition def, Writer writer) throws WSDLException  {
		try{
			WSDLFactory wsdlFactory = WSDLFactory.newInstance();
			WSDLWriter wsdlWriter = wsdlFactory.newWSDLWriter();
			wsdlWriter.writeWSDL(def, writer);
		}catch (Exception e) {
			throw new WSDLException(e.getMessage(), e.getMessage(), e);
		}
	}
	
	public static Definition getJBICompliantDefinition(Definition def, 
													   String processNsPrefix,
													   String processNsURI,
													   String portTypeName, 
													   String serviceName, 
													   String serviceURL,
													   String style) throws WSDLException {
		 
		 WSDLFactory wsdlFactory = WSDLFactory.newInstance();
		 Definition newDef = wsdlFactory.newDefinition(); 
		
		 newDef.addNamespace(processNsPrefix, processNsURI);
		 newDef.setTargetNamespace(processNsURI);
		 copyNamespaces(def, newDef);
		 newDef.setTypes(def.getTypes());
		 copyMessages(def, newDef);
		 
		 // Se cambio il nome della portType devo avere il mapping
		 copyPortType(def, newDef, portTypeName, style);
		 copyBindings(def, newDef, portTypeName, style);
		 copyServices(def, newDef, portTypeName, serviceName, serviceURL);
		 return newDef;
	}
	
	// -----
	
	public static void copyNamespaces(Definition oldDef, Definition newDef){
		Map<String, String> namespacesMap = oldDef.getNamespaces();
		for ( String nsPrefix : namespacesMap.keySet()){
			newDef.addNamespace(nsPrefix, (String)namespacesMap.get(nsPrefix));
		}
	}
	
	public static void copyMessages(Definition oldDef, Definition newDef){
		Map<QName, Message> messages = oldDef.getMessages();
		for ( QName qName : messages.keySet()){
			Message msg = messages.get(qName);
			QName newQName = new QName(newDef.getTargetNamespace(), qName.getLocalPart());
			msg.setQName(newQName);
			newDef.addMessage(msg);
		}
	}
	
	public static void copyPortType(Definition oldDef, Definition newDef, String portTypeName, String style){
		
		Map<QName, PortType> portTypes = oldDef.getPortTypes();
		for ( QName qName : portTypes.keySet()){
			PortType pt = portTypes.get(qName);
			
			QName newQName = new QName(newDef.getTargetNamespace(), portTypeName);
			pt.setQName(newQName);
			
			
			for (Iterator it = pt.getOperations().iterator(); it.hasNext();){
				Operation op = (Operation)it.next();
				if (style.equalsIgnoreCase(IN_ONLY_STYLE)){
					op.setOutput(null);
				}
				
				if ( op.getInput() != null && op.getInput().getMessage() != null){
					op.getInput().setMessage(getMessageInNewDef(op.getInput().getMessage(), newDef));
				}
				if (style.equalsIgnoreCase(IN_OUT_STYLE)){
					if ( op.getOutput() != null && op.getOutput().getMessage() != null){
						op.getOutput().setMessage(getMessageInNewDef(op.getOutput().getMessage(), newDef));
					}
				}
				
			}
			
			newDef.addPortType(pt);
			
		}
		
	}
	
	public static void copyBindings(Definition oldDef, Definition newDef, String portTypeName, String style){

		Map<QName, Binding> bindings = oldDef.getBindings();
		for ( QName qName : bindings.keySet()){
			Binding bin = bindings.get(qName);
			QName newQName = new QName(newDef.getTargetNamespace(), qName.getLocalPart());
			bin.setQName(newQName);
			for (Iterator it = bin.getBindingOperations().iterator(); it.hasNext();){
				BindingOperation op = (BindingOperation)it.next();
				
				if (style.equalsIgnoreCase(IN_ONLY_STYLE)){
					op.setBindingOutput(null);
				}
			}
			//bin.setPortType( getPortTypeInNewDef(bin.getPortType(), newDef, portTypeNamesMapping));
			newDef.addBinding(bin);

		}
	}
	
	public static void copyServices(Definition oldDef, 
									Definition newDef, 
									String portName,
									String serviceName, 
									String serviceURL){
		
		QName soap11AddreddQName = new QName("http://schemas.xmlsoap.org/wsdl/soap/", "address");
		QName soap12AddreddQName = new QName("http://schemas.xmlsoap.org/wsdl/soap12/", "address");
		
		Map<QName, Service> services = oldDef.getServices();
		for ( QName qName : services.keySet()){
			Service ser = services.get(qName);
			QName newQName = new QName(newDef.getTargetNamespace(), serviceName);
			ser.setQName(newQName);
			Map<String, Port> ports = ser.getPorts();
			for (String o : ports.keySet()){
				Port pt = ser.getPort(o);
				pt.setBinding(getBindingInNewDef(pt.getBinding(), newDef));
				pt.getExtensibilityElements();
				
				pt.setName(portName);
				for (Object objExtEl : pt.getExtensibilityElements()){	
					ExtensibilityElement extEL = (ExtensibilityElement)objExtEl;
					
					if (extEL.getElementType().equals(soap11AddreddQName)){
						SOAPAddress soapAddress = (SOAPAddress) extEL;
						soapAddress.setLocationURI(serviceURL);
					}
					if (extEL.getElementType().equals(soap12AddreddQName)){
						SOAP12Address soapAddress = (SOAP12Address) extEL;
						soapAddress.setLocationURI(serviceURL);
					}
				}
			}
			newDef.addService(ser);
		}
	}
	
	public static Message getMessageInNewDef(Message oldMessage, Definition newDef){
		QName newQName = new QName(newDef.getTargetNamespace(), oldMessage.getQName().getLocalPart());
		return newDef.getMessage(newQName);
	}
	
	/*
	public static PortType getPortTypeInNewDef(PortType pt, Definition newDef, Map<String, String> portTypesNamesmapping){
		String nameInNewdef = portTypesNamesmapping.get(pt.getQName().getLocalPart());
		QName newQName = new QName(newDef.getTargetNamespace(), nameInNewdef);
		return newDef.getPortType(newQName);
	}
	*/
	public static Binding getBindingInNewDef(Binding binding, Definition newDef){
		QName newQName = new QName(newDef.getTargetNamespace(), binding.getQName().getLocalPart());
		return newDef.getBinding(newQName);
	}
	
}
